 /*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

 

#ifdef XFree86Server
#include "xf86_ansic.h" /* for Bool */
#else
#include "via_types.h"  /* for Bool */
#endif

#include "via.h"

#include "VidStruc.h"
#include "regrec.h"
#include "Vinline.h"    /* Universal wait function */
#include "debug.h"

extern unsigned long OverlayToUpdate;
static unsigned int VidRegInfo[256]; 
static VIDEOREGISTER VidReg[VIDEO_REG_NUM];
static VIDEOREGISTER VidRegV1[VIDEO_REG_NUM];
static VIDEOREGISTER VidRegV3[VIDEO_REG_NUM];
static unsigned long gdwVidRegCounter;
static unsigned long gdwVidRegCounterV1;
static unsigned long gdwVidRegCounterV3;


/* I N L I N E -------------------------------------------------------------- */
__inline void Macro_VidREGFlush_noWait(unsigned char* pjVideo)
{
    unsigned long i;
    
    for (i=0; i< gdwVidRegCounter; i++ )
    {
        VIDOutD(VidReg[i].dwIndex, VidReg[i].dwData);
        DBG_DD(("Macro_VidREGFlush:%08lx %08lx\n",VidReg[i].dwIndex,VidReg[i].dwData));
    }
}

__inline void Macro_VidREGFlush(unsigned char* pjVideo)
{
    unsigned long i;
    /* WaitVideoCommandFire(pjVideo); */
    UniversalWait(pjVideo, V_COMPOSE_MODE, V1_COMMAND_FIRE, 0);
    if(OverlayToUpdate==(VIDEO_1|VIDEO_3) || OverlayToUpdate==0) {
        for (i=0; i< gdwVidRegCounter; i++ )
        {
            VIDOutD(VidReg[i].dwIndex, VidReg[i].dwData);
            DBG_DD(("Macro_VidREGFlush:%08lx %08lx\n",VidReg[i].dwIndex,VidReg[i].dwData));
        }
    }
    else if(OverlayToUpdate==(VIDEO_3)) {
        for (i=0; i< gdwVidRegCounterV3; i++ )
        {
            VIDOutD(VidRegV3[i].dwIndex, VidRegV3[i].dwData);
        }
    }
    else if(OverlayToUpdate==(VIDEO_1)) {
        for (i=0; i< gdwVidRegCounterV1; i++ )
        {
            VIDOutD(VidRegV1[i].dwIndex, VidRegV1[i].dwData);
        }
    }
}

__inline void Macro_VidREGFlushV3(unsigned char* pjVideo)
/* void Macro_VidREGFlushVPE(LPVIATHKINFO sData) */
{
    unsigned long i;

    /* WaitVideo3CommandFire(pjVideo); */
    UniversalWait(pjVideo, V_COMPOSE_MODE, V3_COMMAND_FIRE, 0);

    for (i=0; i< gdwVidRegCounter; i++ )
    {
     	VIDOutD(VidReg[i].dwIndex, VidReg[i].dwData);
        DBG_DD(("Macro_VidREGFlush V3:%08lx %08lx\n",VidReg[i].dwIndex,VidReg[i].dwData));
    }
}

__inline void Macro_VidREGRec(unsigned long dwAction, unsigned long dwIndex, unsigned long dwData)
{
  /*DBG_DD(("seanzhang: Macro_VidREGRec,dwAction=%p,dwIndex=%p\n",dwAction,dwIndex));*/
  switch (dwAction)
  {
     case VIDREGREC_RESET_COUNTER :
          gdwVidRegCounter = 0;
          gdwVidRegCounterV1 = 0;
          gdwVidRegCounterV3 = 0;
          break;

     case VIDREGREC_SAVE_REGISTER:
          VidReg[gdwVidRegCounter].dwIndex = dwIndex&0x0FFFF;
          VidReg[gdwVidRegCounter].dwData  = dwData;
          gdwVidRegCounter++;
          if ( gdwVidRegCounter > VIDEO_REG_NUM){
             DBG_DD(("Macro_VidREGRec:Out of Video register space"));
          }
          if(dwIndex == (V3_STARTADDR_0|0x80000000)) {
              dwIndex = V3_STARTADDR_0;
              VidRegV3[gdwVidRegCounterV3].dwIndex = dwIndex;
              VidRegV3[gdwVidRegCounterV3].dwData  = dwData;
              gdwVidRegCounterV3++;
          }
          else if(VidRegInfo[dwIndex-(V_OFFSET)]==VIDEO_3) {
              VidRegV3[gdwVidRegCounterV3].dwIndex = dwIndex;
              VidRegV3[gdwVidRegCounterV3].dwData  = dwData;
              gdwVidRegCounterV3++;
          }
          else {
              VidRegV1[gdwVidRegCounterV1].dwIndex = dwIndex;
              VidRegV1[gdwVidRegCounterV1].dwData  = dwData;
              gdwVidRegCounterV1++;
          }
          break;

     default :
          DBG_DD(("Macro_VidREGRec:Unknow action"));
          break;
  }
}
void  vfInitVidREGRec_CLE(void) {
/* V_OFFSET                0x200      */
VidRegInfo[ V_FLAGS                 -(V_OFFSET) ] =0;   /* 0x200 */
VidRegInfo[ V_CAP_STATUS            -(V_OFFSET) ] =0;   /* 0x204 */
VidRegInfo[ V_FLIP_STATUS           -(V_OFFSET) ] =0;   /* 0x204 */
VidRegInfo[ V_ALPHA_WIN_START       -(V_OFFSET) ] =0;   /* 0x208 */
VidRegInfo[ V_Vertical_Count        -(V_OFFSET) ] =0;   /* 0x208 */
VidRegInfo[ V_ALPHA_WIN_END         -(V_OFFSET) ] =0;   /* 0x20C */
VidRegInfo[ V_ALPHA_CONTROL         -(V_OFFSET) ] =0;   /* 0x210 */
VidRegInfo[ V_CRT_STARTADDR         -(V_OFFSET) ] =0;   /* 0x214 */
VidRegInfo[ V_CRT_STARTADDR_2       -(V_OFFSET) ] =0;   /* 0x218 */
VidRegInfo[ V_ALPHA_STRIDE          -(V_OFFSET) ] =0;   /* 0x21C */
VidRegInfo[ V_COLOR_KEY             -(V_OFFSET) ] =0;   /* 0x220 */
VidRegInfo[ V_ALPHA_STARTADDR       -(V_OFFSET) ] =0;   /* 0x224 */
VidRegInfo[ V_CHROMAKEY_LOW         -(V_OFFSET) ] =0;   /* 0x228 */
VidRegInfo[ V_CHROMAKEY_HIGH        -(V_OFFSET) ] =0;   /* 0x22C */
VidRegInfo[ V1_CONTROL              -(V_OFFSET) ] =0;   /* 0x230 */
VidRegInfo[ V12_QWORD_PER_LINE      -(V_OFFSET) ] =0;   /* 0x234 */
VidRegInfo[ V1_STARTADDR1           -(V_OFFSET) ] =0;   /* 0x238 */
VidRegInfo[ V1_STRIDE               -(V_OFFSET) ] =0;   /* 0x23C */
VidRegInfo[ V1_WIN_START_Y          -(V_OFFSET) ] =0;   /* 0x240 */
VidRegInfo[ V1_WIN_START_X          -(V_OFFSET) ] =0;   /* 0x242 */
VidRegInfo[ V1_WIN_END_Y            -(V_OFFSET) ] =0;   /* 0x244 */
VidRegInfo[ V1_WIN_END_X            -(V_OFFSET) ] =0;   /* 0x246 */
VidRegInfo[ V1_STARTADDR2           -(V_OFFSET) ] =0;   /* 0x248 */
VidRegInfo[ V1_ZOOM_CONTROL         -(V_OFFSET) ] =0;   /* 0x24C */
VidRegInfo[ V1_MINI_CONTROL         -(V_OFFSET) ] =0;   /* 0x250 */
VidRegInfo[ V1_STARTADDR0           -(V_OFFSET) ] =0;   /* 0x254 */
VidRegInfo[ V_FIFO_CONTROL          -(V_OFFSET) ] =0;   /* 0x258 */
VidRegInfo[ V1_STARTADDR3           -(V_OFFSET) ] =0;   /* 0x25C */
VidRegInfo[ SND_COLOR_KEY           -(V_OFFSET) ] =0;   /* 0x264 */
VidRegInfo[ ALPHA_V3_PREFIFO_CONTROL-(V_OFFSET) ] =0;   /* 0x268 */
VidRegInfo[ V1_SOURCE_HEIGHT        -(V_OFFSET) ] =0;   /* 0x26C */
VidRegInfo[ V_DISPLAY_TEMP          -(V_OFFSET) ] =0;   /* 0x274 No Use */
VidRegInfo[ ALPHA_V3_FIFO_CONTROL   -(V_OFFSET) ] =0;   /* 0x278 */
VidRegInfo[ V3_SOURCE_WIDTH         -(V_OFFSET) ] =VIDEO_3;   /* 0x27C */
VidRegInfo[ V3_COLOR_KEY            -(V_OFFSET) ] =VIDEO_3;   /* 0x280 */
VidRegInfo[ ColorSpaceReg_1         -(V_OFFSET) ] =0;   /* 0x284 */
VidRegInfo[ ColorSpaceReg_2         -(V_OFFSET) ] =0;   /* 0x288 */
VidRegInfo[ V1_STARTADDR0_U         -(V_OFFSET) ] =0;   /* 0x28C */
VidRegInfo[ V1_OPAQUE_CONTROL       -(V_OFFSET) ] =0;   /* 0x290 To be del */
VidRegInfo[ V3_OPAQUE_CONTROL       -(V_OFFSET) ] =VIDEO_3;/* 0x294 To be del */
VidRegInfo[ V_COMPOSE_MODE          -(V_OFFSET) ] =0;   /* 0x298 */
VidRegInfo[ V3_STARTADDR_2          -(V_OFFSET) ] =VIDEO_3;   /* 0x29C */
VidRegInfo[ V3_CONTROL              -(V_OFFSET) ] =VIDEO_3;   /* 0x2A0 */
VidRegInfo[ V3_STARTADDR_0          -(V_OFFSET) ] =VIDEO_3;   /* 0x2A4 */
VidRegInfo[ V3_STARTADDR_1          -(V_OFFSET) ] =VIDEO_3;   /* 0x2A8 */
VidRegInfo[ V3_STRIDE               -(V_OFFSET) ] =VIDEO_3;   /* 0x2AC */
VidRegInfo[ V3_WIN_START_Y          -(V_OFFSET) ] =VIDEO_3;   /* 0x2B0 */
VidRegInfo[ V3_WIN_START_X          -(V_OFFSET) ] =VIDEO_3;   /* 0x2B2 */
VidRegInfo[ V3_WIN_END_Y            -(V_OFFSET) ] =VIDEO_3;   /* 0x2B4 */
VidRegInfo[ V3_MINI_CONTROL         -(V_OFFSET) ] =VIDEO_3;   /* 0x2C0 */
VidRegInfo[ V3_ZOOM_CONTROL         -(V_OFFSET) ] =VIDEO_3;   /* 0x2BC */
VidRegInfo[ V3_ALPHA_FETCH_COUNT    -(V_OFFSET) ] =VIDEO_3;   /* 0x2B8 */
return; 
}

