 /*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 

/* I N C L U D E S ------------------------------------------------------------ */
#ifdef XFree86Server
#include "xf86_ansic.h" /* for Bool */
#else
#include "via_types.h"  /* for Bool */
#endif


#include "via.h"        /* for H/W register definition */
#include "Vinline.h"
#include "viampg.h"
#include "VidInit.h"
#include "VidStruc.h"   /* VIDDATA, video top level structure */
#include "ovlfCME.h"
#include "vregCME.h"
#include "vdefCME.h"

#include "debug.h"

/* DEFINE */


/* FUNCTION  DECLARATION */

/* EXTERN   GLOBALS */

/* EXTERN   FUNCTIONS */

/* GLOBALS */

/* FUNCTION */

/*
 * In:      dwCtlHQVflag: the HQV setting flag         
 * Out:     lpdwVidCtl:The calculated flags that should be written in video 
 *          engine registers
 * Ret:
 * Desc:    if hqv color space conversion flag is set, compute the video 
 *          format settings 
 */

void vfSetVideoFormatWithHQVCSC_CME(unsigned long dwCtlHQVflag,unsigned long* lpdwVidCtl)
{
    if (dwCtlHQVflag&CTL_HQV_CSC_ENABLE_CME)
    {	            
        if (dwCtlHQVflag&CTL_HQV_CSC_RGB32_CME)
        {        
            *lpdwVidCtl |= (VAL_VIDEO_FORMAT_RGB32_CME | VAL_VIDEO_SWAP_HW_HQV_CME );
        }
        else
        {
            *lpdwVidCtl |= (VAL_VIDEO_FORMAT_RGB16_CME | VAL_VIDEO_SWAP_HW_HQV_CME );
        }
    }
    else
    {
        *lpdwVidCtl |= (VAL_VIDEO_FORMAT_YUV422_CME | VAL_VIDEO_SWAP_HW_HQV_CME );
    }
}

/*
 * In:      pData:    video format data structure pointer
 * Out:     
 * Ret:
 * Desc:   fill the video format data structure according to FOURCC.  
 *         if HQV engine is being used, set its relevent advanced process flags
 */

void vfGetVideoFormat_CME(PVIDGETFORMAT pData)
{
    unsigned long dwVideoFlag           = pData->dwVideoFlag;
    unsigned long dwCtlHQVflag          = pData->dwCtlHQVflag;
    LPDDPIXELFORMAT lpDPFsrc    = pData->lpDPFsrc;
    unsigned long dwDisplayPictStruct   = pData->dwDisplayPictStruct;   /* For MPEG only */
    unsigned long dwDeinterlaceMode     = pData->dwDeinterlaceMode;     /* For MPEG only */
    unsigned long dwMPEGProgressiveMode = pData->dwMPEGProgressiveMode; /* For MPEG only */
    unsigned long dwMPEGInTVFullScreen  = pData->dwMPEGInTVFullScreen;  /* For MPEG only */
    unsigned long dwVDevFlag            = pData->dwVDevFlag;
    unsigned long* lpdwVidCtl          = pData->lpdwVidCtl;
    unsigned long* lpdwHQVCtl          = pData->lpdwHQVCtl;

    if (lpDPFsrc->dwFlags & DDPF_FOURCC)
    {
        *lpdwVidCtl |= VAL_VIDEO_COLORSPACE_SIGN_CME;
        switch (lpDPFsrc->dwFourCC) 
        {
        case FOURCC_YUY2:
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                vfSetVideoFormatWithHQVCSC_CME(dwCtlHQVflag,lpdwVidCtl);
                *lpdwHQVCtl |= HQV_SRC_SW|HQV_YUV422|HQV_ENABLE|HQV_TRIPLE_BUFF;
            }
            else
            {
                *lpdwVidCtl |= VAL_VIDEO_FORMAT_YUV422_CME;
            }
            break;

           case FOURCC_NV12:

            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                vfSetVideoFormatWithHQVCSC_CME(dwCtlHQVflag,lpdwVidCtl);
                *lpdwHQVCtl |= HQV_SRC_SW|HQV_YUV420|HQV_ENABLE|HQV_SW_FLIP |HQV_TRIPLE_BUFF;
            }
            else
            {
                *lpdwVidCtl |= VAL_VIDEO_FORMAT_YUV422_CME;
            }
         break;
        default :
            DBG_DD(("vfGetVideoFormat_CME : Invalid FOURCC format :(%lx)!\n",
                  (unsigned long) lpDPFsrc->dwFourCC));
            break;
        }
    }
    else if (lpDPFsrc->dwFlags & DDPF_RGB) 
    {
        switch (lpDPFsrc->dwRGBBitCount) 
        {
        case 16:
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                *lpdwVidCtl |= VAL_VIDEO_FORMAT_RGB16_CME | VAL_VIDEO_SWAP_HW_HQV_CME ;
                *lpdwHQVCtl |= HQV_SRC_SW|HQV_ENABLE|HQV_TRIPLE_BUFF;
                *lpdwHQVCtl |= (lpDPFsrc->dwGBitMask==0x07E0 ?
                            HQV_RGB16 : HQV_RGB15);
            }
            else
            {
                *lpdwVidCtl |= (lpDPFsrc->dwGBitMask==0x07E0 ?
                                VAL_VIDEO_FORMAT_RGB16_CME : VAL_VIDEO_FORMAT_RGB15_CME);
            }
            break;
        case 32:
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                *lpdwVidCtl |= VAL_VIDEO_FORMAT_RGB32_CME | VAL_VIDEO_SWAP_HW_HQV_CME ;
                *lpdwHQVCtl |= HQV_SRC_SW|HQV_RGB32|HQV_ENABLE|HQV_TRIPLE_BUFF;
            }
            else
            {
                *lpdwVidCtl |= VAL_VIDEO_FORMAT_RGB32_CME;
            }
            break;

        default :
            DBG_DD(("vfGetVideoFormat_CME : invalid RGB format %ld bits\n",
                  lpDPFsrc->dwRGBBitCount));
            break;
        }
    }
   
    if (dwVDevFlag&CTL_UseIGA2Path)
    {
        if ((dwVideoFlag & VIDEO_1_INUSE)||(dwVDevFlag & CTL_UseDuoView))
        {
            /* In SAMM or Duo, we display video on second path */
            *lpdwVidCtl |= V1_ON_SND_DISPLAY;
        }
    }         

    if (dwVideoFlag&VIDEO_YUV_BYPASS)
    {
        *lpdwVidCtl &= ~VAL_VIDEO_COLORSPACE_SIGN_CME;      
        *lpdwVidCtl |= VAL_VIDEO_COLORSPACE_DISABLE_CME;  
    }         
    else {
        *lpdwVidCtl  |= VAL_VIDEO_COLORSPACE_SIGN_CME;      
        *lpdwVidCtl  &= ~VAL_VIDEO_COLORSPACE_DISABLE_CME;  
    }
}

/*
 * In:      rSrc:source overlay coordinates,
 *          rDest:destination overlay coordinates,
 *          dwSrcPitch: source pitch, 
 *          dwCtlHQVflag:HQV control flags
 * Out:     lpdwOffset:HQV's source data offset
 * Ret:
 * Desc:   calculate the starting address offset of video source data.
 *         That is video memory physical address 
 */
void vfCalHQVSrcOffset(OV_RECTL rSrc,OV_RECTL rDest,unsigned long dwSrcPitch,\
        unsigned long* lpdwOffset,unsigned long dwCtlHQVflag)
{    
    unsigned long dwHQVsrcWidth=0,dwHQVdstWidth=0;
    unsigned long dwHQVsrcHeight=0,dwHQVdstHeight=0;
    unsigned long dwAlign = ~3;
    unsigned long dwHQVSrcYOffset=0,dwHQVSrcUVOffset=0;

    dwHQVsrcWidth = (unsigned long)(rSrc.right - rSrc.left);
    dwHQVdstWidth = (unsigned long)(rDest.right - rDest.left);
    dwHQVsrcHeight = (unsigned long)(rSrc.bottom - rSrc.top);
    dwHQVdstHeight = (unsigned long)(rDest.bottom - rDest.top);
    
    if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
    {
        if( (dwHQVsrcWidth<dwHQVdstWidth)||(dwHQVsrcHeight<dwHQVdstHeight))
        {
            /* Change HQV Src offset,Set Video Src offset to zero */
            if (dwHQVsrcWidth<dwHQVdstWidth)
            {
                if (dwHQVsrcHeight<dwHQVdstHeight)
                {
                    /* Both Width and Height zoom in */
                    dwHQVSrcYOffset = ((rSrc.top&~1) * dwSrcPitch) + 
                                        (rSrc.left &~0xf);
                    dwHQVSrcUVOffset = (((rSrc.top&~1)>>1) * dwSrcPitch) + 
                                        (rSrc.left &~0xf);
                    *lpdwOffset = 0 ;       
                }
                else
                {
                    /* Only Width zoom in */
                    dwHQVSrcYOffset = (rSrc.left &~0xf);
                    dwHQVSrcUVOffset = (rSrc.left &~0xf);
                    if (dwHQVsrcHeight>dwHQVdstHeight)
                    {
                        /* Height zoom out */
                        *lpdwOffset = ((rSrc.top* dwHQVdstHeight / dwHQVsrcHeight) * (dwSrcPitch<<1));                
                    }
                    else
                    {
                        *lpdwOffset = (rSrc.top * (dwSrcPitch<<1)); ;
                    }                
                }                             
            }
            else
            {
                /* Only Height zoom in */
                dwHQVSrcYOffset = ((rSrc.top&~1) * dwSrcPitch);
                dwHQVSrcUVOffset = (((rSrc.top&~1)>>1) * dwSrcPitch);
    
                if (dwHQVsrcWidth>dwHQVdstWidth)
                {
                    /* Width zoom out */
                    *lpdwOffset =  (rSrc.left << 1)* dwHQVdstWidth / dwHQVsrcWidth;
                }
                else
                {
                    *lpdwOffset = (rSrc.left << 1);       
                }                        
            }    
        }
        else
        {
            if (dwHQVsrcWidth>dwHQVdstWidth)
            {
                if (dwHQVsrcHeight>dwHQVdstHeight)
                {
                    *lpdwOffset = (((rSrc.top* dwHQVdstHeight / dwHQVsrcHeight) * (dwSrcPitch<<1)) + 
                                (((rSrc.left << 1)* dwHQVdstWidth / dwHQVsrcWidth)&dwAlign));
                }
                else
                {
                    *lpdwOffset = ((rSrc.top * (dwSrcPitch<<1)) + 
                                (((rSrc.left << 1)* dwHQVdstWidth / dwHQVsrcWidth)&dwAlign));
                }                             
            }
            else
            {
                if (dwHQVsrcHeight>dwHQVdstHeight)
                {
                    *lpdwOffset = (((rSrc.top* dwHQVdstHeight / dwHQVsrcHeight) * (dwSrcPitch<<1)) + 
                                ((rSrc.left << 1)&dwAlign));
                }
                else
                {
                    *lpdwOffset = ((rSrc.top * (dwSrcPitch<<1)) + ((rSrc.left << 1)&dwAlign));
                }                        
            }    
        }
    }
    else
    {
        if (dwHQVsrcWidth>dwHQVdstWidth)
        {
            if (dwHQVsrcHeight>dwHQVdstHeight)
            {
                *lpdwOffset = (((rSrc.top* dwHQVdstHeight / dwHQVsrcHeight) * (dwSrcPitch<<1)) + 
                            (((rSrc.left << 1)* dwHQVdstWidth / dwHQVsrcWidth)&dwAlign));
            }
            else
            {
                *lpdwOffset = ((rSrc.top * (dwSrcPitch<<1)) + 
                            (((rSrc.left << 1)* dwHQVdstWidth / dwHQVsrcWidth)&dwAlign));
            }                             
        }
        else
        {
            if (dwHQVsrcHeight>dwHQVdstHeight)
            {
                *lpdwOffset = (((rSrc.top* dwHQVdstHeight / dwHQVsrcHeight) * (dwSrcPitch<<1)) + 
                            ((rSrc.left << 1)&dwAlign));
            }
            else
            {
                *lpdwOffset = ((rSrc.top * (dwSrcPitch<<1)) + ((rSrc.left << 1)&dwAlign));
            }                        
        }        
    }
}

/*
 * In/Out:  pData video source data structure pointer
 * Ret:
 * Desc:   calculate the starting address offset of video source data.
 *         The HQV alignment should be considered.
 */
void vfGetSrcStartAddress_CME(PVIDGETSRCADDR pData)
{
    unsigned long dwVideoFlag   = pData->dwVideoFlag;
    unsigned long dwSrcPitch    = pData->dwSrcPitch;
    OV_RECTL rSrc          = pData->rSrc;
    OV_RECTL rDest         = pData->rDest;
    LPDDPIXELFORMAT lpDPFsrc    = pData->lpDPFsrc;
    unsigned long* lpdwOffset = pData->lpdwOffset;    

    unsigned long dwHQVsrcWidth=0;
    unsigned long dwHQVdstWidth=0;
    unsigned long dwHQVsrcHeight=0;
    unsigned long dwHQVdstHeight=0;
    unsigned long dwHQVSrcTopOffset=0;
    unsigned long dwHQVSrcLeftOffset=0;
    unsigned long dwAlign = ~3;
   
    dwHQVsrcWidth   = (unsigned long)(rSrc.right - rSrc.left);
    dwHQVsrcHeight  = (unsigned long)(rSrc.bottom - rSrc.top);
       
   	dwHQVdstWidth = (unsigned long)(rDest.right - rDest.left);
   	dwHQVdstHeight = (unsigned long)(rDest.bottom - rDest.top);        

    if (dwVideoFlag&VIDEO_HQV_INUSE)
    {
        dwAlign = ~3; /* 4 bytes alignment if use HQV */
    }
    else
    {
        if (dwHQVsrcWidth>dwHQVdstWidth)
        {
            dwAlign = ~15; /* 16 bytes alignment if minify */
        }
        else
        {
            dwAlign = ~3; /* 4 bytes alignment if no minify */
        }
    }

    if ( (rSrc.left != 0) || (rSrc.top != 0) )
    {
        if (lpDPFsrc->dwFlags & DDPF_FOURCC)
        {
            switch (lpDPFsrc->dwFourCC)
            {
            case FOURCC_YUY2:
            case FOURCC_UYVY:
            case FOURCC_YVYU:
                DBG_DD(("vfGetSrcStartAddress_CME:FOURCC_YUY2\n"));
                if (dwVideoFlag&VIDEO_HQV_INUSE)
                {
                    if (dwHQVsrcHeight>dwHQVdstHeight)
                    {
                        dwHQVSrcTopOffset = 
                        (rSrc.top*dwHQVdstHeight/dwHQVsrcHeight)*dwSrcPitch;
                    }
                    else
                    {
                        dwHQVSrcTopOffset = rSrc.top * dwSrcPitch;
                    }
                    
                    if (dwHQVsrcWidth>dwHQVdstWidth)
                    {
                        dwHQVSrcLeftOffset = 
                        ((rSrc.left<<1)&dwAlign)*dwHQVdstWidth/dwHQVsrcWidth;
                    }
                    else
                    {
                        dwHQVSrcLeftOffset = (rSrc.left << 1)&dwAlign ;
                    }
                    *lpdwOffset= dwHQVSrcTopOffset+dwHQVSrcLeftOffset;
                }
                else
                {   
                    *lpdwOffset = ((rSrc.top * dwSrcPitch) +
                               ((rSrc.left << 1)&dwAlign));
                }
                break;
           case FOURCC_NV12:
                DBG_DD(("vfGetSrcStartAddress_CME:FOURCC_NV12\n"));
                dwSrcPitch *= 2;
                if (dwVideoFlag&VIDEO_HQV_INUSE)
                {
                    if (dwHQVsrcHeight>dwHQVdstHeight)
                    {
                        dwHQVSrcTopOffset = 
                        (rSrc.top*dwHQVdstHeight/dwHQVsrcHeight)*dwSrcPitch;
                    }
                    else
                    {
                        dwHQVSrcTopOffset = rSrc.top * dwSrcPitch;
                    }
                    
                    if (dwHQVsrcWidth>dwHQVdstWidth)
                    {
                        dwHQVSrcLeftOffset = 
                        ((rSrc.left<<1)&dwAlign)*dwHQVdstWidth/dwHQVsrcWidth;
                    }
                    else
                    {
                        dwHQVSrcLeftOffset = (rSrc.left << 1)&dwAlign ;
                    }
                    *lpdwOffset= dwHQVSrcTopOffset+dwHQVSrcLeftOffset;
                }
                else
                {   
                    *lpdwOffset = ((rSrc.top * dwSrcPitch) +
                               ((rSrc.left << 1)&dwAlign));
                }
                break;
            default:
                DBG_DD(("vfGetSrcStartAddress_CME : Invalid FOURCC format:(%lx)!\n", (unsigned long) lpDPFsrc->dwFourCC));
                break;
            }
        }
        else if (lpDPFsrc->dwFlags & DDPF_RGB)
        {
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                if (dwHQVsrcHeight>dwHQVdstHeight)
                {
                    dwHQVSrcTopOffset = 
                    (rSrc.top * dwHQVdstHeight/dwHQVsrcHeight)* dwSrcPitch;
                }
                else
                {
                    dwHQVSrcTopOffset = rSrc.top * dwSrcPitch;
                }
                
                switch (lpDPFsrc->dwRGBBitCount) 
                {
                case 16:
                    if (dwHQVsrcWidth>dwHQVdstWidth)
                    {
                        dwHQVSrcLeftOffset = 
                        ((rSrc.left<<1)&dwAlign) * dwHQVdstWidth/dwHQVsrcWidth;
                    }
                    else
                    {
                        dwHQVSrcLeftOffset = (rSrc.left << 1)&dwAlign ;
                    }
                    break;
                case 32:
                    if (dwHQVsrcWidth>dwHQVdstWidth)
                    {
                        dwHQVSrcLeftOffset = 
                        ((rSrc.left<<2)&dwAlign) * dwHQVdstWidth/dwHQVsrcWidth;
                    }
                    else
                    {
                        dwHQVSrcLeftOffset = (rSrc.left << 2)&dwAlign ;
                    }
                    break;
        
                default :
                    DBG_DD(("vfGetSrcStartAddress_CME : invalid RGB format %ld bits\n",lpDPFsrc->dwRGBBitCount));
                    if (dwHQVsrcWidth>dwHQVdstWidth)
                    {
                        dwHQVSrcLeftOffset = 
                        ((rSrc.left<<1)&dwAlign) * dwHQVdstWidth/dwHQVsrcWidth;
                    }
                    else
                    {
                        dwHQVSrcLeftOffset = (rSrc.left << 1)&dwAlign ;
                    }
                    break;
                }                    
                *lpdwOffset = dwHQVSrcTopOffset+dwHQVSrcLeftOffset;
            }
            else
            {
                *lpdwOffset = (rSrc.top * dwSrcPitch) +
                       (((rSrc.left * lpDPFsrc->dwRGBBitCount) >> 3)&dwAlign);
            }
        }
    }
}


/*
 * In/Out:  pData :video fetch count structure pointer
 * Ret:
 * Desc:   calculate HQV engine's fetch count.
 */

void vfGetFetch_CME(PVIDGETFETCH pData)
{
    unsigned long dwVideoFlag   = pData->dwVideoFlag;
    unsigned long dwCtlHQVflag  = pData->dwCtlHQVflag;
    LPDDPIXELFORMAT lpDPFsrc = pData->lpDPFsrc;
    unsigned long dwSrcWidth    = pData->dwSrcWidth;
    unsigned long dwDstWidth    = pData->dwDstWidth;
    unsigned long dwOriSrcWidth = pData->dwOriSrcWidth;
        
    unsigned long* lpdwFetch   = pData->lpdwFetch;
    unsigned long* lpdwHQVsrcFetch = pData->lpdwHQVsrcFetch;
   
    if (lpDPFsrc->dwFlags & DDPF_FOURCC)
    {
        switch (lpDPFsrc->dwFourCC) 
        {
        case FOURCC_NV12:
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                if ((dwCtlHQVflag&CTL_HQV_CSC_ENABLE_CME)&&
                    (dwCtlHQVflag&CTL_HQV_CSC_RGB32_CME))
                {                        
                    *lpdwHQVsrcFetch = dwOriSrcWidth<<1;
                    if (dwDstWidth >= dwSrcWidth)
                    {
                        if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
                        {                        
                            *lpdwFetch = 
                            ((((dwDstWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                            ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                        }
                        else
                        {
                            *lpdwFetch = 
                            ((((dwSrcWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                            ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                        }
                    }
                    else
                    {
                        *lpdwFetch = 
                        ((((dwDstWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                        ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                    }
                }
                else
                {
                    *lpdwHQVsrcFetch = dwOriSrcWidth;
                    if (dwDstWidth >= dwSrcWidth)
                    {
                        if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
                        {                        
                            *lpdwFetch = 
                            ((((dwDstWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                            ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                        }
                        else
                        {
                            *lpdwFetch = 
                            ((((dwSrcWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                            ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                        }
                    }
                    else
                    {
                        *lpdwFetch = 
                        ((((dwDstWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                        ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                    }
                }                
            }
            else
            {
                /* fetch one more quadword to avoid get less video data */
                *lpdwFetch = ((((dwSrcWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                                                                  ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
            }
            break;
        case FOURCC_UYVY:
        case FOURCC_YVYU:
        case FOURCC_YUY2:
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                if ((dwCtlHQVflag&CTL_HQV_CSC_ENABLE_CME)&&
                    (dwCtlHQVflag&CTL_HQV_CSC_RGB32_CME))
                {                        
                    *lpdwHQVsrcFetch = dwOriSrcWidth<<1;
                    if (dwDstWidth >= dwSrcWidth)
                    {
                        if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
                        {                        
                            *lpdwFetch = 
                            ((((dwDstWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                            ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                        }
                        else
                        {
                            *lpdwFetch = 
                            ((((dwSrcWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                            ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                        }
                    }
                    else
                    {
                        *lpdwFetch = 
                        ((((dwDstWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                        ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                    }
                }
                else
                {
                    *lpdwHQVsrcFetch = dwOriSrcWidth<<1;
                    if (dwDstWidth >= dwSrcWidth)
                    {
                        if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
                        {                        
                            *lpdwFetch = 
                            ((((dwDstWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                            ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                        }
                        else
                        {
                            *lpdwFetch = 
                            ((((dwSrcWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                            ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                        }
                    }
                    else
                    {
                        *lpdwFetch = 
                        ((((dwDstWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                        ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                    }
                }                
            }
            else
            {
                /* fetch one more quadword to avoid get less video data */
                *lpdwFetch = 
                ((((dwSrcWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
            }
            break;
        default :
            DBG_DD(("vfGetFetch_CME : Invalid FOURCC format :(%lx)!\n", (unsigned long) lpDPFsrc->dwFourCC));
            break;
        }
    }
    else if (lpDPFsrc->dwFlags & DDPF_RGB) 
    {
        switch (lpDPFsrc->dwRGBBitCount) 
        {
        case 16:
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                *lpdwHQVsrcFetch = dwOriSrcWidth<<1;
                if (dwDstWidth >= dwSrcWidth)
                {
                    if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
                    {
                        *lpdwFetch = 
                        ((((dwDstWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                        ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;                        
                    }
                    else
                    {
                        *lpdwFetch = 
                        ((((dwSrcWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                        ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                    }
                }
                else
                {
                    *lpdwFetch = 
                    ((((dwDstWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                    ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                }
            }
            else
            {
                /* fetch one more quadword to avoid get less video data */
                *lpdwFetch = 
                ((((dwSrcWidth<<1)+V1_FETCHCOUNT_ALIGNMENT)&
                ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
            }
           break;
        case 32:
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                *lpdwHQVsrcFetch = dwOriSrcWidth<<2;
                if (dwDstWidth >= dwSrcWidth)
                {
                    if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
                    {
                        *lpdwFetch = 
                        ((((dwDstWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                        ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;                        
                    }
                    else
                    {
                        *lpdwFetch = 
                        ((((dwSrcWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                        ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                    }
                }
                else
                {
                    *lpdwFetch = 
                    ((((dwDstWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                    ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
                }
            }
            else
            {
                /* fetch one more quadword to avoid get less video data */
                *lpdwFetch = 
                ((((dwSrcWidth<<2)+V1_FETCHCOUNT_ALIGNMENT)&
                ~V1_FETCHCOUNT_ALIGNMENT) >> V1_FETCHCOUNT_UNIT)+1;
            }
           break;

        default :
            DBG_DD(("vfGetFetch_CME : invalid RGB format %ld bits\n",lpDPFsrc->dwRGBBitCount));
            break;
        }
    }

    /* Fix plannar mode problem */
    if (*lpdwFetch <4)
    {
        *lpdwFetch = 4;
    }
}

/*
 * In/Out:  pData :video zoom-in/out structure pointer
 * Ret:     success/fail
 * Desc:   calculate HQV or video engine's width zoom-in/out control parameters 
 */

unsigned long vfCalcZoomWidth_CME(PVIDCALCZOOMW pData)
{
    unsigned long dwCtlHQVflag      = pData->dwCtlHQVflag;
    unsigned long dwSrcWidth        = pData->dwSrcWidth;
    unsigned long dwDstWidth        = pData->dwDstWidth;
    unsigned long* lpdwZoomCtl     = pData->lpdwZoomCtl;
    unsigned long* lpdwMiniCtl     = pData->lpdwMiniCtl;
    unsigned long* lpdwHQVminiCtl  = pData->lpdwHQVminiCtl;
    unsigned long* lpdwHQVzoomflagH= pData->lpdwHQVzoomflagH;
    unsigned long* lpdwMiniHFactor = pData->lpdwMiniHFactor;
    unsigned long* lpdwFetchAlignment = pData->lpdwFetchAlignment;

    unsigned long dwTmp=0;

    if (dwSrcWidth == dwDstWidth)
    {       
        *lpdwMiniHFactor = 1;/* Default value */
    }
    else
    {    
        if (dwSrcWidth < dwDstWidth) 
        {
            /* zoom in */
            if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
            {
                dwTmp = (dwSrcWidth-1)*0x0800*0x400 / (dwDstWidth-1);
                dwTmp = dwTmp / 0x400 + ((dwTmp&0x3ff)?1:0);
                *lpdwHQVminiCtl= dwTmp;
                *lpdwHQVminiCtl=((*lpdwHQVminiCtl)&0x7FF)|VAL_HQV_H_UP_SCALE_CME;
            }
            else
            {
                *lpdwZoomCtl = dwSrcWidth*0x0800 / dwDstWidth;
                *lpdwZoomCtl = (((*lpdwZoomCtl)&0x7FF)<<16)|V1_X_ZOOM_ENABLE;
                *lpdwMiniCtl |= ( V1_X_INTERPOLY );  /* set up interpolation */
                *lpdwHQVzoomflagH = 1;
            }
            *lpdwMiniHFactor = 1;
        } 
        else if (dwSrcWidth > dwDstWidth) 
        {
            /* zoom out */
            unsigned long dwSrcWidth1;
    
            /* HQV rounding patch */
            dwTmp = dwDstWidth*0x0800*0x400 / dwSrcWidth;
            dwTmp = dwTmp / 0x400 + ((dwTmp & 0x3ff)?1:0);

            *lpdwHQVminiCtl = (dwTmp & 0x7FF)| HQV_H_MINIFY_ENABLE;
            *lpdwHQVminiCtl |= VAL_HQV_H_DOWN_SCALE_CME;    
    
            dwSrcWidth1 = dwSrcWidth >> 1;
            if (dwSrcWidth1 <= dwDstWidth) 
            {
                *lpdwMiniCtl |= V1_X_DIV_2+V1_X_INTERPOLY;
                *lpdwFetchAlignment = 3;
                *lpdwMiniHFactor = 2;                
            }
            else 
            {
                dwSrcWidth1 >>= 1;
    
                if (dwSrcWidth1 <= dwDstWidth) 
                {
                    *lpdwMiniCtl |= V1_X_DIV_4+V1_X_INTERPOLY;
                    *lpdwFetchAlignment = 7;
                    *lpdwMiniHFactor = 4;
                }
                else 
                {
                    dwSrcWidth1 >>= 1;
    
                    if (dwSrcWidth1 <= dwDstWidth) 
                    {
                        *lpdwMiniCtl |= V1_X_DIV_8+V1_X_INTERPOLY;
                        *lpdwFetchAlignment = 15;
                        *lpdwMiniHFactor = 8;
                    }
                    else 
                    {
                        dwSrcWidth1 >>= 1;
    
                        if (dwSrcWidth1 <= dwDstWidth) 
                        {
                            *lpdwMiniCtl |= V1_X_DIV_16+V1_X_INTERPOLY;
                            *lpdwFetchAlignment = 15;
                            *lpdwMiniHFactor = 16;
                        }
                        else 
                        {
                            *lpdwMiniCtl |= V1_X_DIV_16+V1_X_INTERPOLY;
                            *lpdwFetchAlignment = 15;
                            *lpdwMiniHFactor = 16;
                        }
                    }
                }
            }
    
            if (dwSrcWidth1 < dwDstWidth) 
            {
                *lpdwZoomCtl = (dwSrcWidth1-2)*0x0800 / dwDstWidth;                
                if (*lpdwZoomCtl!=0)
                {
                    *lpdwZoomCtl = ((*lpdwZoomCtl&0x7FF)<<16)|V1_X_ZOOM_ENABLE;
                }
            }
        }
    }

    return VMI_OK;

}

/*
 * In/Out:  pData :video zoom-in/out structure pointer
 * Ret:     success/fail
 * Desc:   calculate HQV or video engine's width zoom-in/out control parameters. 
 *         The function is written for 3293/3353.Because their scalling
 *         register setting is differnet from other chips and the register 
 *         index differs too.
 */
unsigned long vfCalcZoomWidth_CME_293(PVIDCALCZOOMW pData)
{
    unsigned long dwCtlHQVflag      = pData->dwCtlHQVflag;
    unsigned long dwSrcWidth        = pData->dwSrcWidth;
    unsigned long dwDstWidth        = pData->dwDstWidth;
    unsigned long* lpdwZoomCtl     = pData->lpdwZoomCtl;
    unsigned long* lpdwMiniCtl     = pData->lpdwMiniCtl;
    unsigned long* lpdwHQVzoomflagH= pData->lpdwHQVzoomflagH;
    unsigned long* lpdwMiniHFactor = pData->lpdwMiniHFactor;
    unsigned long* lpdwFetchAlignment = pData->lpdwFetchAlignment;
    unsigned long*  lpdwHQVScaleCtlH= pData->lpdwHQVScaleCtlH; 

    unsigned long dwTmp=0;

    if (dwSrcWidth == dwDstWidth) /*Need not zoom*/
    {       
        *lpdwMiniHFactor = 1;/* Default value */
         return VMI_OK;
    }
       
    if (dwSrcWidth < dwDstWidth)   /* zoom in */
    {
        if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
        {
            dwTmp = (dwSrcWidth)*0x1000 / dwDstWidth;   
            *lpdwHQVScaleCtlH= dwTmp;
            *lpdwHQVScaleCtlH= ((*lpdwHQVScaleCtlH)&0x7FFF)|HQV_H_SCALE_ENABLE|HQV_H_SCALE_UP;
        }
        else
        {
            *lpdwZoomCtl = dwSrcWidth*0x0800 / dwDstWidth;
            *lpdwZoomCtl = (((*lpdwZoomCtl)&0x7FF)<<16)|V1_X_ZOOM_ENABLE;
            *lpdwMiniCtl |= ( V1_X_INTERPOLY );  /* set up interpolation */
            *lpdwHQVzoomflagH = 1;
        }
        *lpdwMiniHFactor = 1;
        return VMI_OK;    
    } 

    if (dwSrcWidth > dwDstWidth) /*Zoom out*/
    {
        unsigned long dwSrcWidth1;
        unsigned long dwtmpHQVministep; 
                                    
    
        dwSrcWidth1 = dwSrcWidth >> 1;
        if (dwSrcWidth1 <= dwDstWidth)      /*1 - 1/2*/ 
        {
            *lpdwMiniCtl |= V1_X_DIV_2+V1_X_INTERPOLY;
            *lpdwFetchAlignment = 3;
            *lpdwMiniHFactor = 2;       
            
            dwtmpHQVministep=0;  
        }
        else 
        {
            dwSrcWidth1 >>= 1;
    
            if (dwSrcWidth1 <= dwDstWidth)   /*1/2 - 1/4*/ 
            {
                *lpdwMiniCtl |= V1_X_DIV_4+V1_X_INTERPOLY;
                *lpdwFetchAlignment = 7;
                *lpdwMiniHFactor = 4;

                dwtmpHQVministep=0;   
            }
            else 
            {
                dwSrcWidth1 >>= 1;
    
                if (dwSrcWidth1 <= dwDstWidth) /*1/4 - 1/8*/ 
                {
                    *lpdwMiniCtl |= V1_X_DIV_8+V1_X_INTERPOLY;
                    *lpdwFetchAlignment = 15;
                    *lpdwMiniHFactor = 8;

                    if (dwSrcWidth1 < dwDstWidth) /*  < 1/8 */
                    {
                        dwtmpHQVministep=1;   
                    }
                    else /*  = 1/8 */
                    {
                        dwtmpHQVministep=2;   
                    }
                }
                else 
                {
                    dwtmpHQVministep=2;   
                    
                    dwSrcWidth1 >>= 1;
    
                    if (dwSrcWidth1 <= dwDstWidth) /*1/8 - 1/16*/ 
                    {
                        *lpdwMiniCtl |= V1_X_DIV_16+V1_X_INTERPOLY;
                        *lpdwFetchAlignment = 15;
                        *lpdwMiniHFactor = 16;
                          
                    }
                    else /* 1/16 -*/
                    {
                        *lpdwMiniCtl |= V1_X_DIV_16+V1_X_INTERPOLY;
                        *lpdwFetchAlignment = 15;
                        *lpdwMiniHFactor = 16;
                    }
                }
            }
        }
    
        if (dwSrcWidth1 < dwDstWidth)
        {
            *lpdwZoomCtl = (dwSrcWidth1-2)*0x0800 / dwDstWidth;                
            if (*lpdwZoomCtl!=0)
            {
                *lpdwZoomCtl = ((*lpdwZoomCtl&0x7FF)<<16)|V1_X_ZOOM_ENABLE;
            }
        }

        /*calcaulate HQV zoom out factor*/

        switch(dwtmpHQVministep)
        {
        case 0:
            dwTmp = ((dwSrcWidth*0x2000) / ((dwDstWidth<<1)+1));               
            *lpdwHQVScaleCtlH = HQV_H_SCALE_DOWN_FOURTH_TO_1;
            break;
        case 1:
            dwTmp = ((dwSrcWidth*0x1000) / (dwDstWidth));
            *lpdwHQVScaleCtlH = HQV_H_SCALE_DOWN_FOURTH_TO_EIGHTH;
            break;
        case 2:
            dwTmp = (dwDstWidth*0x1000 / (dwSrcWidth));
            *lpdwHQVScaleCtlH = HQV_H_SCALE_DOWN_UNDER_EIGHTH;
            break;
        }

        if (((dwTmp * 0x1000) & 0xfff) > 1)
        {
            dwTmp += 1;
        }
        *lpdwHQVScaleCtlH |= (dwTmp & 0x7FFF) | HQV_H_SCALE_ENABLE;

        return VMI_OK;
    }
    
    return VMI_OK;
}

/*
 * In/Out:  pData :video display count structure pointer
 * Ret:     
 * Desc:   calculate the display count that will be shown on screen.
 *         Usually the result is actual horizontal and vertical line number 
 *          which will be written in video display count registers
 */
void vfGetDisplayCount_CME(PVIDGETDISPLAYCOUNT pData)
{
    unsigned long dwVideoFlag       = pData->dwVideoFlag;
    LPDDPIXELFORMAT lpDPFsrc= pData->lpDPFsrc;
    unsigned long dwSrcWidth        = pData->dwSrcWidth;
    unsigned long dwMiniHFactor     = pData->dwMiniHFactor;
    unsigned long* lpdwDisplayCountW = pData->lpdwDisplayCountW;
        
    if (lpDPFsrc->dwFlags & DDPF_FOURCC)
    {
        switch (lpDPFsrc->dwFourCC) 
        {
        case FOURCC_NV12:
        case FOURCC_UYVY:
        case FOURCC_YVYU:
        case FOURCC_YUY2:
        case FOURCC_V444:
        case FOURCC_V410:
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                *lpdwDisplayCountW = dwSrcWidth - 1;
            }
            else
            {
                *lpdwDisplayCountW = dwSrcWidth - dwMiniHFactor;
            }
            break;
        default :
            DBG_DD(("vfGetDisplayCount_CME: Invalid FOURCC format :(%lx)!\n", (unsigned long) lpDPFsrc->dwFourCC));
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                *lpdwDisplayCountW = dwSrcWidth - 1;
            }
            else
            {
                *lpdwDisplayCountW = dwSrcWidth - dwMiniHFactor;
            }
            break;
        }
    }
    else if (lpDPFsrc->dwFlags & DDPF_RGB) 
    {
        switch (lpDPFsrc->dwRGBBitCount) 
        {
        case 16:
        case 32:
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                *lpdwDisplayCountW = dwSrcWidth - 1;
            }
            else
            {
                *lpdwDisplayCountW = dwSrcWidth - dwMiniHFactor;
            }
            break;

        default :
            DBG_DD(("vfGetDisplayCount_CME: invalid RGB format %ld bits\n",lpDPFsrc->dwRGBBitCount));
            if (dwVideoFlag&VIDEO_HQV_INUSE)
            {
                *lpdwDisplayCountW = dwSrcWidth - 1;
            }
            else
            {
                *lpdwDisplayCountW = dwSrcWidth - dwMiniHFactor;
            }
            break;
        }
    }   
}

/*
 * In/Out:  pData :video zoom-in/out structure pointer
 * Ret:     success/fail
 * Desc:   calculate HQV or video engine's height zoom-in/out control parameters 
 */

unsigned long vfCalcZoomHeight_CME(PVIDCALCZOOMH pData)
{
    unsigned long dwCtlHQVflag      = pData->dwCtlHQVflag;
    unsigned long dwSrcHeight       = pData->dwSrcHeight;
    unsigned long dwDstHeight       = pData->dwDstHeight;
    unsigned long* lpdwZoomCtl     = pData->lpdwZoomCtl;
    unsigned long* lpdwMiniCtl     = pData->lpdwMiniCtl;
    unsigned long* lpdwHQVminiCtl  = pData->lpdwHQVminiCtl;
    unsigned long* lpdwHQVzoomflagV= pData->lpdwHQVzoomflagV;

    unsigned long dwTmp=0;
    
    if (dwSrcHeight < dwDstHeight) 
    {
        /* zoom in */
        if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
        {
            /* -1 to avoid lacking one line */
            dwTmp = (dwSrcHeight-1) * 0x0800*0x400 / (dwDstHeight-1);
            /*Rounding process */
            dwTmp = dwTmp / 0x400 + ((dwTmp&0x3ff)?1:0);
            *lpdwHQVminiCtl |= (((dwTmp & 0x7FF)<<16) | VAL_HQV_V_UP_SCALE_CME);
        }
        else
        {
            dwTmp = dwSrcHeight * 0x0400 / dwDstHeight;
            *lpdwZoomCtl |= ((dwTmp & 0x3FF) | V1_Y_ZOOM_ENABLE);
            *lpdwMiniCtl |= (V1_Y_INTERPOLY | V1_YCBCR_INTERPOLY);
            *lpdwHQVzoomflagV = 1;
        }
    } 
    else if (dwSrcHeight == dwDstHeight)
    {       

    }
    else if (dwSrcHeight > dwDstHeight) 
    {
        /* zoom out */
        unsigned long dwSrcHeight1;
      
        /* HQV rounding patch */
        dwTmp = dwDstHeight*0x0800*0x400 / dwSrcHeight;
        dwTmp = dwTmp / 0x400 + ((dwTmp & 0x3ff)?1:0);
        
        *lpdwHQVminiCtl |= ((dwTmp& 0x7FF)<<16)|HQV_V_MINIFY_ENABLE;
        *lpdwHQVminiCtl |= VAL_HQV_V_DOWN_SCALE_CME;
      
        dwSrcHeight1 = dwSrcHeight >> 1;
        if (dwSrcHeight1 <= dwDstHeight) 
        {
            *lpdwMiniCtl |= V1_Y_DIV_2;
        }
        else 
        {
            dwSrcHeight1 >>= 1;
            if (dwSrcHeight1 <= dwDstHeight) 
            {
                *lpdwMiniCtl |= V1_Y_DIV_4;
            }
            else 
            {
                dwSrcHeight1 >>= 1;
      
                if (dwSrcHeight1 <= dwDstHeight) 
                {
                    *lpdwMiniCtl |= V1_Y_DIV_8;
                }
                else 
                {
                    dwSrcHeight1 >>= 1;
      
                    if (dwSrcHeight1 <= dwDstHeight) 
                    {
                        *lpdwMiniCtl |= V1_Y_DIV_16;
                    }
                    else 
                    {
                        *lpdwMiniCtl |= V1_Y_DIV_16;
                    }
                }
            }
        }      

        if (dwSrcHeight1 < dwDstHeight) 
        {
            dwTmp = dwSrcHeight1 * 0x0400 / dwDstHeight;
            *lpdwZoomCtl |= ((dwTmp & 0x3FF) | V1_Y_ZOOM_ENABLE);
            *lpdwMiniCtl |= ( V1_Y_INTERPOLY|V1_YCBCR_INTERPOLY);
        }
    }
    
    return VMI_OK;
}

/*
 * In/Out:  pData :video zoom-in/out structure pointer
 * Ret:     success/fail
 * Desc:   calculate HQV or video engine's height zoom-in/out control parameters. 
 *         The function is written for 3293/3353.Because their scalling
 *         register setting is differnet from other chips and the register 
 *         index differs too.
 */

unsigned long vfCalcZoomHeight_CME_293(PVIDCALCZOOMH pData)
{
    unsigned long dwCtlHQVflag      = pData->dwCtlHQVflag;
    unsigned long dwSrcHeight       = pData->dwSrcHeight;
    unsigned long dwDstHeight       = pData->dwDstHeight;
    unsigned long* lpdwZoomCtl     = pData->lpdwZoomCtl;
    unsigned long* lpdwMiniCtl     = pData->lpdwMiniCtl;
    unsigned long* lpdwHQVzoomflagV= pData->lpdwHQVzoomflagV;

    unsigned long* lpdwHQVScaleCtlV= pData->lpdwHQVScaleCtlV;

    unsigned long dwTmp=0;

    if (dwSrcHeight == dwDstHeight) /*need not zoom*/
    {       
         return VMI_OK;
    }

    if (dwSrcHeight < dwDstHeight)   /* zoom in */
    {

        if (dwCtlHQVflag&CTL_HQV_SCALING_UP_ENABLE_CME)
        {
            dwTmp = (dwSrcHeight) * 0x1000 / (dwDstHeight);
             /*Rounding process*/
            *lpdwHQVScaleCtlV |= (dwTmp & 0x1FFFF)|HQV_V_SCALE_ENABLE;
            *lpdwHQVScaleCtlV |= HQV_V_SCALE_UP;
        }
        else
        {
            dwTmp = dwSrcHeight * 0x0400 / dwDstHeight;
            *lpdwZoomCtl |= ((dwTmp & 0x3FF) | V1_Y_ZOOM_ENABLE);
            *lpdwMiniCtl |= (V1_Y_INTERPOLY | V1_YCBCR_INTERPOLY);
            *lpdwHQVzoomflagV = 1;
        }
        return VMI_OK;        
    } 
    
    if (dwSrcHeight > dwDstHeight)  /* zoom out */
    {
        unsigned long dwSrcHeight1;
      
        dwTmp = dwSrcHeight * 0x2000 / ((dwDstHeight<<1)+1);

        if (((dwTmp * 0x1000) & 0xfff) > 1) 
        {
            dwTmp += 1;
        }
            
        *lpdwHQVScaleCtlV = (dwTmp & 0x1FFFF)|HQV_V_SCALE_ENABLE;
        *lpdwHQVScaleCtlV |= HQV_V_SCALE_DOWN;

      
        dwSrcHeight1 = dwSrcHeight >> 1;
        if (dwSrcHeight1 <= dwDstHeight) 
        {
            *lpdwMiniCtl |= V1_Y_DIV_2;
        }
        else 
        {
            dwSrcHeight1 >>= 1;
            if (dwSrcHeight1 <= dwDstHeight) 
            {
                *lpdwMiniCtl |= V1_Y_DIV_4;
            }
            else 
            {
                dwSrcHeight1 >>= 1;
      
                if (dwSrcHeight1 <= dwDstHeight) 
                {
                    *lpdwMiniCtl |= V1_Y_DIV_8;
                }
                else 
                {
                    dwSrcHeight1 >>= 1;
      
                    if (dwSrcHeight1 <= dwDstHeight) 
                    {
                        *lpdwMiniCtl |= V1_Y_DIV_16;
                    }
                    else 
                    {
                        *lpdwMiniCtl |= V1_Y_DIV_16;
                    }
                }
            }
        }      

        if (dwSrcHeight1 < dwDstHeight) 
        {
            dwTmp = dwSrcHeight1 * 0x0400 / dwDstHeight;
            *lpdwZoomCtl |= ((dwTmp & 0x3FF) | V1_Y_ZOOM_ENABLE);
            *lpdwMiniCtl |= ( V1_Y_INTERPOLY|V1_YCBCR_INTERPOLY);
        }
        return VMI_OK;
    }
    
    return VMI_OK;
}

 /*
 * In/Out:  pData :TV flicker control structrue pointer
 * Ret:     
 * Desc:   compute VT flicker control settings. 
 */
void vfTVFlickerFilterControl_CME(PVIDTVFILTER pData)
{
    PVIDDEVICEINFO pVDevInfo= pData->pVDevInfo;
    OV_RECTL dwOriSrcRect      = pData->dwOriSrcRect;
    OV_RECTL dwOriDestRect     = pData->dwOriDestRect;
    unsigned long dwMPEGInTVFullScreen  = pData->dwMPEGInTVFullScreen;
    unsigned long dwDisableTVFlicker    = pData->dwDisableTVFlicker;
    unsigned long* lpdwMPEGInTVFullScreen = pData->lpdwMPEGInTVFullScreen;

    if (dwDisableTVFlicker)
    {
        if (pVDevInfo->dwcxScreen == 720)
        {
            DBG_DD(("DVD on 720 mode\n"));
            if (pVDevInfo->dwcyScreen == 480)
            {
                DBG_DD(("DVD on NTSC 720*480 mode\n"));
                if ((dwOriSrcRect.top == 0) &&
                    (dwOriSrcRect.left == 0) &&
                    (dwOriSrcRect.bottom == 480) &&
                    (dwOriSrcRect.right == 720))
                {
                    DBG_DD(("DVD is not out of Screen\n"));
                    if (((dwOriSrcRect.bottom - 
                          dwOriSrcRect.top) == 
                         (dwOriDestRect.bottom -
                          dwOriDestRect.top)) || 
                        ((dwOriSrcRect.right -
                          dwOriSrcRect.left) ==
                         (dwOriDestRect.right -
                          dwOriDestRect.left)))
                    {
                        DBG_DD(("DVD display on NTSC 720*480 full Screen\n"));
                        if (!dwMPEGInTVFullScreen)
                        {
                            *lpdwMPEGInTVFullScreen = 1;
                        }
                    }
                    else
                    {
                        if (dwMPEGInTVFullScreen)
                        {
                            *lpdwMPEGInTVFullScreen = 0;
                        }
                    }
                }
                else
                {
                    if (dwMPEGInTVFullScreen)
                    {
                        *lpdwMPEGInTVFullScreen = 0;
                    }
                }
            }
            else if (pVDevInfo->dwcyScreen == 576)    
            {
                DBG_DD(("DVD on PAL 720*576 mode\n"));
                if ((dwOriSrcRect.top == 0) &&
                    (dwOriSrcRect.left == 0) &&
                    (dwOriSrcRect.bottom == 576) &&
                    (dwOriSrcRect.right == 720))
                {
                    DBG_DD(("DVD is not out of Screen\n"));
                    if (((dwOriSrcRect.bottom - 
                          dwOriSrcRect.top) == 
                         (dwOriDestRect.bottom -
                          dwOriDestRect.top)) || 
                        ((dwOriSrcRect.right -
                          dwOriSrcRect.left) ==
                         (dwOriDestRect.right -
                          dwOriDestRect.left)))
                    {
                        DBG_DD(("DVD display on PAL 720*576 full Screen\n"));
                        if (!dwMPEGInTVFullScreen)
                        {
                            *lpdwMPEGInTVFullScreen = 1;
                        }
                    }
                    else
                    {
                        if (dwMPEGInTVFullScreen)
                        {
                            *lpdwMPEGInTVFullScreen = 0;
                        }
                    }
                }
                else
                {
                    if (dwMPEGInTVFullScreen)
                    {
                        *lpdwMPEGInTVFullScreen = 0;
                    }
                }
            }
            else
            {
                DBG_DD(("Not in NTSC 720*480 or PAL 720*576 mode\n"));
            }
        }
    }
    else
    {   
        /* Enable TV Flicker Filter */
        if (dwMPEGInTVFullScreen)
        {
            *lpdwMPEGInTVFullScreen = 0;
        }        
    }    
}

 
/*
* In/Out:  pData :structure pointer to describe HQV tile mode,color space
*          conversion and control registers
* Ret:     
* Desc:   calculate tile mode. color space conversion and deblocking setting.
*         Usually, the value is for HQV 0x3dc register
*/
void vfSetHQVTileModeAndCSCcontrol_CME(PVIDSETTILEANDCSC pData)
{
    unsigned long dwCtlHQVflag      = pData->dwCtlHQVflag;
    unsigned long dwTCGDRegVal      = pData->dwTCGDRegVal;
    unsigned long dwOriSrcWidth     = pData->dwOriSrcWidth;
    unsigned long dwOriSrcHeight    = pData->dwOriSrcHeight;
    unsigned long dwUseCSC          = pData->dwUseCSC;
    unsigned long* lpdwHQVTileCSCVal = pData->lpdwHQVTileCSCVal;

    unsigned long dwImageSize=0;
    
    DBG_DD(("vfSetHQVTileModeAndCSCcontrol_CME:\n"));
    *lpdwHQVTileCSCVal = dwTCGDRegVal&MASK_TILE_CSC_CME;
    /* Setting HQV Color Space Conversion mode
     * Only YUV source can use HQV CSC function
     */
    if (dwUseCSC)
    {
        if (dwCtlHQVflag&CTL_HQV_CSC_ENABLE_CME)
        {
            if (dwCtlHQVflag&CTL_HQV_CSC_RGB32_CME)
            {
                *lpdwHQVTileCSCVal |= VAL_HQV_CSC_ENABLE_CME|VAL_HQV_CSC_RGB32_CME;
            }
            else
            {
                *lpdwHQVTileCSCVal |= VAL_HQV_CSC_ENABLE_CME|VAL_HQV_CSC_RGB16_CME;            
            }
        }
    }
    
    /* Setting HQV Tile mode */
    if (dwCtlHQVflag&CTL_HQV_TILEMODE_ENABLE_CME)
    {
        if (dwCtlHQVflag&CTL_HQV_TILEMODE_512BITS_CME)
        {
            *lpdwHQVTileCSCVal |= VAL_HQV_TILEMODE_512BITS_CME;
        }
        else
        {
            *lpdwHQVTileCSCVal |= VAL_HQV_TILEMODE_256BITS_CME;            
        }
    }

    if (dwCtlHQVflag&CTL_HQV_DEINTERLACE_ENABLE_CME)
    {
        dwImageSize = ((dwOriSrcWidth*dwOriSrcHeight)>>10)&MASK_HQV_IMAGE_SIZE_CME;
        *lpdwHQVTileCSCVal |= dwImageSize;
    }

	/* For Deblocking */
	if ( dwCtlHQVflag&CTL_HQV_DEBLOCKING_ENABLE )
	{
       	 *lpdwHQVTileCSCVal |= VAL_HQV_DEBLOCKING_ENABLE;
       }
       else
       {
       	 *lpdwHQVTileCSCVal &= ~VAL_HQV_DEBLOCKING_ENABLE;
       }

    if (dwCtlHQVflag&CTL_HQV_TVFIELDSYNC_ENABLE)
    {
        *lpdwHQVTileCSCVal|=VAL_HQV_TVFIELDSYNC_ENABLE;

        if (dwCtlHQVflag&CTL_HQV_TV_ON_ADD_BUS)
        {
            *lpdwHQVTileCSCVal|=VAL_HQV_TV_ON_ADD_BUS;            
        }
        else
        {
            *lpdwHQVTileCSCVal &= ~VAL_HQV_TV_ON_ADD_BUS;                      
        }
    }
    else
    {
        *lpdwHQVTileCSCVal&= ~ (VAL_HQV_TVFIELDSYNC_ENABLE|
                                VAL_HQV_TV_ON_ADD_BUS);
    }
}

/*
* In/Out:  pData :structure pointer to describe HQV motion-adaptive & deinterlace
*          function
* Ret:     
* Desc:   calculate HQV deinterlace setting values.
*         Usually, the value is for HQV 0x3e4 register
*/
void vfSetHQVDeinterlaceControl_CME(PVIDSETDEINTERLACE pData)
{
    unsigned long dwCtlHQVflag              = pData->dwCtlHQVflag;
    unsigned long* lpdwHQVDeinterlaceVal   = pData->lpdwHQVDeinterlaceVal;
    
    DBG_DD(("vfSetHQVDeinterlaceControl_CME:"));
    if (dwCtlHQVflag&CTL_HQV_DEINTERLACE_ENABLE_CME)
    {
        DBG_DD(("Deinterlace Control Enable\n"));
        *lpdwHQVDeinterlaceVal = VAL_HQV_EDGE_DETECTION_THRESHOLD_16_CME|
                                 VAL_HQV_EDGE_DETECTION_ENABLE_CME;


        if (dwCtlHQVflag&CTL_HQV_3to2PULLDOWN_ENABLE_CME)
        {
            *lpdwHQVDeinterlaceVal |= VAL_PULL_DOWN_2_2_ENABLE_CME|
                                VAL_PULL_DOWN_INTER_FIELD_THRESHOLD_16_CME|
                                VAL_PULL_DOWN_3_2_ENABLE_CME|
                                VAL_PULL_DOWN_INTRA_FIELD_THRESHOLD_FACTOR_2_CME|
                                VAL_PULL_DOWN_FACTOR2_4_8_CME|
                                VAL_PULL_DOWN_FACTOR1_2_8_CME|
                                VAL_PULL_DOWN_MIN_THRESHOLD_2_CME;
        }
    }
    else
    {
        *lpdwHQVDeinterlaceVal = 0;        
    }
}

/*
* In/Out:  pData :structure pointer to describe HQV source data offset
* Ret:     
* Desc:   calculate HQV source data offset setting .Usually the value is
*          used for set HQV 0x3cc register when HQV is used for scalling up video
*/

void vfGetHQVSourceOffset_CME(PVIDGETHQVSRCOFFSET pData)
{
    OV_RECTL rSrc      = pData->rSrc;
    LPDDPIXELFORMAT lpDPFsrc= pData->lpDPFsrc;
    unsigned long* lpdwHQVOffset   = pData->lpdwHQVOffset;

    DBG_DD(("vfGetHQVSourceOffset_CME:\n"));

    if (lpDPFsrc->dwFlags & DDPF_FOURCC)
    {
        switch (lpDPFsrc->dwFourCC) 
        {
        case FOURCC_NV12:       
            *lpdwHQVOffset=((rSrc.left&~0x00000001)<<16)|rSrc.top;
            break;
        case FOURCC_YUY2:
        default :
            *lpdwHQVOffset=(((rSrc.left&~0x00000001)<<1)<<16)|rSrc.top;
            break;
        }
    }
    else if (lpDPFsrc->dwFlags & DDPF_RGB) 
    {
        switch (lpDPFsrc->dwRGBBitCount) 
        {
        case 32:
            *lpdwHQVOffset=(((rSrc.left&~0x00000001)<<2)<<16)|rSrc.top;
            break;
        case 16:
        default :
            *lpdwHQVOffset=(((rSrc.left&~0x00000001)<<1)<<16)|rSrc.top;
            break;
        }
    }    
}

/*
* In/Out:  pData :structure pointer to describe HQV source data offset
* Ret:     
* Desc:   get source data offset .Usually the value is for VT3353 0x388
*/

void vfGetHQVSrcDataOffset_CME(PVIDGETHQVSRCOFFSET pData)
{
    OV_RECTL rSrc = pData->rSrc;

    DBG_DD(("vfGetHQVSrcDataOffset_CME:\n"));

    /* HQV380: Should be set to 0. */
    *pData->lpdwHQVSrcDataOffsetStart = 0;

    /* HQV388[26:16]: Horizontal offset of end point for video location in destination picture. */
    /* HQV388[10:0]: Vertical offset of end point for video location in destination picture. */    
	/* Zero base, so need to minus 1 for end position. */
    *pData->lpdwHQVSrcDataOffsetEnd = ((rSrc.right - 1 ) << 16) | (rSrc.bottom - 1);
}            

