/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 
#ifndef __VIDSTRUC__
#define __VIDSTRUC__

#include "viampg.h"        /* for VIAMPGSURFACE, VIAMACROBLOCK struct */
#include "viavid.h"        /* for DDLOCK, DDUPDATEOVERLAY struct */

/**************************************************************************
* Defination for Video function
**************************************************************************/
#define BASE_VIDIOCPRIVATE	192		/* for V4L private ioctl, 192-255 are private */

/* H/W mpeg surfaces ( frame buffers ) we use */
#define NUM_FRAME_BUFFERS           20

#if 1
/*define WKNAGPCONTROL   */
#define MAX_FRAMEENDINDEX          0x10

typedef unsigned long DWORD;
typedef unsigned int  WORD;


typedef struct _BufferRecord{
    unsigned long   Start_pointer;
    unsigned long   End_pointer;
    unsigned long   Selector;
} BUFREC;

typedef struct _FrameEndRecord
{
    unsigned long   dwFrameEndAddr;
    unsigned long   dwRewindCount;
} FrameEndRecord;


typedef struct _VIDEO_AGP_MEMORY_PARAMS
{
    void * VirtualContext; 
    unsigned long PhysicalContext; 
    unsigned long             NumAGPPages;                      
    unsigned long             dwAGPMaxPages;                    
    void * KernelModeLinearAddress;
    void * UserModeLinearAddress;
} VIDEO_AGP_MEMORY_PARAMS, *PVIDEO_AGP_MEMORY_PARAMS;

typedef struct _WKNAGPCmdBufControl{

    unsigned char   *pHWAGPCur; 
    unsigned char   *pMMIOBase;
    unsigned long   dwFlag;
    unsigned long   dwCmdBufType;

    unsigned long   AGPBasePHYSADD; 
    unsigned long   AGPBaseVirtual; 

    BUFREC  BufRec[3];
    unsigned long   dwCurBufNo;
	 
    unsigned long   dwCmdBufBegin;
    unsigned long   dwCmdBufEnd;
	 
    unsigned long   dwCmdBufPhys;
    unsigned long   dwCmdBufSize;
    FrameEndRecord   FrameEndAddr[MAX_FRAMEENDINDEX];
    unsigned long   dwCurSWFrameIdx;
    unsigned long   dwCurHWFrameIdx;

    unsigned long   dwStartPointer;
    unsigned long   dwEndPointer;
    unsigned long   dwLastPause;
    unsigned long   dwHwLastPointer;

    unsigned long   dwSWCurPointer;

    unsigned long   dwRewindCount;

    void    *pvmHeapSt;
    void    *pvmHeapCur;
    unsigned long   dwNumHeaps;
    unsigned char  *dwRegBase;
    unsigned long   dwHWCaps;

    VIDEO_AGP_MEMORY_PARAMS  AGPAllocInfo;
    unsigned long       dwUseAGP;
    unsigned long dwAGPVQStatus;
    unsigned long   dwMutex;
} WKNAGPCMDBUFCONTROL, *LPWKNAGPCMDBUFCONTROL;

typedef struct _WKNAGPControl{

    unsigned long   dwAGPUsedCount;
    unsigned long   dwFlagDriverRunning;
    unsigned long   dwD3DAGPUsedCount;
    unsigned long   dwD3DPCIUsedCount;
    unsigned long   dwICDAGPUsedCount;
    unsigned long   dwICDPCIUsedCount;
    unsigned long   dw3DUsedCount;
    unsigned long   dwMPEGAGPUsedCount;

    unsigned long   dwFlag;

    unsigned long   AGPBaseVirtual;
    unsigned long   AGPBasePHYSADD;
    unsigned long   dwCmdBufSize;
} WKNAGPCONTROL, *LPWKNAGPCONTROL;



#endif

/**************************************************************************
* Structure defination for Video function
**************************************************************************/
/*Video HW Difference Structure*/
typedef struct _VIDHWDIFFERENCE
{
    unsigned long dwThreeHQVBuffer;             /*Use Three HQV Buffers*/
    unsigned long dwV3SrcHeightSetting;         /*Set Video Source Width and Height*/
    unsigned long dwSupportExtendFIFO;          /*Support Extand FIFO*/
    unsigned long dwHQVFetchByteUnit;           /*HQV Fetch Count unit is byte*/
    unsigned long dwHQVInitPatch;               /*Initialize HQV Engine 2 times*/
    unsigned long dwSupportV3Gamma;             /*Support V3 Gamma*/
    unsigned long dwUpdFlip;                    /*Set HQV3D0[15] to flip video*/
    unsigned long dwHQVDisablePatch;            /*Change Video Engine Clock setting for HQV disable bug*/
    unsigned long dwSUBFlip;                    /*Set HQV3D0[15] to flip video for sub-picture blending*/
    unsigned long dwNeedV1Prefetch;             /*V1 pre-fetch function for K8*/
    unsigned long dwNeedV3Prefetch;             /*V3 pre-fetch function for K8*/
    unsigned long dwNeedV4Prefetch;             /*V4 pre-fetch function for K8*/
    unsigned long dwUseSystemMemory;            /*Use system memory for DXVA compressed data buffers*/
    unsigned long dwExpandVerPatch;             /*Patch video HW bug in expand SIM mode or same display path*/
                                        		/*SW needs to calculate vertical scaling factor.*/
    unsigned long dwExpandVerHorPatch;          /*Patch video HW bug in expand SAMM mode or same display path*/
    unsigned long dwExpandSAMMHorPatch;         /*Patch video HW bug in expand SAMM mode. Video horizontal start position needs to -3.*/
    unsigned long dwBypassVerExpandMagnify;     /*HW bypass the expand vertical magnify.*/
                                        		/*SW need to calculate the Y magnify of video by itself.*/
             		               		/*This flag only support VT3285, VT3204 A1 and VT3259.*/
    unsigned long dwBypassHorExpandMagnify;     /*HW bypass the expand horizontal magnify.*/
                                        		/*SW need to calculate the X magnify of video by itself.*/
                                        		/*This flag only support VT3285, VT3204 A1 and VT3259.*/
    unsigned long dwV3ExpireNumTune;            /*Change V3 expire number setting for V3 bandwidth issue*/
    unsigned long dwV3FIFOThresholdTune;        /*Change V3 FIFO, Threshold and Pre-threshold setting for V3 bandwidth issue*/
    unsigned long dwMCRacingPatch;              /*Patch MC MPEG Engine Racing Problem*/
    unsigned long dwSLICERacingPatch;           /*Patch MPEG Slice(use VQ) and HQV Racing Problem*/
    unsigned long dwCheckHQVFIFOEmpty;          /*HW Flip path, need to check HQV FIFO status*/
    unsigned long dwUseMPEGAGP;                 /*Use MPEG AGP function*/
    unsigned long dwV3FIFOPatch;                /*For CLE V3 FIFO Bug (srcWidth <= 8)*/
    unsigned long dwSupportTwoColorKey;         /*Support two color key*/
    unsigned long dwEnableErrorConcealment;     /* Enable error concealment */
    unsigned long dwUseMMX;                     /* Enable MMX inctruction */
    unsigned long dw205BFIFO64;                 /* For new V3 FIFO setting of 205B*/
    unsigned long dwSupport512MAddr;            /* Support 512M memory address */
    unsigned long dwHQVOutputFIFO32Level;       /* HQV Output FIFO 32 level, else is 64 level. */
    unsigned long dwVideoFlipOddEvenBit31;      /* Video engine(without HQV) flip Odd/Even Bit change(27->31) */
    unsigned long dwSupportVideoVQ;             /* HW support Video VQ. only for VT3259 */
    unsigned long dwOnlyOneVideoOverlay;
    unsigned long dwAGPSupportHeader7;         
}VIDHWDIFFERENCE, * LPVIDHWDIFFERENCE;

/*For Video HW Difference */
#define VID_HWDIFF_TRUE           0x00000001
#define VID_HWDIFF_FALSE          0x00000000


/*	DDPIXELFORMAT
 *  purpose : determine the video data format
 */
typedef struct _DDPIXELFORMAT
{
    unsigned long   dwSize;                 /* size of structure */
    unsigned long   dwFlags;                /* pixel format flags */
    unsigned long   dwFourCC;               /* (FOURCC code) */

    unsigned long   dwRGBBitCount;          /* how many bits per pixel */
    unsigned long   dwYUVBitCount;          /* how many bits per pixel */
    unsigned long   dwZBufferBitDepth;      /* how many bits for z buffers */
    unsigned long   dwAlphaBitDepth;        /* how many bits for alpha channels */

    unsigned long   dwRBitMask;             /* mask for red bit */
    unsigned long   dwYBitMask;             /* mask for Y bits */

    unsigned long   dwGBitMask;             /* mask for green bits */
    unsigned long   dwUBitMask;             /* mask for U bits */

    unsigned long   dwBBitMask;             /* mask for blue bits */
    unsigned long   dwVBitMask;             /* mask for V bits */

    unsigned long   dwRGBAlphaBitMask;      /* mask for alpha channel */
    unsigned long   dwYUVAlphaBitMask;      /* mask for alpha channel */
    unsigned long   dwRGBZBitMask;          /* mask for Z channel */
    unsigned long   dwYUVZBitMask;          /* mask for Z channel */
} DDPIXELFORMAT;
typedef DDPIXELFORMAT * LPDDPIXELFORMAT;

/* dwFlags definition of DDPIXELFORMAT */
/* The FourCC code is valid.  */
#define DDPF_FOURCC                             0x00000004l

/* The RGB data in the pixel format structure is valid. */
#define DDPF_RGB                                0x00000040l

/*Added for SAMM*/
typedef struct {
    unsigned long dwWidth;          /* On screen Width                  */
    unsigned long dwHeight;         /* On screen Height                 */
    unsigned long dwSaveWidth;      /* On screen Width for Panning      */
    unsigned long dwSaveHeight;     /* On screen Height for Panning     */
    unsigned long dwCurWidth;       /* Width set by xrandr     */
    unsigned long dwCurHeight;      /* Height set by xrandr     */   
    unsigned long dwBPP;            /* Bits Per Pixel                   */
    unsigned long dwPitch;          /* On screen pitch= width x BPP     */
    unsigned long dwRefreshRate;    /* Refresh rate of the mode         */
    unsigned long TotalVRAM;        /* Total Video RAM in unit of byte  */
    unsigned long VideoHeapBase;    /* Physical Start of video heap     */
    unsigned long VideoHeapEnd;     /* Physical End of video heap       */
    unsigned long dwDVIOn;    	    /* Is it DVI simultaneous mode ?    */
    unsigned long dwLCDOn;    	    /* Is it LCD simultaneous mode ?    */
    unsigned long dwExpand;    	    /* Is DVI in expand mode ?          */
    unsigned long dwPanning;    	/* Is DVI in Pannig mode ?          */
    unsigned long IsDownScaling;    /* Is down scaling enabled?         */
    unsigned long dwPanelWidth;     /* Panel physical Width             */
    unsigned long dwPanelHeight;    /* Panel physical Height            */
    unsigned long dwActiveDevice;   /* Active Device; ex. CRT or LCD    */
}SCREENINFO, * LPSCREENINFO;

typedef struct {
    int MMIOhandle;
    int MMIObase;
    int MMIOsize;
    int FBbase;
    int FBsize;
    unsigned long FBhandle;
    
    unsigned long HQVFilter_ManualSelect;   /* HQV Filter_Manual Select         */
    int MpegHQV_VFilterMode;                /* MPEG HQV vertical filter mode    */
    int MpegHQV_HFilterMode;                /* MPEG HQV horizontal filter mode  */
    int CapHQV_VFilterMode;                 /* Capture HQV vertical filter mode */
    int CapHQV_HFilterMode;                 /* Capture HQV horizontal filter mode */
    unsigned long SetMpeg_FBNumber;         /* Set Mpeg decode frame buffer number */
    unsigned long HQV_ManualSwitch;         /* HQV engine manual switch select      */
    Bool  IsSupportedColorSpaceChange;
    Bool  ColorSpaceChanged;                /* The setting value of color space been changed by user    */
    unsigned long ColorSpaceValue1;         /* User set color value from utility , after driver convert */
    unsigned long ColorSpaceValue2;         /* User set color value from utility , after driver convert */
    unsigned long VideoUseSecondDisplay;    /* Video display to second display, for Hitach-DavinCi Twin FS454 case */
    Bool  DRMEnabled;                       /* kernel module DRM flag           */
    Bool  Screen1IsLeft;                    /* True: Screen1 LeftOf Screen0 ; False: Screen1 RightOf Screen0 */
    Bool  Screen1IsAbove;                   /* True: Screen1 Above Screen0 ; False: Screen1 Below Screen0 */
    Bool  SAMM;                             /* True: XServer in SAMM mode  */
    unsigned long dwRevisionID;             /* The chip revision ID */
    unsigned long dwDeviceID;               /* The chip device ID */
    unsigned long dwProjectIDRevision;
    unsigned long dwActiveDevice;           /* ActiveDevice; ex. SAMM: CRT+LCD  */
    int scrnIndex;                          /* The index of current screen. */
    SCREENINFO  Screen[2];                  /* Information for Primary & Secondary Screens*/
    unsigned long NoMPEGAddOneLineOnBottom; /* Disable HQV bob-deinterlacing, 0x3D0[16] */
    unsigned long VideoColorKey;
    Bool  ReDrawColorkey;                   /* ReDrawColorKey at viaPutImageG everytime*/
    unsigned long ScreenPhysAddr;           /* Physical address of Video Memory */
	unsigned long dwSurfMgrEnabled;         /* indicate if V4L surface manager initialized */
	unsigned long dwXServerEnabled;         /* indicate if XServer On */
    Bool DeBlockingEnable;                  /* DeBlockingEnable */
    Bool dw3to2PullDownDisable;             /* 3:2/2:2 pull down option */

    Bool bUse2DBitBltSignature;             /* add signature on MPEG surface to make sure MPG decode END */
    Bool bPatchMPEGDecodeSliceErrorBitstreamDisable;  /* patch decode error bitstream */
    short rotate,duoview;                   /*Xserver rotation mode 0:no,1:CW,-1:CCW. duoview for DuoView of Xserver*/
    Bool bVideoEngineUsedOut;               /*No Video Engine available.*/	
    unsigned long IGAPanningStatus;         /* pBIOSInfo->IGA1SettingInfo.IsPanning
                                            | pBIOSInfo->IGA2SettingInfo.IsPanning<<1, initial at FillGraphicInfo() */
    unsigned long Screen_IGA_Map[3];        /* Map Screen to IGA, initial at FillGraphicInfo() */
    unsigned long IGA_Video_Map[3];         /* Indicate which IGA use which Video Engine, V1 or V3, initial at UpdateOverlay() */
    unsigned long OverlayToUpdate;          /* Which Video Overlay need to be Update, used by UpdateOverlay() */
    Bool IsHWCursorEnabled;                 /* Indicate whether the HW cursor is enabled. */
    Bool IsRotationEnabled;                 /* Indicate whether the Rotation function is enabled, no matter the rotation is done by SW, HW or RandR. */ 
    Bool IsSurfaceUsedForRotate;            /* Indicate whether the surface is created for video rotation. */
    unsigned long TMDSChipID;               /* TMDS CHIP ID*/
    unsigned long HDCPPowerOn;              /* HDCP Power On/Off*/
}VIAGRAPHICINFO, * LPVIAGRAPHICINFO;

/* Define for DeBlocking minimum threshold unit=pixel */
#define DEBLOCKING_MINIMUM_WIDTH_DEFAULT	320
#define DEBLOCKING_MINIMUM_HEIGHT_DEFAULT	240


/* Device ID Definition 
  *
  *  3123   CLE266
  *  3205   KN400A/KM400A
  *  3204   K8N800/k8M800
  *  3259   PM800/PN800/CN400
  *  3314   P4M800Pro/VN800/CN700
  *  3324   CX700
  *  3336   K8N890/K8M890
  *  3327   P4M890/VN890/CN800
  *  3293   CN750/VN900/VN900M 
  */
#define VIA_DEVICE_CLE1             0x3022
#define VIA_DEVICE_CLE2             0x3122
#define VIA_DEVICE_VT3205           0x7205
#define VIA_DEVICE_VT3204           0x3108
#define VIA_DEVICE_VT3259           0x3118
#define VIA_DEVICE_VT3314           0x3344
#define VIA_DEVICE_VT3324           0x3157
#define VIA_DEVICE_VT3336           0x3230
#define VIA_DEVICE_VT3327           0x3343
#define VIA_DEVICE_VT3293           0x3225
#define VIA_DEVICE_VT3364           0x3371
#define VIA_DEVICE_VT3353           0x1122 

/* Revision ID Definition */
#define VIA_REVISION_CLEC0          0x10
#define VIA_REVISION_CLEC1          0x11
#define VIA_REVISION_CLECX          0x10
#define VIA_REVISION_CLEAX          0x0F

#define UNICHROME_VT3123AX          0x01
#define UNICHROME_VT3123CX          0x02
#define UNICHROME_VT3205            0x03
#define UNICHROME_VT3204            0x04
#define UNICHROME_VT3259            0x05
#define UNICHROME_VT3259Lite        0x06
#define UNICHROME_VT3264            0x07
#define UNICHROME_VT3314            0x08
#define UNICHROME_VT3324            0x09
#define UNICHROME_VT3336            0x0a
#define UNICHROME_VT3327            0x0b
#define UNICHROME_VT3293            0x0c
#define UNICHROME_VT3364            0x0d
#define UNICHROME_VT3324M           0x0e
#define UNICHROME_VT3324M2          0x0f
#define UNICHROME_VT3353            0x10


typedef struct _VIDEOHWINFO
{
    unsigned char* pjScreen;                /* Points to base screen address */
    unsigned char* pjVideo;                 /* Start of MM I/O for Video Regs */

    unsigned char* xpjScreen;               /* Points to base screen address for Xserver [2005.6.24 seanzhang]*/
    unsigned char* xpjVideo;                /* Start of MM I/O for Video Regs for Xserver [2005.6.24 seanzhang ]*/

    unsigned char* pjIoBase;                /* Mapped IO port base */
    unsigned long dwVidCtl;
    unsigned long dwHQVCtl;
    unsigned long dwHQVSPCtl_Pitch;         /* HQV Sub-picture control and stride */
    unsigned long dwHQVSPSrcAddr;           /* HQV Sub-picture address */
    unsigned long dwHQVDeinterlaceCtl;      /* Motion adaptive de-interlace control */
    unsigned long dwHQVTCGDCtl;             /* Tile,CSC,Gamma,Deblocking control */
    unsigned long dwVOffset;
    unsigned long dwHQVOffset;              /* if use HQV zoom in we use this value */
    unsigned long dwHQVSrcDataOffsetStart;
    unsigned long dwHQVSrcDataOffsetEnd;   
    unsigned long dwHQVSrcAddr[3];          /* HQV engine source address0:Y,1:U,2:V */
    unsigned long dwHQVDstAddr[3];			/* HQV Dst Buffer for HQV0 */
    unsigned long dwHQV1DstAddr[3];         /* HQV Dst Buffer for HQV1 */
    unsigned long dwVSrcAddr[3];            /* VIDEO engine source address */
    unsigned long dwV3SrcAddr[3];           /* VIDEO_3 engine source address */
    unsigned long dwHQVSrcCountWH;          /* HQV source data line count[H] */
                                            /* and fetch count[W] */
    unsigned long dwFetch;                  /* Video engine fetch count */
    unsigned long dwHQVSrcFetch;
    unsigned long dwHQVSrcLine;
    unsigned long dwHQVSrcPitch;            /* HQV engine source stride */
    unsigned long dwHQVDstPitch;            /* HQV engine destination stride */
    unsigned long dwSrcFetch;
    unsigned long dwSrcLine;
    unsigned long dwVDisplayCount;          /* Video engine display count 0x26c,0x27c */
    unsigned long dwVPitch;                 /* Video engine stride */
    unsigned long dwWinStartXY;
    unsigned long dwWinEndXY;
    unsigned long dwCompose;
    unsigned long dwV1FIFO;
    unsigned long dwV3FIFO;
    unsigned long dwV3PreFIFO;
    unsigned long dwminiCtl;
    unsigned long dwzoomCtl;
    unsigned long dwzoomCtlV1DuoView; 
    unsigned long dwHQVMiniZoomCtl;
    unsigned long dwColorKey;
    unsigned long dwLCDColorKey;
    unsigned long dwChromaKeyLow;
    unsigned long dwChromaKeyHigh;
    unsigned long dwAlphaAddr;
    unsigned long dwVideoVQ_Offset;
    unsigned long dwVideoVQ_Size;
    unsigned long dwVQUseCount;             /* Keep VQ Request Count */
    /*Capture engine */
    unsigned long dwCapCtl;
    unsigned long dwCapSrcAddr[3];          /* Capture engine source address */
    unsigned long dwCapSrcPitch;            /* Capture engine source stride */
    unsigned long dwCapActCount;            /* Capture engine active video count */
    unsigned long dwCap601HoriRange;        /* Capture engine 601 video horizontal range */
    unsigned long dwCap601VertRange;        /* Capture engine 601 video vertical range */
    unsigned long dwCapMiniCtl;

    /*HQV scale control for VT3293*/
    unsigned long dwHQVScaleCtlH;            
    unsigned long dwHQVScaleCtlV; 
    
    unsigned long dw2DVQUseCount;           /*Keep 2D VQ Request Count*/

    unsigned long dwScalDuoV1ZoomCtl;       /*3D scaling special video zoom factor*/
    unsigned long dwScalDuoV3ZoomCtl;
    unsigned long dwScalDuoHQVMiniZoomCtl;
}VIDEOHWINFO,*LPVIDEOHWINFO;

/* Replace FPGAAGPCTL by VIDAGPCTL, and add elements for buffer control. */
typedef struct _VIDAGPCTL
{
    unsigned long * agpLinearStart;         /* agp linear start address */
    unsigned long *lpAGPSt_buffer;          /* Starting addr. of cmd. buffer for PCI burst path. */
    unsigned long *lpAGPCurr_buffer;        /* Current addr. of cmd. buffer for PCI burst path. */
    unsigned long dwAGPSt_Pointer;          /* Starting addr. of requested cmd. buffer shared from 2D/3D AGP cmd. buffer. */
    unsigned long dwAGPCurr_Pointer;        /* Current addr. of requested cmd. buffer shared from 2D/3D AGP cmd. buffer. */
    unsigned long dwAGPCMCount;             /* Command Count. */
    unsigned long dwAGPHeaderType;          /* Header type. */
    /* Is buffer locked? Before adding command data to buffer, we must request buffer size and this flag
     * will be set to 1. That means buffer is locked to add command data. Once command data in buffer are
     * fired, this flag will be cleared to 0.
	 */
    unsigned long dwAGPRequest;
    unsigned long dwAGPH5Reg;               /* Register setting for header type 5. */
    /* Keep buffer type info. of shared AGP cmd. buffer.
     * Valid values:AGPCMDBUF_DISABLE, AGPCMDBUF_RING, AGPCMDBUF_DOUBLE.
	 */
    unsigned long dwVIdeoAGPUseNum;	 
} VIDAGPCTL, *LPVIDAGPCTL;

typedef struct _VIDEOSTATUS
{
    unsigned long   dwVideoFlagMPEG[2];
    unsigned long   dwVideoFlagTV0;
    unsigned long   dwVideoFlagTV1;
    unsigned long   dwVideoFlagSW[2];       /*2 swdevice for decode 2 stream mp1/4 */
    unsigned long   dwAlphaEnabled;		    /* For Alpha blending use */
                                            /* Coz enable alpha should disalbe colorkey */
}VIDEOSTATUS,*LPVIDEOSTATUS;

typedef struct _VIDEOCONTROL
{
    unsigned long   dwVideoFlag[2];         /*2 swdevice for decode 2 stream mp1/4 */
    unsigned long   dwDXVAFlag;
    unsigned long   dwCtlHQVflag;
    unsigned long   dwCtlMPEGFlag[2];
    unsigned long   dwCtlCMFlag;
    unsigned long   dwNTSCFlag;

    unsigned long   dwYUVByPassEnabled;
    unsigned long   dwTVFieldSyncEnabled;
}VIDEOCONTROL,*PVIDEOCONTROL;

typedef struct _VIDDEVICEINFO
{
	unsigned long   dwVendorID;             /* VGA Vendor ID */
    unsigned long   dwDeviceID;             /* VGA Device ID */
    unsigned long   dwRevisionID;           /* VGA Revision ID */
    unsigned long   dwVDevFlag;             /* CTL_UseDualView_CME 0x00000001 */
                                            /* CTL_UseDuoView_CME  0x00000002 */
                                            /* CTL_UseIGA2Path_CME 0x00000004 */
                                            /* CTL_UseExpand_CME   0x00000008 */
                                            /* CTL_UsePanning_CME  0x00000010 */
                                            /* CTL_InDOSFullScreen_CME 0x00000020 */
    unsigned long   dwcxScreen;             /* Visible screen width */
    unsigned long   dwcyScreen;             /* Visible screen height */
    /*Panel Device information */
    unsigned long   dwMaxResX;
    unsigned long   dwMaxResY;

}VIDDEVICEINFO,*PVIDDEVICEINFO;

/* dwVDevFlag,Video Device status */
#define CTL_UseDualView         0x00000001
#define CTL_UseDuoView          0x00000002
#define CTL_UseIGA2Path         0x00000004
#define CTL_UseSIMView          0x00000008
#define CTL_UseExpand           0x00000010
#define CTL_UsePanning          0x00000020

/**************************************************************************
 * OVERLAYRECORD structure keep all overlay infromation					  *
 **************************************************************************/
typedef struct
{
    unsigned char*   pjVideo;               /* Start of MM I/O for Video Regs */
    /* Keep Video Info */
    unsigned long   dwVideoFlag;            /* Video control flag */
    unsigned long   dwVideoOn;              /* For Non-HQV path, video on or not */
    unsigned long   dwOverlayInUseType;     /* For Non-HQV path, Overlay use in which type(Video, Capture, MPEG). */
    unsigned long   dwVideoPatchFlag;       /* Video Patch Control flag */
    unsigned long   dwVideoRegistryFlag;    /* get the video control info from registry. */
    unsigned long   dwFlipFlag;             /* lpFlip->dwFlags. keep DDFlip function flag. */

    DDPIXELFORMAT   OriVideoDPF;            /* keep video pixel format */
    unsigned long   dwStartAddr;            /* keep the Surface Physical Start Address */
    unsigned long   dwOriSrcPitch;          /* keep the Surface pitch */
    unsigned long   dwOriSrcWidth;          /* keep the Surface Width */
    unsigned long   dwOriSrcHeight;         /* keep the surface Height */
    unsigned long   dwDeinterlaceMode;      /* DDOVER_INTERLEAVED or DDOVER_BOB */
    unsigned long   dwWidth;                /* Video Dest RECTL width */
    unsigned long   dwHeight;               /* Video Dest RECTL height */
    OV_RECTL   dwOriSrcRect;                /* keep updateoverlay source RECTL */
    OV_RECTL   dwOriDestRect;               /* keep updateoverlay destination RECTL */

    /* Colorkey related info. */
    unsigned long   dwColorKeyOn;
    unsigned long   dwKeyLow;
    unsigned long   dwKeyHigh;

    /* Chromakey related info. */  
    unsigned long   dwChromaKeyOn;
    int             dwChromaIC;             /* chroma key inverse control */
    unsigned long   dwChromaLow;
    unsigned long   dwChromaHigh;

    /* Keep the upd_video, upd_capture, upd_mpeg function need info,
     * Initial in DDupdateOverlay, please don't set those parameters in other function */
    OV_RECTL   dwOverlaySrcRect;            /* keep calculated updateoverlay source RECTL */
    OV_RECTL   dwOverlayDestRect;           /* keep calculated updateoverlay dest RECTL */
    unsigned long   dwOverlayDestWidth;     /* In SAMM and Duo expand, this parameter = dwWidth * panel_X_size / On screen x resoltion
                                               If no expand, this parameter = dwWidth; */
    unsigned long   dwOverlayDestHeight;    /* In SAMM and Duo expand, this parameter = dwHeight * panel_Y_size / On screen y resoltion
                                               In SIM expand, this parameter = dwHeight+1. If no expand, this paramter = dwHeight */
    OV_RECTL   dwExpandDestRect;            /* If expand. the RECTL need re-calculate
                                               If no expand, the RECTL = dwOriDestRect. */

    /* Keep HQV info */
    unsigned long   dwHQVheapInfo;          /* video memory heap of the HQV buffer */
    unsigned long   dwHQVAllocInit;         /* HQV memory allocate or not. 1:allocate */
    unsigned long   dwHQVon;                /* HQV enable flag */
    unsigned long   dwHQV1on;               /* HQV1 enable flag for DuoView */
    unsigned long   dwHQVInUseType;         /* HQV use in which type(Video, Capture, MPEG, VMR) */
    unsigned long   dwHQVAddr[3];           /* Keep HQV Address */
    unsigned long   dwHQV1Addr[3];          /* Keep HQV1 Address */
    unsigned long   dwHQVWidth;             /* HQV Width */
    unsigned long   dwHQVHeight;            /* HQV Height */
    unsigned long   dwHQVInfo;              /* HQV surface info. In FOURCC, the parameter = FOURCC. */
                                            /* In RGB, this parameter = RGB bit count. */
    unsigned char*  dwHQVLinearAddress;     /* The linear address of HQV buffer */

    /* Keep video info for all Overlay function */
    unsigned long   dwOffset;               /* Video offset or Y offset of plane YUV */
    unsigned long   dwUVoffset;             /* UV offset of plane YUV */
    unsigned long   dwFlipTime;             /* Patch Flip Lock Delay */
    unsigned long   dwFlipTag;              /* Patch Flip Lock Delay */
    unsigned long   dwFetchAlignment;       /* Vdieo Fetch Alignment */
    unsigned long   dwminifyH;              /* Horizontal minify factor */
    unsigned long   dwminifyV;              /* Vertical minify factor */
    unsigned long   dwColorKey;             /* Keep 2D color fill value, only for patch MPEG overlay. */
    unsigned long   dwBandWidth;            /* Keep BandWidth issue info */
    unsigned long   dwOvlBufCount;          /* Keep Overlay surface allocate count. */
    unsigned long   dwMPEGInTVFullScreen;   /* DVD play on TV(NTSC 720*480, PAL 720*480). */
                                            /* We need to disable TV flicker filter. */
    /* Keep MPEG Display Info */
    unsigned long   dwMpegDecoded;          /* MPEG engine has decoded a picture */
    unsigned long   dwFlip;                 /* Find the match */

    void*  pvmHeapCur;
    unsigned long   dwSPOffset;             /* MPEG Subpic start address offset */
    unsigned long   dwSPstartAddr;          /* MPEG Subpic start address */
    unsigned long   dwSPPitch;              /* MPEG Subpic pitch */

    /* for Single overlay surface */
    unsigned long   dwSingleOverlaySurf;    /* Single Overlay surface on */
    DDPIXELFORMAT   DPFOverlaySurf;         /* Keep the Single Ovrelay Surface info */

    unsigned long   dwVideoVQHeapInfo;
    unsigned long   dwAlphaHeapInfo;
    unsigned long   dwMPEGTempHeapInfo;

    /* for VT3259 Video Rotation */
    unsigned long   dwRotSrcheapInfo;
    unsigned long   dwRotDstheapInfo;

    unsigned long   dwHQVFilterManualSelect;
    unsigned long   dwMpegHQVVFilterMode;   /* for Upd_Video and Upd_MPEG */
    unsigned long   dwMpegHQVHFilterMode;   /* for Upd_Video and Upd_MPEG */
    unsigned long   dwCapHQVVFilterMode;    /* for Upd_Captutre */
    unsigned long   dwCapHQVHFilterMode;    /* for Upd_Captutre */
    unsigned long   dwTVHasOptimized_Cap;   /* When TV full screen, disable TV flicker filter */

} OVERLAYRECORD, *LPOVERLAYRECORD;

typedef struct _OVERLAYRECORDSET
{
    OVERLAYRECORD   OverlayRecordMPEG[2];
    OVERLAYRECORD   OverlayRecordSW[2];     /*for 2 sw decode and display*/
}OVERLAYRECORDSET,*LPOVERLAYRECORDSET;


/**************************************************************************
 * structure for surface manager										  *
 **************************************************************************/
struct offrange
{
    unsigned long   StartAddr;
    unsigned long   EndAddr;
    unsigned long   size;
    unsigned char   type;
    unsigned long   ID;
    unsigned char   priority;
    unsigned char   capability;
    unsigned char   status;
    char            nextarray;
};
typedef struct offrange OffMemRange ;

typedef struct
{
    OffMemRange *unused;
    OffMemRange *used;
} ViaOffScrnRec, *ViaOffScrnPtr;


/**************************************************************************
 * H/W mpeg & Capture surface information								  *
 **************************************************************************/
/* Structures for H/W mpeg surface information */
typedef struct _MPGDEVICE
{
    VIAMPGSURFACE  VIAMPGSurface;             /* Pointer to LPVIAMPGSURFACE */
    unsigned char * lpVideoMemIO;             /* Pointer to Video Memory MAP IO */

    /* For MPEG and Subpic start address */
    unsigned char * lpMPEGOverlaySurface[NUM_FRAME_BUFFERS];    /* Max 4 Pointers to MPEG Overlay Surface */
    unsigned long  dwMPEGPhysicalAddr[NUM_FRAME_BUFFERS];  	    /* Max 4 Physical address to MPEG Overlay Surface */
    unsigned long  dwMPEGYPhysicalAddr[NUM_FRAME_BUFFERS];      /* Physical address to MPEG Y Overlay Surface  */
    unsigned long  dwMPEGCbPhysicalAddr[NUM_FRAME_BUFFERS];     /* Physical address to MPEG Cb Overlay Surface */
    unsigned long  dwMPEGCrPhysicalAddr[NUM_FRAME_BUFFERS];     /* Physical address to MPEG Cr Overlay Surface */
    /* for VT3259 */
    unsigned long  dwTempMPEGYPhysicalAddr;			  /* MPEG temp. Physical address for Y */
    unsigned long  dwTempMPEGCbPhysicalAddr;          /* MPEG temp. Physical address for U,V */
    unsigned long  dwTempMPEGCrPhysicalAddr;          /* MPEG temp. Physical address for U,V */
    unsigned long  dwTempMPEGLinearAddr;              /* MPEG temp. Linear address for Y */
    unsigned long  dwMPEGDisplayYPhysicalAddr;        /* MPEG Y physical address for Displaying */
    unsigned long  dwMPEGDisplayCbPhysicalAddr;       /* MPEG U,V physical address for Displaying */

    unsigned long  HWControl;                /* set mpeg engine level : MC, MC_IDCT, MC_IDCT_SLICE		*/
    unsigned long  dwWidth;                  /* MPEG coded_picture_width                                */
    unsigned long  dwHeight;                 /* MPEG coded_picture_height                               */
    unsigned long  dwPitch;                  /* MPEG frame buffer pitch                                 */
    unsigned long  dwPageNum;                /* Frame buffer Number                                     */
    unsigned char  byDeviceType;             /* Device type. Such as DEV_MPEG and DEV_SUBP              */
    unsigned long  gdwSetBufferIndex;        /* Used to assigned buffer pointer in SetOverlayBuffer()   */
    unsigned long  gdwMPGState;              /* MPG states                                              */
    unsigned long  gdwSUBPState;             /* Sub Picture states                                      */
    unsigned long  dwSubpPageNum;            /* SubPicture Frame buffer Number                          */
    unsigned long  dwSUBPPitch;              /* SubPicture Pitch                                        */
    unsigned long  gdwSUBPSrcLeft;           /* SubPicture Position : Left                              */
    unsigned long  gdwSUBPSrcTop;            /* SubPicture Position : Top                               */
    unsigned long  gdwSUBPSrcWidth;          /* SubPicture Source Width                                 */
    unsigned long  gdwSUBPSrcHeight;         /* SubPicture Source Height                                */
    unsigned long  gdwSUBPDisplayIndex;      /* Subpicture Display Index                                */
    unsigned long  gdwMPGSrcWidth;           /* MPEG Source Width                                       */
    unsigned long  gdwMPGSrcHeight;          /* MPEG Source Height                                      */
    unsigned long  gdwMPGDstWidth;           /* MPEG Destination Width                                  */
    unsigned long  gdwMPGDstHeight;          /* MPEG Destination Height                                 */
    unsigned long  gdwMPGDstLeft;            /* MPEG Position : Left                                    */
    unsigned long  gdwMPGDstTop;             /* MPEG Position : Top                                     */
    unsigned long  dwDeinterlaceMode;        /* BOB / WEAVE                                             */
    unsigned long  gdwSUBP_NotVisible;
    unsigned long  dwMPEGDisplayIndex ;      /* Currently display index                     */
    unsigned long  dwHQVAddr[3];             /* Physical address to HQV surface -- CLE_C0   */
    unsigned long  dwHQV1Addr[3];            /* HQV1 Start address */
    unsigned long  dwBitStreamCount;         /* Slice bitstream byte count */
    unsigned char* lpBitStreamAddr;          /* Slice bitstream start address */
    unsigned long  dwSliceTotalByteCount;    /* For Encryption function, we need set this parameter. */
    unsigned long  dwEnableErrorConcealment; /* For MPEG ErrorConcealment                   */
    unsigned long  dwBeginDecode;
    Bool  bInitialized;             		 /* Indicate H/W MPEG engine initialize or not */

    /* for VT3259 */
    unsigned long  dwPictDesc;               /* Picture description register value */

    /* keep DXVA Slice Encryption/Decryption info */
    unsigned long  dwRotationKey;
    unsigned long  dwEncrypKey;
    unsigned long  dwEncryption;

    unsigned long  dwMPEGSignCheckTimes;     /* Record MPEG Signature Check fail times */

    unsigned long  dwDecodeEndCountPhysicalAddr;    /* Record MPEG decode really end. event tag in VQ to record decode end*/
}MPGDEVICE, * LPMPGDEVICE;

/* definition for HWControl */
#define USE_SLICE                 0
#define USE_MC                    1
#define USE_MC_IDCT               2

/* Device Type of MPGDEVICE */
#define DEV_MPEG    0
#define DEV_SUBP    1


/**************************************************************************
 * Top level video structure											  *
 **************************************************************************/
typedef struct _VIDDATA
{
    VIDEOSTATUS     VidStatus;

    VIDEOCONTROL    VidCtl;

    VIDEOHWINFO     Vhwinfo;
    VIDAGPCTL       VidAGPCtl;
    VIDDEVICEINFO   VDevInfo;
    VIAGRAPHICINFO  VIAGraphicInfo;
    VIAVIDCTRL      VideoControl;
    VIDHWDIFFERENCE VideoHWDifference;

    DDLOCK          ddLock;                     /* video device struct */
    SWDEVICE        SWDevice[2];                /* 2 swdevice for decode 2 stream mp1/4 */
    SWDEVICE        SWDevice_rot[2];            /* Structure used to save rotate related data. */	
    MPGDEVICE   	MPGDevice[2];			    /* MPEG H/W related struct */
    ViaOffScrnRec 	MemLayOut; 
    DDPIXELFORMAT   DPFsrcTV;                   /* To indicate which fourcc used for TV0 & TV1  */
    DDPIXELFORMAT   DPFsrc[2];                  /* To indicate which fourcc used for MPEG/SW video */
    DDPIXELFORMAT   DPFsrc_rot;					/* add for video rotation */	
    DDUPDATEOVERLAY UpdateOverlayRec;           /* Keep the updataoverlay record via V4L. */    
    DDUPDATEOVERLAY UpdateOverlayMPEGBackup;    /* For HQVcontrol func. use save original MPEG updateoverlay info.*/
    DDUPDATEOVERLAY UpdateOverlaySWBackup;      /* For HQVcontrol func. use save original SW updateoverlay info.*/
    DDUPDATEOVERLAY UpdateOverlayTV0Backup;     /* For HQVcontrol func. use save original TV0 updateoverlay info.*/
    DDUPDATEOVERLAY UpdateOverlayTV1Backup;     /* For HQVcontrol func. use save original TV1 updateoverlay info.*/
    OV_RECTL    AlphaDestRect;                  /* Alpha blending destination RECTL */
    Bool        NeedToCorrectVideoSize;         /* Indicate if video is resized because of down scaling. */
    Bool        isVideoWidthDownScaling;        /* Indicate if the width of video is resized because of down scaling. */
    Bool        isVideoHeightDownScaling;       /* Indicate if the height of video is resized because of down scaling. */
    OV_RECTL    DownScalingDestRect;            /* Use to record new destination rect after down scaling. */
        
    int     panning_old_x[2];           /* The old offset value in panning mode     */
    int     panning_old_y[2];           /* 0: for 1st screen, 1: for 2nd screen use */
    int     panning_x[2];               /* The offset value in panning mode         */
    int     panning_y[2];               /* 0: for 1st screen, 1: for 2nd screen use */

    unsigned char   Save_3C4_16;
    unsigned char   Save_3C4_17;
    unsigned char   Save_3C4_18;
    unsigned long   dwUseExtendedFIFO;

    Bool   dwRejectOverlay;             /* Flag for decide overlay support or not*/
    Bool   dwVideoEngineSwitch;         /* V1 and V3 switch */

    Bool Address0_Switch;       /* 1: buffer 0 switch to 2
                                   0: buffer 2 switch back to 0 */
    Bool Address1_Switch;       /* 1: buffer 1 switch to 3
                                   0: buffer 3 switch back to 1 */
    Bool V3Address0_Switch;     /* 1: buffer 0 switch to 2
                                   0: buffer 2 switch back to 0 */
    Bool V3Address1_Switch;     /* 1: buffer 1 switch to 3
                                   0: buffer 3 switch back to 1 */
    Bool AUDIO_MUTE;            /* keep tuner's audio status */
    int bSetFirst;              /* check if capture enable already */

	OVERLAYRECORDSET OverlayRecordSet;

	void *          lpWaitHWInfo;
    void *          lpDXVAStatus;
    void *          lpVPEStatus;
    int Use_SMBus;
    unsigned short SMBus_Base;

	unsigned long dwV1, dwV3, dwHQV;	/* store the V1, V3,HQV value, for viaRestoreVideo() use */
	unsigned long dwSaveHQVdstaddr0;    /* store HQV dst addr, for viaRestoreVideo() use */
	unsigned long dwSaveHQVdstaddr1;
	unsigned long dwSaveHQVdstaddr2;

    VMI_INFO_PARAM VMI_Info;            /* keep the handle info. */
    VMI_FLIP_PARAM VMI_Flip;            /* keep flip info. */

    char            video0Path[20];
    char            video1Path[20];
    char            video2Path[20];

	Bool  bVideoOnSND; 	                /*Added for SAMM Mode */
	/* Surface structure */			        
    DDSURFACEDESC DDSurfaceDesc;

    OffMemRange     MemLayOutBufPool[27];
    unsigned char   AddrUnused0;
    unsigned char   AddrUsed0;
}VIDDATA,*PVIDDATA;

typedef struct _PACKETVIDDATA
{
	PVIDDATA pVidData;
}PACKETVIDDATA,*PPACKETVIDDATA;

typedef struct _VIA_I2C
{
    unsigned char slaveAddr;
	unsigned char subAddr;
	unsigned char i2cData;
}VIA_I2C,*PVIA_I2C;


typedef struct _V3SRCADDR
{
    unsigned long V3SrcAddr[3];
} V3SRCADDR, *PV3SRCADDR;

/* Data sturcture for vfControlYUVByPass() */
typedef struct _VIDYUVBYPASS
{
    unsigned long       dwDestWidth;
    unsigned long       dwDestHeight;
    unsigned long       dwIsYUVSource;
    unsigned long       dwShowVideo;
    PVIDDATA    pVidData;
}VIDYUVBYPASS,*PVIDYUVBYPASS;

typedef struct _FLIPHANDLE
{
    unsigned long handle;
    unsigned long DisplayBufferIndex;
} FLIPHANDLE, *PFLIPHANDLE;


#endif  /*end of __VIDSTRUC__ */
