/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __AGPCTL_H__
#define __AGPCTL_H__
#include "VidStruc.h"
#include "via_drm.h"

void dump_agp_cmd(void* pbuf, int size);

/* 3D register */
/* PCI IO Base register */
#define HC_REG_BASE             0x0400
#define HC_REG_EngStatus        0 + HC_REG_BASE
#define HC_3D_ENG_BUSY          0x00000001
#define HC_2D_ENG_BUSY          0x00000002
#define HC_BST_ENG_BUSY         0x00000004
#define HC_CMD_RGTR_BUSY        0x00000080
#define HC_VR_QUEUE_EMPTY       0x00020000

/* Writeable Hardware Status */
/* Transmission IO Space */
#define HC_REG_TRANS_SET        (0x003c + HC_REG_BASE)
#define HC_REG_CR_TRANS         HC_REG_TRANS_SET
#define HC_REG_3D_TRANS         HC_REG_TRANS_SET

#define HC_REG_Hpara0           (0x0040 + HC_REG_BASE)    /* Begin */
#define HC_REG_HparaAF          (0x02fc + HC_REG_BASE)    /* End */

/* Command Header */
#define HC_AGPHeader_MASK       0xFE000000
#define HC_AGPHeader0           0xE0000000
#define HC_AGPHeader1           0xF0000000
#define HC_AGPHeader2           0xF2000000
#define HC_AGPHeader3           0xF4000000
#define HC_AGPHeader4           0xF6000000

/* AGP Command Setting */
#define HC_SubA_HAGPBstL        0x60000000
#define HC_SubA_HAGPBendL       0x61000000
#define HC_SubA_HAGPCMNT        0x62000000
#define HC_SubA_HAGPBpL         0x63000000
#define HC_SubA_HAGPBpH         0x64000000

#define HC_ParaType_PreCR       0x00100000

/* HC_SubA_HAGPCMNT             0x62 */
#define HC_HAGPCMNT             0x00800000
#define HC_HCmdErrClr           0x00400000
#define HC_HAGPBendH_MASK       0x0000ff00
#define HC_HAGPBstH_MASK        0x000000ff
#define HC_HAGPBendH_SHIFT      8
#define HC_HAGPBstH_SHIFT       0

/* HC_SubA_HAGPBpL              0x63 */
#define HC_HAGPBpL_MASK         0x00fffffc
#define HC_HAGPBpID_MASK        0x00000003
#define HC_HAGPBpID_PAUSE       0x00000000
#define HC_HAGPBpID_JUMP        0x00000001
#define HC_HAGPBpID_STOP        0x00000002

/* HC_SubA_HAGPBpH              0x64 */
#define HC_HAGPBpH_MASK         0x00ffffff

/* AGP Cycle Status */
#define HC_REG_AGP_CUR          (0x0018 + HC_REG_BASE)
#define HC_REG_AGP_CycPause     (0x001c + HC_REG_BASE)
#define HC_REG_AGP_CYC          (0x0048 + HC_REG_BASE)

#define HC_REG_AGP_Pause_Mask   0x80000000
#define HC_REG_AGP_Idle_Mask    0x40000000

#define MPEG_AGP_CMDBUF_SIZE    0x400000   /* 2M * 2 */
#define AGP_CMDBUF_SIZE         0x400000
#define AGP_Header4_MPEG        0x00000001

/*  Driver Running Status */
#define WKN_GDI_USED_AGP        0x00000010
#define WKN_2D_USED_AGP         0x00000001
#define WKN_D3D_USED_AGP        0x00000002
#define WKN_ICD_USED_AGP        0x00000004
#define WKN_MPEG_USED_AGP       0x00000008
#define WKN_AGP_USED            0x0000000F
#define WKN_VIDEO_USED_AGP      0x00000020

#define WKN_D3D_RUNNING         0x00000100
#define WKN_ICD_RUNNING         0x00000200
#define WKN_GDI_RUNNING         0x00000400

/* D3D & ICD AGP control for ICDInfo */
#define WKN_AGP_D3DWAIT             0x00000001      /* D3D Set Wait to ICD to wait */
#define WKN_AGP_ICDRING             0x00000002      /* ICD is using Ring */
#define WKN_AGP_ICDDOUBLE           0x00000004      /* ICD is using Double */
#define WKN_AGP_REQUSET_DOUBLE      0x00010000      /* request agp command buffer to be double buffer */
#define WKN_AGP_REQUEST_CUSTOM      0x00020000      /* No D3D use agp command */


/* Use Buffer Type for WKNAGPCmdBufControl.dwCmdBufType */
#define AGPCMDBUF_DISABLE           0x0
#define AGPCMDBUF_RING              0x1
#define AGPCMDBUF_DOUBLE            0x2


#define Low_Last_3Byte_Mask        0x00FFFFFF
#define High_Byte_Mask             0xFF000000


/* Definition of WKNAGPCONTROL.dwFlag */
#define AGP_DD_INITIALIZED         0x00000001
#define AGP_MPEG_INITIALIZED       0x00000002
#define AGP_DD_USE_AGP_CMD         0x00000004

/* Definition of WKNAGPCMDBUFCONTROL.dwFlag */
#define AGPSTATUS_ENABLED          0x00000001
#define AGPSTATUS_CMDBUF_ALLOCATED 0x00000010
#define AGPSTATUS_CMDBUF_OVERFLOW  0x00000020
#define AGPSTATUS_CMDBUF_LOCK      0x00000040
#define AGPSTATUS_BUFTYPE_CHANGE   0x00000100
#define AGPSTATUS_BUFTYPE_WAITSWAP 0x00000200
#define AGPSTATUS_BUFTYPE_SETTED   0x00000400
#define AGPSTATUS_RING_RESET       0x00001000

#define AGPSTATUS_RING_FrameEnd    0x00002000

#define HWAGP_GETBUSY           (*((unsigned long volatile *)(pAGP->dwRegBase+HC_REG_EngStatus)) \
                                    & (HC_CMD_RGTR_BUSY | HC_2D_ENG_BUSY | HC_3D_ENG_BUSY))


/* Hardware Capabilities */
#define CAPS_AGPCMD_RING_3205                        0x00000001
#define CAPS_AGPCMD_RING_3204                        0x00000002
#define CAPS_AGPCMD_RING_3259                        0x00000004

/* AGP current read back data address */
#define VT3205_REG_AGP_CUR      (0x00c + HC_REG_BASE)

#define PushInBuf_H6(pVidData,dwIndex,dwData)  \
{ \
    if(pVidData->VidAGPCtl.dwAGPCMCount +2 >= 0x10000)\
    {\
        pVidData->VidAGPCtl.dwAGPCurr_Pointer=((unsigned long)lpAGPbuffer)-gdwAGPBaseVirtual;\
        vfFireAGPcommand(pVidData);\
        vfRequestAGPbuffer(pVidData,VIDEO_AGPHeader6,0);\
        lpAGPbuffer= (unsigned long *)(pVidData->VidAGPCtl.dwAGPCurr_Pointer+gdwAGPBaseVirtual);\
    }\
    *lpAGPbuffer++= dwIndex; \
    *lpAGPbuffer++= dwData; \
    pVidData->VidAGPCtl.dwAGPCMCount++;\
 }


typedef void (*FPSWAPAGPCOMMAND)(LPWKNAGPCMDBUFCONTROL pAGP, PVIDDATA pVidData);

#define UMA_HW_WAIT_3D_IDLE             0x80000000
#define UMA_HW_WAIT_2D_IDLE             0x40000000
#define INV_CR_WAIT_2D_IDLE             0x00400000
#define INV_CR_WAIT_3D_IDLE             0x00800000

#define FLAG_WAIT_2D_IDLE   0x1
#define FLAG_WAIT_3D_IDLE   0x2

/*The following macro is for H2*/
#define HC_DUMMY		0xCCCCCCCC
#define HC_HEADER2		0xF210F110

#define HC_ParaType_NotTex      0x0001

#define ADD2DCmd_H2(pCmd, dwAddr, dwCmd)                  \
{                                                           \
    *(pCmd)++ = ( ((dwAddr) >> 2) | 0xF0000000 );                      \
    *(pCmd)++ = (dwCmd);                                    \
}

#define Add2DHdr4Cmd_H2(pCmd, dwCount, id)                                                 \
{                                                                                          \
    *(pCmd)++ = ((0xF6000000) | (((dwCount) << 9) & 0x1FFFE00) | ((id) & 0x1FF));            \
}

#define ALIGN_H2_CMD(_buf, _size)  do{   \
    switch (_size & 0x7){                                                       \
        case 2:                                                                             \
            _buf[_size++] = HC_HEADER2;                                     \
            _buf[_size++] = (HC_ParaType_NotTex << 16) ;            \
            _buf[_size++] = HC_DUMMY;                                           \
            _buf[_size++] = HC_DUMMY;                                       \
            _buf[_size++] = HC_DUMMY;                                           \
            _buf[_size++] = HC_DUMMY;                                               \
            break;                                                                          \
        case 4:                                                                         \
            _buf[_size++] = HC_HEADER2;                                 \
            _buf[_size++] = (HC_ParaType_NotTex << 16) ;            \
            _buf[_size++] = HC_DUMMY;                                       \
            _buf[_size++] = HC_DUMMY;                                       \
            break;                                                                          \
        case 6:                                                                             \
             _buf[_size++] = HC_HEADER2;                                    \
            _buf[_size++] = (HC_ParaType_NotTex << 16) ;                \
            _buf[_size++] = HC_DUMMY;                                           \
            _buf[_size++] = HC_DUMMY;                                           \
            _buf[_size++] = HC_DUMMY;                                           \
            _buf[_size++] = HC_DUMMY;                                           \
            _buf[_size++] = HC_DUMMY;                                           \
            _buf[_size++] = HC_DUMMY;                                           \
            _buf[_size++] = HC_DUMMY;                                           \
            _buf[_size++] = HC_DUMMY;                                            \
            break;                                                                              \
        case 0:                                                                                 \
            break;                                                                               \
    }                                                                                                 \
    }while(0)

#define ALIGN_H2_DATA(_buf, _size)                                 \
do {                                                               \
    if(_size & 0x1)                                                \
    {                                                              \
        _buf[_size + 1] = _buf[_size];                             \
        _size++;                                                   \
    }                                                              \
}while(0)

#define FLUSH_DMA_H2(pVia, _buf, _size)               \
do{                                                                         \
   int ret;                                                                     \
   drm_via_cmdbuffer_t bufI;                                       \
   bufI.size=_size;                                                      \
   bufI.buf= (char *)_buf;                                                        \
   do {                                                                         \
        ret = drmCommandWrite(pVia->drmFD, DRM_VIA_CMDBUFFER,       \
        &bufI, sizeof(bufI));                                                                       \
    } while (ret == -EAGAIN);                                                               \
}while(0)                                                   

/*The following macro is for INV*/
/* NULL Command*/
extern DWORD NULL_COMMAND_INV[];

#define INV_HSWFlag_ADDR_ENCODE(x)   0xCC000000

#define INV_AGPHeader0              0xFE000000
#define INV_AGPHeader1              0xFE010000
#define INV_AGPHeader4              0xFE040000
#define H5_HC_DUMMY		0xCC000000

/* Transmission IO Space*/
#define INV_REG_23D_WAIT            0x326C
/*INV CR wait command*/
#define INV_CR_WAIT_ENGINE_IDLE             0x0000006C

#define ADDCmdHeader4_INVI(pCmd, dwAddr, dwCount, id)           \
{                                                               \
    while ( ((unsigned long)(pCmd)) & 0xF )                         \
    {                                                                                                    \
        *(pCmd)++ = NULL_COMMAND_INV[((unsigned long)(pCmd)) & 0x3];\
    }                                                                                                       \
    *(pCmd)++ = INV_AGPHeader4 | (dwAddr);                      \
    *(pCmd)++ = (dwCount);                                      \
    *(pCmd)++ = (id);                                           \
    *(pCmd)++ = 0;                                              \
}

#define ADDCmdHeader0_INVI(pCmd, dwCount);                      \
{                                                               \
    while ( ((unsigned long)(pCmd)) & 0xF )                         \
    {                                                           \
        *(pCmd)++ = NULL_COMMAND_INV[((unsigned long)(pCmd)) & 0x3];\
    }                                                                                                   \
    *(pCmd)++ = INV_AGPHeader0;                                 \
    *(pCmd)++ = (dwCount);                                      \
    *(pCmd)++ = 0;                                              \
    *(pCmd)++ = (DWORD)INV_HSWFlag_ADDR_ENCODE(pCmd);           \
}

#define ADDCmdHeader1_INVI(pCmd, dwCount, dwAddr);              \
{                                                               \
    while ( ((unsigned long)(pCmd)) & 0xF )                     \
    {                                                           \
        *(pCmd)++ = H5_HC_DUMMY;                                \
    }                                                           \
    *(pCmd)++ = INV_AGPHeader1 | (dwAddr);                      \
    *(pCmd)++ = (dwCount);                                      \
    *(pCmd)++ = 0;                                              \
    *(pCmd)++ = (DWORD)INV_HSWFlag_ADDR_ENCODE(pCmd);           \
}

#define ADD2DCmd_INVI(pCmd, dwAddr, dwCmd)                  \
{                                                           \
    *(pCmd)++ = (dwAddr);                                   \
    *(pCmd)++ = (dwCmd);                                    \
}


#define FLUSH_DMA_INV(pVia, buf, size)  \
do{                                                                                        \
     drm_s3g_flush_t bufI;                                                           \
    int ret;                                                                                   \
    bufI.dma_cmd_type = flush_dma_buffer;                                    \
    bufI.cmd_size = size;                                                               \
    bufI.usermode_dma_buf = buf;                                                    \
    do {                                                                                            \
        ret = drmCommandWrite(pVia->drmFD, DRM_S3G_FLUSH,       \
            &bufI, sizeof(bufI));                                                               \
    } while (ret == -EAGAIN);                                                               \
}while(0)

#define  FLUSE_AGP_CMD(pVia, IsH2, buf, size)                                                \
do{                                                                                                                 \
    unsigned int _n = size;                                                                  \
    volatile unsigned int*_pb = buf;                                                                 \
    if(!(pVia->IsPCI)){                                                                                     \
        if(IsH2)                                                                                                      \
        {                                                                                                                \
            ALIGN_H2_CMD(_pb, _n);                                                     \
            FLUSH_DMA_H2(pVia, buf, 4*_n);                                               \
        }                                                                                                               \
        else                                                                                                        \
        {                                                                                                               \
            while(_n&3){                                                                                        \
                _pb[_n++] = H5_HC_DUMMY;                                                          \
            }                                                                                                       \
            FLUSH_DMA_INV(pVia, buf, _n);                                                     \
        }                                                                                                             \
    }                                                                                                                   \
}while(0)
#endif
