/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _VIA_H
#define _VIA_H

#ifdef XFree86Server
#include "xf86_ansic.h" /* for Bool */
#else
#include "via_types.h"  /* for Bool */
#endif

/* Macros for different codes in V4L & XServer */
#ifdef XFree86Server

#define VIDEO_CLIPCOUNT     REGION_NUM_RECTS(clipBoxes)

#endif /* XFree86Server */


/* Video status flag (VideoFlag) */
#define VIDEO_HIDE                          0x00000000  /*Video off*/
#define VIDEO_SHOW                          0x80000000  /*Video on*/
/* for VT3259 */
#define VIDEO_VQ_INUSE                      0x40000000  /* Video is used with Virtural Queue */
#define VIDEO_AGP_INUSE                     0x20000000  /* Video is used with AGP command */
#define VIDEO_ACTIVE                        0x10000000  /*Video active*/
#define VIDEO_MPEG_INUSE                    0x08000000  /*Video is used with MPEG */
#define VIDEO_HQV_INUSE                     0x04000000  /*Video is used with HQV*/
#define VIDEO_CAPTURE0_INUSE                0x02000000  /*Video is used with CAPTURE 0*/
#define VIDEO_CAPTURE1_INUSE                0x00000000  /*Video is used with CAPTURE 1*/
#define VIDEO_1_INUSE                       0x01000000  /*Video 1 is used with software flip*/
#define VIDEO_3_INUSE                       0x00000000  /*Video 3 is used with software flip*/
#define VIDEO_YUV_BYPASS                    0x00800000  /*definition for YVU bypass */
#define VIDEO_ON                            0x00100000
#define SECOND_MPEG                         0x00200000
#define VIDEO_1_ON_SND                      0x00400000  /*for duo view*/

#define VIDEO_ON_TOP                        0x00080000  /*[19]: 1 toggle, video on top of another in the two stream case, 0: maintain the default status*/
#define VIDEO_STAY_OLD                      0x00000800  /*[11]: 1 maintain old relative position toanother video*/
#define MPEG_USE_V1                         0x00010000  /*[16] : 1:MPEG use V1, 0:MPEG use V3*/
#define MPEG_USE_V3                         0x00000000  /*[16] : 1:MPEG use V1, 0:MPEG use V3*/
#define MPEG_USE_HQV                        0x00020000  /*[17] : 1:MPEG use HQV,0:MPEG not use HQV*/
#define MPEG_USE_HW_FLIP                    0x00040000  /*[18] : 1:MPEG use H/W flip,0:MPEG use S/W flip*/
#define MPEG_USE_SW_FLIP                    0x00000000  /*[18] : 1:MPEG use H/W flip,0:MPEG use S/W flip*/
#define CAP0_USE_HQV                        0x00002000  /*[13] : 1:Capture 0 use HQV,0:Capture 0 not use HQV*/
#define CAP1_USE_HQV                        0x00040000  /*[ 9] : 1:Capture 1 use HQV,0:Capture 1 not use HQV*/
#define SW_USE_HQV                          0x00040000  /*[ 5] : 1:Capture 1 use HQV,0:Capture 1 not use HQV*/

#define CAP0_ON_SND                         0x00000001
#define CAP1_ON_SND                         0x00000002
#define MPEG_ON_SND                         0x00000004
#define SW_ON_SND                           0x00000008


     

/*****************************************************************************
 *     CLE HQV Control status(lpOverlayRecord->dwHQVon)                      *
 *                                                                           *
 *****************************************************************************/
#define HQV_USE_ENABLE                      0x00000001  /* HQV is in USE */
#define HQV_USE_DISABLE                     0x00000000  /* HQV is not USE */


/*****************************************************************************
 *     CLE HQV USE Control status(lpOverlayRecord->dwHQVInUseType)           *
 *                                                                           *
 *****************************************************************************/
#define HQV_IN_VIDEO                        0x00000001  /* Video overlay use HQV */
#define HQV_IN_CAPTURE                      0x00000002  /* Capture overlay use HQV */
#define HQV_IN_MPEG                         0x00000004  /* MPEG overlay use HQV */
#define HQV_IN_VMR                          0x00000008  /* VMR Deinterlace use HQV */

/*****************************************************************************
 *     CLE HQV USE Control status(lpOverlayRecord->dwOverlayInUseType)       *
 *                                                                           *
 *****************************************************************************/
#define Overlay_IN_VIDEO                    0x00000001  /* Video use overlay, not use HQV */
#define Overlay_IN_CAPTURE                  0x00000002  /* Capture use overlay,not use HQV */
#define Overlay_IN_MPEG                     0x00000004  /* MPEG use overlay, out use HQV */

/*for lpVIAGraphicInfo->duoview*/
#define DUOVIEW_ENABLED                     0x0001
#define DUOVIEW_V3_ON_IGA2                  0x0002
#define DUOVIEW_VIDEOOPTION                 0x8000
#define DUOVIEW_VIDEOONTV                   0x4000
#define DUOVIEW_VIDEOONDVI                  0x2000
#define DUOVIEW_VIDEOONLCD                  0x1000
#define DUOVIEW_VIDEOONCRT                  0x0800
/* H/W registers for Video Engine */

/* bus master */
#define PCI_MASTER_ENABLE                   0x01
#define PCI_MASTER_SCATTER                  0x00
#define PCI_MASTER_SINGLE                   0x02
#define PCI_MASTER_GUI                      0x00
#define PCI_MASTER_VIDEO                    0x04
#define PCI_MASTER_INPUT                    0x00
#define PCI_MASTER_OUTPUT                   0x08

/* 440 */
#define GE_TRANS_SET                        0x43c   /* Transmission Setting */
#define GE_Hpara0                           0x440   /* Transmission Space */

/* video registers */
#define V_OFFSET                            0x200   /* offset from 2D Engine register */
#define V_FLAGS                             0x200
#define V_CAP_STATUS                        0x204
#define V_FLIP_STATUS                       0x204
#define V_ALPHA_WIN_START                   0x208
#define V_Vertical_Count                    0x208
#define V_ALPHA_WIN_END                     0x20C
#define V_ALPHA_CONTROL                     0x210
#define V_CRT_STARTADDR                     0x214
#define V_CRT_STARTADDR_2                   0x218
#define V_ALPHA_STRIDE                      0x21C
#define V_COLOR_KEY                         0x220
#define V_ALPHA_STARTADDR                   0x224
#define V_CHROMAKEY_LOW                     0x228
#define V_CHROMAKEY_HIGH                    0x22C
#define V1_CONTROL                          0x230
#define V12_QWORD_PER_LINE                  0x234
#define V1_STARTADDR1                       0x238
#define V1_STRIDE                           0x23C
#define V1_WIN_START_Y                      0x240
#define V1_WIN_START_X                      0x242
#define V1_WIN_END_Y                        0x244
#define V1_WIN_END_X                        0x246
#define V1_STARTADDR2                       0x248
#define V1_ZOOM_CONTROL                     0x24C
#define V1_MINI_CONTROL                     0x250
#define V1_STARTADDR0                       0x254
#define V_FIFO_CONTROL                      0x258
#define V1_STARTADDR3                       0x25C
#define SND_COLOR_KEY                       0x264
#define ALPHA_V3_PREFIFO_CONTROL            0x268
#define V1_SOURCE_HEIGHT                    0x26C
#define V_DISPLAY_TEMP                      0x274  
#define ALPHA_V3_FIFO_CONTROL               0x278
#define V3_SOURCE_WIDTH                     0x27C
#define V3_COLOR_KEY                        0x280
#define ColorSpaceReg_1                     0x284
#define ColorSpaceReg_2                     0x288
#define V1_STARTADDR0_U                     0x28C
#define V1_OPAQUE_CONTROL                   0x290
#define V3_OPAQUE_CONTROL                   0x294 
#define V_COMPOSE_MODE                      0x298
#define V3_STARTADDR_2                      0x29C
#define V3_CONTROL                          0x2A0
#define V3_STARTADDR_0                      0x2A4
#define V3_STARTADDR_1                      0x2A8
#define V3_STRIDE                           0x2AC
#define V3_WIN_START_Y                      0x2B0
#define V3_WIN_START_X                      0x2B2
#define V3_WIN_END_Y                        0x2B4
#define V3_WIN_END_X                        0x2B6
#define V3_ALPHA_FETCH_COUNT                0x2B8
#define V3_ZOOM_CONTROL                     0x2BC
#define V3_MINI_CONTROL                     0x2C0
#define V3_ColorSpaceReg_1                  0x2C4
#define V3_ColorSpaceReg_2                  0x2C8
#define V3_DISPLAY_TEMP                     0x2CC
#define V1_STARTADDR1_U                     0x2E4
#define V1_STARTADDR2_U                     0x2E8
#define V1_STARTADDR3_U                     0x2EC
#define V1_STARTADDR0_V                     0x2F0
#define V1_STARTADDR1_V                     0x2F4
#define V1_STARTADDR2_V                     0x2F8
#define V1_STARTADDR3_V                     0x2FC
/*CN400 and older Hardware Icon engine register*/
#define HI_POSSTART                         0x208
#define HI_CENTEROFFSET                     0x20C
#define HI_FBOFFSET                         0x224
#define HI_CONTROL                          0x260
#define HI_TRANSPARENT_COLOR                0x270
#define HI_INVTCOLOR                        0x274
/* VT3324 primary Hardware Icon engine register */
#define PRIM_HI_POSEND                      0x290
#define V327_HI_INVTCOLOR                   0x2E4
#define PRIM_HI_FIFO                        0x2E8
#define PRIM_HI_TRANSCOLOR                  0x2EC
#define PRIM_HI_CTRL                        0x2F0
#define PRIM_HI_FBOFFSET                    0x2F4
#define PRIM_HI_POSSTART                    0x2F8
#define PRIM_HI_CENTEROFFSET                0x2FC
#define PRIM_HI_INVTCOLOR                   0x120C

/* Video Capture Engine Registers 
 * Capture Port 0 */
#define CAPTURE0_CONTROL                    0x310
#define CAPTURE0_H_RANGE                    0x314
#define CAPTURE0_V_RANGE                    0x318
#define CAPTURE0_SCAL_CONTROL               0x31C
#define CAPTURE0_STARTADDR0                 0x340
#define CAPTURE0_STARTADDR1                 0x344
#define CAPTURE0_STARTADDR2                 0x348
#define CAPTURE0_ACT_VIDEO_STRIDE           0x350


/* Capture Port 1 */
#define CAPTURE1_CONTROL                    0x354
#define CAPTURE1_STARTADDR0                 0x384
#define CAPTURE1_STARTADDR1                 0x388
#define CAPTURE1_ACT_VIDEO_STRIDE           0x38C


/* for VT3324 */
#define REG_Capture1_INDEX                  0x00001000

/* Add new HQV Registers for VT3353: */
#define HQV_SRC_DATA_OFFSET_CONTROL1        0x380
#define HQV_SRC_DATA_OFFSET_CONTROL2        0x384
#define HQV_SRC_DATA_OFFSET_CONTROL3        0x388
#define HQV_SRC_DATA_OFFSET_CONTROL4        0x38C
#define HQV_HW_TUNING_PERFORMANCE           0x390
#define HQV_EXTENDED_CONTROL                0x394
#define HQV_STATIC_RECORD_FB_STARTADDR      0x398
#define HQV_STATIC_RECORD_FB_STRIDE         0x39C
#define HQV_COLOR_ADJUSTMENT_CONTROL1       0x3A0
#define HQV_COLOR_ADJUSTMENT_CONTROL2       0x3A4
#define HQV_COLOR_ADJUSTMENT_CONTROL3       0x3A8
#define HQV_COLOR_ADJUSTMENT_CONTROL5       0x3AC

/* HQV scale factor register for VT3293*/
#define HQV_H_SCALE_CONTORL                 0x3B0
#define HQV_V_SCALE_CONTORL                 0x3B4

/* Add new HQV Registers for VT3353: */
#define HQV_DEFAULT_VIDEO_COLOR             0x3B8

/* SUBPICTURE */
#define SUBP_CONTROL_STRIDE                 0x3C0
#define SUBP_STARTADDR                      0x3C4
#define RAM_TABLE_CONTROL                   0x3C8
#define RAM_TABLE_READ                      0x3CC
#define DEBOLCKING_FACTOR                   0x3BC
/* HQV Registers */
#define HQV_CONTROL                         0x3D0
#define HQV_SRC_STARTADDR_Y                 0x3D4
#define HQV_SRC_STARTADDR_U                 0x3D8
#define HQV_SRC_STARTADDR_V                 0x3DC
#define HQV_SRC_FETCH_LINE                  0x3E0
#define HQV_FILTER_CONTROL                  0x3E4
#define HQV_MINI_CONTROL                    0x3E8
#define HQV_DST_STARTADDR0                  0x3EC
#define HQV_DST_STARTADDR1                  0x3F0
#define HQV_DST_STRIDE                      0x3F4
#define HQV_SRC_STRIDE                      0x3F8
#define HQV_DST_STARTADDR2                  0x3FC  /* HQV_DST_STARTADDR2_32045, HQV_DST_STARTADDR2_3123C0 */

/* HQV1 Registers */
#define HQV1_CONTROL                        0x13D0


/* PCI DMA Registers */
#define PCI_DMA_MAR0                        0xE40   /* Memory Address Register of Channel 0 */
#define PCI_DMA_DAR0                        0xE44   /* Device Address Register of Channel 0 */
#define PCI_DMA_BCR0                        0xE48   /* Byte Count Register of Channel 0 */
#define PCI_DMA_DPR0                        0xE4C   /* Descriptor Pointer Register of Channel 0 */
#define PCI_DMA_MAR1                        0xE50   /* Memory Address Register of Channel 1 */
#define PCI_DMA_DAR1                        0xE54   /* Device Address Register of Channel 1 */
#define PCI_DMA_BCR1                        0xE58   /* Byte Count Register of Channel 1 */
#define PCI_DMA_DPR1                        0xE5C   /* Descriptor Pointer Register of Channel 1 */
#define PCI_DMA_MAR3                        0xE70   /* Descriptor Pointer Register of Channel 3 */
#define PCI_DMA_DAR3                        0xE74   /* Descriptor Pointer Register of Channel 3 */
#define PCI_DMA_BCR3                        0xE78   /* Descriptor Pointer Register of Channel 3 */
#define PCI_DMA_DPR3                        0xE7C   /* Descriptor Pointer Register of Channel 3 */
#define PCI_DMA_MR0                         0xE80   /* Mode Register of Channel 0 */
#define PCI_DMA_MR1                         0xE84   /* Mode Register of Channel 1 */
#define PCI_DMA_MR3                         0xE8C   /* Mode Register of Channel 3 */
#define PCI_DMA_CSR0                        0xE90   /* Command/Status Register of Channel 0 */
#define PCI_DMA_CSR1                        0xE94   /* Command/Status Register of Channel 1 */
#define PCI_DMA_CSR3                        0xE9C   /* Command/Status Register of Channel 3 */
#define PCI_DMA_PTR                         0xEA0   /* Priority Type Register */



#define PCI_DMA_CSR1_VT3353                 0xE0C   /* Command/Status Register of Channel 1 */
#define PCI_DMA_CSR3_VT3353                 0xE1C   /* Command/Status Register of Channel 3 */
#define PCI_DMA_MR1_VT3353                  0xE08   /* Mode Register of Channel 1 */
#define PCI_DMA_MR3_VT3353                  0xE18   /* Mode Register of Channel 3 */
#define PCI_DMA_DPRL1_VT3353                0xE54   /* Descriptor Pointer Low Register of Channel 1 */
#define PCI_DMA_DPRL3_VT3353                0xE94   /* Descriptor Pointer Low Register of Channel 3 */
#define PCI_DMA_DPRH1_VT3353                0xE58   /* Descriptor Pointer High Register of Channel 1 */
#define PCI_DMA_DPRH3_VT3353                0xE98   /* Descriptor Pointer High Register of Channel 3 */



#define DMA_PAGE_SIZE                       0x1000  /* Default: 4K size */

/* Define for DMA engine */
#define DMA_END_OF_CHAIN                    0x00000002
#define DMA_USE_CHAINING_MODE               0x00000001
#define DMA_TRANSFER_DONE_INT_ENABLE        0x00000002
#define DMA_DESCRIPTOR_DONE_INT_ENABLE      0x00000004
#define DMA_TRANSFER_DONE                   0x00000008
#define DMA_DESCRIPTOR_DONE_CLEAR           0x00000010
#define DMA_ENABLE                          0x00000001
#define DMA_TRANSFER_START                  0x00000002
#define DMA_TRANSFER_DONE_CLEAR             DMA_TRANSFER_DONE



/* For Deblocking  */
#define CTL_HQV_DEBLOCKING_ENABLE           0x08000000
#define CTL_HQV_DEBLOCKING_FACTOR0          0x50000000
#define CTL_HQV_DEBLOCKING_FACTOR1          0x30000000
#define CTL_HQV_DEBLOCKING_FACTOR2          0x98000000
/* For 3324 deblocking*/
#define VAL_HQV_DEBLOCKING_FACTOR0          0x0000000A 
#define VAL_HQV_DEBLOCKING_FACTOR1          0x000000C0
#define VAL_HQV_DEBLOCKING_FACTOR2          0x00004C00


/* Video command definition */
#define INT_EN                              (1<<31) // interrupt enable
#define DVP1_SENSE_INT_EN                   (1<<30) // for AD9389 hotplug interrupt on vt3353
#define CAP0_VBI_END_INT_EN                 (1<<29)
#define CAP0_ACTIVE_END_INT_EN              (1<<28)
#define DVP1_SENSE_INT_STS                  (1<<27) // for AD9389 hotplug interrupt on vt3353
#define CAP1_VBI_END_INT_EN                 (1<<25)   
#define HQV0_ACTIVE_END_INT_EN              (1<<25) // After CN400 project
#define CAP1_ACTIVE_END_INT_EN              (1<<24)
#define DMA1_TRANS_END_INT_EN               (1<<23)
#define DMA1_ENTRY_END_INT_EN               (1<<22)
#define DMA0_TRANS_END_INT_EN               (1<<21)
#define DMA0_ENTRY_END_INT_EN               (1<<20)
#define IGA1_VSYNC_INT_EN                   (1<<19)
#define MPEG_MC_END_INT_EN                  (1<<18)
#define IGA2_VSYNC_INT_EN                   (1<<17)
#define DVO0_SENSE_INT_EN                   (1<<16)
#define TMDS_SENSE_INT_EN                   (1<<16)
#define IGA2_VSYNC_INT_STS                  (1<<15) // IGA2 Vsync interrupt status
#define CAP0_VBI_END_INT_STS                (1<<13)
#define CAP0_ACTIVE_END_INT_STS             (1<<12)
#define HQV1_ACTIVE_END_INT_EN              (1<<11) // After CN400 project
#define HQV1_ACTIVE_END_INT_STS             (1<<10) // After CN400 project
#define CAP1_VBI_END_INT_STS                (1<<9)
#define HQV0_ACTIVE_END_INT_STS             (1<<9)  // After CN400 project
#define CAP1_ACTIVE_END_INT_STS             (1<<8)
#define DMA1_TRANS_END_INT_STS              (1<<7)
#define DMA1_ENTRY_END_INT_STS              (1<<6)
#define DMA0_TRANS_END_INT_STS              (1<<5)
#define DMA0_ENTRY_END_INT_STS              (1<<4)
#define IGA1_VSYNC_INT_STS                  (1<<3)
#define MPEG_MC_END_INT_STS                 (1<<2)
#define IGA1_VBLANK_STS                     (1<<1)
#define TMDS_SENSE_INT_STS                  1
 
 
/* #define V_FLIP_STATUS           0x204 */
#define V3_STARTADDR_UPDSTATUS              0x00000008

/* #define V_ALPHA_CONTROL         0x210 */
#define ALPHA_WIN_EXPIRENUMBER_4            0x00040000
#define ALPHA_WIN_CONSTANT_FACTOR_4         0x00004000
#define ALPHA_WIN_CONSTANT_FACTOR_12        0x0000c000
#define ALPHA_WIN_BLENDING_CONSTANT         0x00000000
#define ALPHA_WIN_BLENDING_ALPHA            0x00000001
#define ALPHA_WIN_BLENDING_GRAPHIC          0x00000002
#define ALPHA_WIN_PREFIFO_THRESHOLD_12      0x000c0000
#define ALPHA_WIN_FIFO_THRESHOLD_8          0x000c0000
#define ALPHA_WIN_FIFO_DEPTH_16             0x00100000

/* V_CHROMAKEY_LOW         0x228 */
#define V_CHROMAKEY_V3                      0x80000000

/* V1_CONTROL                   0x230 */
#define V1_ENABLE               0x00000001
#define V1_FULL_SCREEN          0x00000002
#define V1_YUV422               0x00000000
#define V1_RGB32                0x00000004
#define V1_RGB15                0x00000008
#define V1_RGB16                0x0000000C
#define V1_YCbCr420             0x00000010
#define V1_COLORSPACE_SIGN      0x00000080
#define V1_SRC_IS_FIELD_PIC     0x00000200
#define V1_SRC_IS_FRAME_PIC     0x00000000
#define V1_BOB_ENABLE           0x00400000
#define V1_FIELD_BASE           0x00000000
#define V1_FRAME_BASE           0x01000000
#define V1_SWAP_SW              0x00000000
#define V1_SWAP_HW_HQV          0x02000000
#define V1_SWAP_HW_CAPTURE      0x04000000
#define V1_SWAP_HW_MC           0x06000000
#define V1_EXPIRE_NUM           0x00050000
#define V1_EXPIRE_NUM_A         0x000a0000
#define V1_EXPIRE_NUM_F         0x000f0000 
#define V1_FIFO_EXTENDED        0x00200000
#define V1_ON_CRT               0x00000000
#define V1_ON_SND_DISPLAY       0x80000000
#define V1_FIFO_32V1_32V2       0x00000000
#define V1_FIFO_48V1_32V2       0x00200000
#define V1_PREFETCH_ON_3336     0x40000000  /*V1_PREFETCH_ON*/
#define V1_GAMMA_ENABLE_3336    0x20000000  /*V1_Gamma_ENABLE*/

#define V1_FETCH_COUNT          0x3ff00000
#define V1_FETCHCOUNT_ALIGNMENT 0x0000000f
#define V1_FETCHCOUNT_UNIT      0x00000004   /* Doubld QWORD */

/* V1_STRIDE */
#define V1_STRIDE_YMASK         0x00001fff
#define V1_STRIDE_UVMASK        0x1ff00000

/* V1_ZOOM_CONTROL              0x24C */
#define V1_X_ZOOM_ENABLE        0x80000000
#define V1_Y_ZOOM_ENABLE        0x00008000

/* V1_MINI_CONTROL              0x250 */
#define V1_X_INTERPOLY          0x00000002  /* X interpolation */
#define V1_Y_INTERPOLY          0x00000001  /* Y interpolation */
#define V1_YCBCR_INTERPOLY      0x00000004  /* Y, Cb, Cr all interpolation */
#define V1_X_DIV_2              0x01000000
#define V1_X_DIV_4              0x03000000
#define V1_X_DIV_8              0x05000000
#define V1_X_DIV_16             0x07000000
#define V1_Y_DIV_2              0x00010000
#define V1_Y_DIV_4              0x00030000
#define V1_Y_DIV_8              0x00050000
#define V1_Y_DIV_16             0x00070000

/* V1_STARTADDR0               0x254 */
#define SW_FLIP_ODD             0x08000000

/* V_FIFO_CONTROL               0x258
 * IA2 has 32 level FIFO for packet mode video format
 *         32 level FIFO for planar mode video YV12. with extension reg 230 bit 21 enable
 *         16 level FIFO for planar mode video YV12. with extension reg 230 bit 21 disable
 * BCos of 128 bits. 1 level in IA2 = 2 level in VT3122
 */
#define V1_FIFO_DEPTH12         0x0000000B
#define V1_FIFO_DEPTH16         0x0000000F
#define V1_FIFO_DEPTH32         0x0000001F
#define V1_FIFO_DEPTH48         0x0000002F
#define V1_FIFO_DEPTH64         0x0000003F   
#define V1_FIFO_DEPTH122        0x00000079
#define V1_FIFO_THRESHOLD6      0x00000600
#define V1_FIFO_THRESHOLD8      0x00000800
#define V1_FIFO_THRESHOLD12     0x00000C00
#define V1_FIFO_THRESHOLD16     0x00001000
#define V1_FIFO_THRESHOLD24     0x00001800
#define V1_FIFO_THRESHOLD32     0x00002000
#define V1_FIFO_THRESHOLD40     0x00002800  
#define V1_FIFO_THRESHOLD48     0x00003000   
#define V1_FIFO_THRESHOLD56     0x00003800  
#define V1_FIFO_THRESHOLD61     0x00003D00  
#define V1_FIFO_THRESHOLD113    0x00007000
#define V1_FIFO_PRETHRESHOLD10  0x0A000000
#define V1_FIFO_PRETHRESHOLD12  0x0C000000
#define V1_FIFO_PRETHRESHOLD29  0x1d000000
#define V1_FIFO_PRETHRESHOLD40  0x28000000  
#define V1_FIFO_PRETHRESHOLD44  0x2c000000
#define V1_FIFO_PRETHRESHOLD56  0x38000000   
#define V1_FIFO_PRETHRESHOLD61  0x3D000000   
#define V1_FIFO_RRETHRESHOLD113 0x70000000

/*#define HI_CONTROL              0x260*/
#define V4_PREFETCH_ON_3204     0x40000000     /*V4_PREFETCH_ON*/
#define V4_DISPLAY_PATH		    0x80000000	/* 0: IGA1, 1: IGA2 */

/* ALPHA_V3_FIFO_CONTROL        0x278
 * IA2 has 32 level FIFO for packet mode video format
 *         32 level FIFO for planar mode video YV12. with extension reg 230 bit 21 enable
 *         16 level FIFO for planar mode video YV12. with extension reg 230 bit 21 disable
 *          8 level FIFO for ALPHA
 * BCos of 128 bits. 1 level in IA2 = 2 level in VT3122
 */
#define V3_FIFO_DEPTH16         0x0000000F
#define V3_FIFO_DEPTH24         0x00000017
#define V3_FIFO_DEPTH32         0x0000001F
#define V3_FIFO_DEPTH48         0x0000002F
#define V3_FIFO_DEPTH64         0x0000003F   
#define V3_FIFO_DEPTH100        0x00000063   
#define V3_FIFO_THRESHOLD8      0x00000800
#define V3_FIFO_THRESHOLD12     0x00000C00
#define V3_FIFO_THRESHOLD16     0x00001000
#define V3_FIFO_THRESHOLD24     0x00001800
#define V3_FIFO_THRESHOLD29     0x00001D00 
#define V3_FIFO_THRESHOLD32     0x00002000
#define V3_FIFO_THRESHOLD40     0x00002800  
#define V3_FIFO_THRESHOLD48     0x00003000   
#define V3_FIFO_THRESHOLD56     0x00003800   
#define V3_FIFO_THRESHOLD61     0x00003D00   
#define V3_FIFO_THRESHOLD63     0x00003F00
#define V3_FIFO_THRESHOLD88     0x00005800
#define V3_FIFO_THRESHOLD89     0x00005900
#define V3_FIFO_PRETHRESHOLD10  0x0000000A
#define V3_FIFO_PRETHRESHOLD12  0x0000000C
#define V3_FIFO_PRETHRESHOLD29  0x0000001d
#define V3_FIFO_PRETHRESHOLD40  0x00000028  
#define V3_FIFO_PRETHRESHOLD44  0x0000002c
#define V3_FIFO_PRETHRESHOLD56  0x00000038   
#define V3_FIFO_PRETHRESHOLD61  0x0000003D   
#define V3_FIFO_PRETHRESHOLD63  0x0000003F
#define V3_FIFO_PRETHRESHOLD88  0x00000058
#define V3_FIFO_PRETHRESHOLD89  0x00000059
#define V3_FIFO_MASK            0x0000007F
#define V3_FIFO_MASK_3314       0x000000FF
#define V3_FIFO_MASK_3324       0x000000FF
#define ALPHA_FIFO_DEPTH8       0x00070000
#define ALPHA_FIFO_DEPTH16      0x000f0000
#define ALPHA_FIFO_THRESHOLD4   0x04000000
#define ALPHA_FIFO_MASK         0xffff0000
#define ALPHA_FIFO_PRETHRESHOLD4 0x00040000

/* IA2 */
#define ColorSpaceValue_1       0x140020f2
#define ColorSpaceValue_2       0x0a0a2c00

#define ColorSpaceValue_1_3123C0      0x13000DED
#define ColorSpaceValue_2_3123C0      0x13171000
#define ColorSpaceValue_1_32045       0x13000DED 
#define ColorSpaceValue_2_32045       0x13171000 

/* V_COMPOSE_MODE               0x298 */
#define SELECT_VIDEO_IF_COLOR_KEY               0x00000001  /* select video if (color key),otherwise select graphics */
#define SELECT_VIDEO3_IF_COLOR_KEY              0x00000020  /* For 3123C0, select video3 if (color key),otherwise select graphics */
#define SELECT_VIDEO_IF_CHROMA_KEY              0x00000002  /* 0x0000000a  //select video if (chroma key ),otherwise select graphics */
#define ALWAYS_SELECT_VIDEO                     0x00000000  /* always select video,Chroma key and Color key disable */
#define CHROMA_KEY_INVERSE_CONTROL              0x00000008  /* 0: display video if chroma key not match ,1: display video if chroma key match */
#define COMPOSE_V1_V3                           0x00000000  /* V1 on top of V3 */
#define COMPOSE_V3_V1                           0x00100000  /* V3 on top of V1 */
#define COMPOSE_V1_TOP                          0x00000000
#define COMPOSE_V3_TOP                          0x00100000
#define V1_COMMAND_FIRE                         0x80000000  /* V1 commands fire */
#define V3_COMMAND_FIRE                         0x40000000  /* V3 commands fire */
#define V_COMMAND_LOAD                          0x20000000  /* Video register always loaded */
#define V_COMMAND_LOAD_VBI                      0x10000000  /* Video register always loaded at vbi without waiting source flip */
#define V3_COMMAND_LOAD                         0x08000000  /* CLE_C0 Video3 register always loaded */
#define V3_COMMAND_LOAD_VBI                     0x00000100  /* CLE_C0 Video3 register always loaded at vbi without waiting source flip */
#define SECOND_DISPLAY_COLOR_KEY_ENABLE         0x00010000

#define BYPASS_LCD_HORIZONTAL_MAGNIFY           0x00000020;
#define BYPASS_LCD_VERTICAL_MAGNIFY             0x00000010;

#define PLAY_ODD_FIELD          0x08000000
#define PLAY_EVEN_FIELD         0x00000000

/* V3_ZOOM_CONTROL              0x2bc */
#define V3_X_ZOOM_ENABLE        0x80000000
#define V3_Y_ZOOM_ENABLE        0x00008000

/* V3_MINI_CONTROL              0x2c0 */
#define V3_X_INTERPOLY              0x00000002  /* X interpolation */
#define V3_Y_INTERPOLY              0x00000001  /* Y interpolation */
#define V3_YCBCR_INTERPOLY          0x00000004  /* Y, Cb, Cr all interpolation */
#define V3_X_DIV_2                  0x01000000
#define V3_X_DIV_4                  0x03000000
#define V3_X_DIV_8                  0x05000000
#define V3_X_DIV_16                 0x07000000
#define V3_Y_DIV_2                  0x00010000
#define V3_Y_DIV_4                  0x00030000
#define V3_Y_DIV_8                  0x00050000
#define V3_Y_DIV_16                 0x00070000

/* SUBP_CONTROL_STRIDE              0x3c0 */
#define SUBP_HQV_ENABLE             0x00010000
#define SUBP_IA44                   0x00020000
#define SUBP_AI44                   0x00000000
#define SUBP_STRIDE_MASK            0x00001fff
#define SUBP_CONTROL_MASK           0x00070000

/* RAM_TABLE_CONTROL                0x3c8 */
#define RAM_TABLE_RGB_ENABLE        0x00000007


/* CAPTURE0_CONTROL                  0x310 */
#define C0_ENABLE                       0x00000001
#define BUFFER_2_MODE                   0x00000000
#define BUFFER_3_MODE                   0x00000004
#define BUFFER_4_MODE                   0x00000006
#define SWAP_YUYV                       0x00000000 
#define SWAP_UYVY                       0x00000100   
#define SWAP_YVYU                       0x00000200
#define SWAP_VYUY                       0x00000300
#define IN_601_8                        0x00000000
#define IN_656_8                        0x00000010
#define IN_601_16                       0x00000020
#define IN_656_16                       0x00000030
#define DEINTER_ODD                     0x00000000
#define DEINTER_EVEN                    0x00001000   
#define DEINTER_ODD_EVEN                0x00002000
#define DEINTER_FRAME                   0x00003000
#define VIP_1                           0x00000000 
#define VIP_2                           0x00000400
#define H_FILTER_2                      0x00010000
#define H_FILTER_4                      0x00020000 
#define H_FILTER_8_1331                 0x00030000 
#define H_FILTER_8_12221                0x00040000
#define CLK_INPUT_INVERSE               0x00080000
#define VIP_ENABLE                      0x00000008
#define EN_FIELD_SIG                    0x00000800  
#define VREF_INVERT                     0x00100000
#define FIELD_INPUT_INVERSE             0x00400000
#define FIELD_INVERSE                   0x40000000

#define C1_H_MINI_EN                    0x00000800
#define C0_H_MINI_EN                    0x00000800
#define C1_V_MINI_EN                    0x04000000
#define C0_V_MINI_EN                    0x04000000
#define C1_H_MINI_2                     0x00000400

/* CAPTURE1_CONTROL                  0x354 */
#define C1_ENABLE                       0x00000001

/* V3_CONTROL                   0x2A0 */
#define V3_ENABLE               0x00000001
#define V3_FULL_SCREEN          0x00000002
#define V3_YUV422               0x00000000
#define V3_RGB32                0x00000004
#define V3_RGB15                0x00000008
#define V3_RGB16                0x0000000C
#define V3_COLORSPACE_SIGN      0x00000080
#define V3_EXPIRE_NUM           0x00040000
#define V3_EXPIRE_NUM_F         0x000f0000 
#define V3_EXPIRE_NUM_3205      0x00080000
#define V3_EXPIRE_NUM_3204      0x00100000
#define V3_BOB_ENABLE           0x00400000
#define V3_FIELD_BASE           0x00000000
#define V3_FRAME_BASE           0x01000000
#define V3_SWAP_SW              0x00000000
#define V3_SWAP_HW_HQV          0x02000000
#define V3_FLIP_HW_CAPTURE0     0x04000000
#define V3_FLIP_HW_CAPTURE1     0x06000000
#define V3_GAMMA_ENABLE_3204    0x20000000  /*V3_Gamma_ENABLE*/
#define V3_PREFETCH_ON_3204     0x40000000  /*V3_PREFETCH_ON*/
#define V3_ON_SND_DISPLAY_32045 0x80000000  /*V3_ON_SND_DISPLAY*/

/* V3_ALPHA_FETCH_COUNT           0x2B8 */
#define V3_FETCH_COUNT          0x3ff00000
#define ALPHA_FETCH_COUNT       0x000003ff

/* V1 V3 identity */
#define VIDEO_1                 0x00000001
#define VIDEO_3                 0x00000002

/* HQV_CONTROL             0x3D0 */
#define HQV_RGB32           0x00000000
#define HQV_RGB16           0x20000000
#define HQV_RGB15           0x30000000
#define HQV_YUV422          0x80000000
#define HQV_YUV420          0xC0000000
#define HQV_ENABLE          0x08000000
#define HQV_SRC_SW          0x00000000
#define HQV_SRC_MC          0x01000000
#define HQV_SRC_CAPTURE0    0x02000000
#define HQV_SRC_CAPTURE1    0x03000000
#define HQV_FLIP_EVEN       0x00000000
#define HQV_FLIP_ODD        0x00000020
#define HQV_SW_FLIP         0x00000010   /* Write 1 to flip HQV buffer */
#define HQV_INT_ENABLE      0x00000080
#define HQV_DEINTERLACE     0x00010000   /* First line of odd field will be repeated 3 times */
#define HQV_FIELD_2_FRAME   0x00020000   /* Src is field. Display each line 2 times */
#define HQV_FRAME_2_FIELD   0x00040000   /* Src is field. Display field */
#define HQV_FRAME_UV        0x00000000   /* Src is Non-interleaved */
#define HQV_FIELD_UV        0x00100000   /* Src is interleaved */
#define HQV_IDLE            0x00000008   
#define HQV_FLIP_STATUS     0x00000001   
#define HQV_DOUBLE_BUFF     0x00000000
#define HQV_TRIPLE_BUFF     0x04000000
#define HQV_SUB_FLIP        0x00008000
#define HQV_FIFO_STATUS     0x00001000  

/* HQV_FILTER_CONTROL      0x3E4 */
#define HQV_H_LOWPASS_2TAP  0x00000001
#define HQV_H_LOWPASS_4TAP  0x00000002
#define HQV_H_LOWPASS_8TAP1 0x00000003   /* To be deleted */
#define HQV_H_LOWPASS_8TAP2 0x00000004   /* To be deleted */
#define HQV_H_HIGH_PASS     0x00000008
#define HQV_H_LOW_PASS      0x00000000
#define HQV_V_LOWPASS_2TAP  0x00010000
#define HQV_V_LOWPASS_4TAP  0x00020000
#define HQV_V_LOWPASS_8TAP1 0x00030000
#define HQV_V_LOWPASS_8TAP2 0x00040000
#define HQV_V_HIGH_PASS     0x00080000
#define HQV_V_LOW_PASS      0x00000000
#define HQV_H_HIPASS_F1_DEFAULT 0x00000040
#define HQV_H_HIPASS_F2_DEFAULT 0x00000000
#define HQV_V_HIPASS_F1_DEFAULT 0x00400000
#define HQV_V_HIPASS_F2_DEFAULT 0x00000000
#define HQV_H_HIPASS_F1_2TAP    0x00000050
#define HQV_H_HIPASS_F2_2TAP    0x00000100
#define HQV_V_HIPASS_F1_2TAP    0x00500000
#define HQV_V_HIPASS_F2_2TAP    0x01000000
#define HQV_H_HIPASS_F1_4TAP    0x00000060
#define HQV_H_HIPASS_F2_4TAP    0x00000200
#define HQV_V_HIPASS_F1_4TAP    0x00600000
#define HQV_V_HIPASS_F2_4TAP    0x02000000
#define HQV_H_HIPASS_F1_8TAP    0x00000080
#define HQV_H_HIPASS_F2_8TAP    0x00000400
#define HQV_V_HIPASS_F1_8TAP    0x00800000
#define HQV_V_HIPASS_F2_8TAP    0x04000000
/* IA2 NEW */
#define HQV_V_FILTER2           0x00080000
#define HQV_H_FILTER2           0x00000008
#define HQV_H_TAP2_11           0x00000041
#define HQV_H_TAP4_121          0x00000042
#define HQV_H_TAP4_1111         0x00000401
#define HQV_H_TAP8_1331         0x00000221
#define HQV_H_TAP8_12221        0x00000402
#define HQV_H_TAP16_1991        0x00000159
#define HQV_H_TAP16_141041      0x0000026A
#define HQV_H_TAP32             0x0000015A
#define HQV_V_TAP2_11           0x00410000
#define HQV_V_TAP4_121          0x00420000
#define HQV_V_TAP4_1111         0x04010000
#define HQV_V_TAP8_1331         0x02210000
#define HQV_V_TAP8_12221        0x04020000
#define HQV_V_TAP16_1991        0x01590000
#define HQV_V_TAP16_141041      0x026A0000
#define HQV_V_TAP32             0x015A0000
#define HQV_V_FILTER_DEFAULT    0x00420000
#define HQV_H_FILTER_DEFAULT    0x00000040




/* HQV_MINI_CONTROL        0x3E8 */
#define HQV_H_MINIFY_ENABLE 0x00000800
#define HQV_V_MINIFY_ENABLE 0x08000000
#define HQV_VDEBLOCK_FILTER 0x80000000
#define HQV_HDEBLOCK_FILTER 0x00008000

/* HQV_SRC_STRIDE          0x3F8 */
#define HQV_FIFO_DEPTH_1          0x00010000   /* HQV_FIFO_DEPTH_32045, HQV_FIFO_DEPTH_3123C0 */
#define HQV_FIFO_DEPTH_3          0x00030000

/* HQV Default Vodeo Color 0x3B8 */
#define HQV_FIX_COLOR           0x0643212c 


#define CHROMA_KEY_LOW          0x00FFFFFF
#define CHROMA_KEY_HIGH         0x00FFFFFF

/* V_CAP_STATUS */
#define V_ST_UPDATE_NOT_YET     0x00000003
#define V1_ST_UPDATE_NOT_YET    0x00000001
#define V3_ST_UPDATE_NOT_YET    0x00000008

#define VBI_STATUS              0x00000002

#define VO_PATH_SW      0x01
#define VO_PATH_VMI     0x02
/*
 *      Macros for Video MMIO
 */
#define VIDInB(port)            *((volatile unsigned char *)(pjVideo + (port)))
#define VIDInW(port)            *((volatile unsigned short *)(pjVideo + (port)))
#define VIDInD(port)            *((volatile CARD32 *)(pjVideo + (port)))
#define VIDOutB(port, data)     *((volatile unsigned char *)(pjVideo + (port))) = (data)
#define VIDOutW(port, data)     *((volatile unsigned short *)(pjVideo + (port))) = (data)
#define VIDOutD(port, data)     *((volatile CARD32 *)(pjVideo + (port))) = (data)
#define MPGOutD(port, data)     *((volatile CARD32 *)(pjVideo +(port))) = (data)
#define MPGInD(port)            *((volatile CARD32 *)(pjVideo +(port)))

/*
 *      Macros for GE MMIO
 */
#define GEInW(port)             *((volatile unsigned short *)(pjVideo + (port)))
#define GEInD(port)             *((volatile CARD32 *)(pjVideo + (port)))
#define GEOutW(port, data)      *((volatile unsigned short *)(pjVideo + (port))) = (data)
#define GEOutD(port, data)      *((volatile CARD32 *)(pjVideo + (port))) = (data)

/*
 *      Macros for GPIO MMIO
 */
#define INB(addr)        *((volatile unsigned char *)(pjVideo+0x8000 + (addr)))
#define OUTB(val, addr)  *((volatile unsigned char *)(pjVideo+0x8000 + (addr))) = (val)

/*
 *      Macros for general MMIO
 */
#define dMMWR(p, i, v)      *((volatile CARD32 *)(p + (i))) = (v)
#define dMMRD(p,i)          *((volatile CARD32 *)(p + (i)))

/*william[20071101]: for unified interface*/
#define VIAMMIOW8(p, i, v)      *((volatile CARD8 *)(p + (i)))   = (v)
#define VIAMMIOR8(p,i)            *((volatile CARD8 *)(p + (i)))
#define VIAMMIOW16(p, i, v)    *((volatile CARD16 *)(p + (i))) = (v)
#define VIAMMIOR16(p,i)          *((volatile CARD16 *)(p + (i)))
#define VIAMMIOW32(p, i, v)    *((volatile CARD32 *)(p + (i))) = (v)
#define VIAMMIOR32(p,i)          *((volatile CARD32 *)(p + (i)))


#endif /* _VIA_H */
