/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _VIAMPG
#define _VIAMPG

#ifdef XFree86Server
#include "xf86_ansic.h" /* for Bool */
#else
#include "via_types.h"  /* for Bool */
#endif

typedef struct _VIAMCSURFACE
{
    unsigned long dwSize;                       /* Size of this structure */
    unsigned long dwVersion;                    /* Interface Version */
    unsigned long pDevice;                      /* Useless */
    void * BeginPicture;                        /* Pointer to VIA_BeginPicture */
    void * EndPicture;                          /* Pointer to VIA_Endicture */
    void * SetMacroBlock;                       /* Pointer to VIA_SetMacroBlock */
    void * SliceReceiveData;                    /* Pointer to VIA_SliceReceiveData */
    void * DriverProc;
    void * DisplayControl;
    void * SubPicture;
    void * SetDeInterlaceMode;                  /* Pointer to VIA_SetDeinterlaceMode */
    unsigned long dwState;                      /* Current state of MPG  */
    unsigned long dwBufferNumber;               /* Total no. of decoder buffer created. (Always 4 ) */
    Bool  bInitialized;                         /* Except dwState */
    unsigned long dwReserved[8];                /* For future expansion */

    /* Client fills out the following values before calling BeginPicture */
    unsigned long dwTaskType;                   /* Null, Decode or Display */

    /* Decode Picture */
    unsigned long dwPictureType;                /* I, P or B pictures */
    unsigned long dwDecodePictStruct;           /* Top Field, Bottom Field or Frame */
    unsigned long dwDecodeBuffIndex;            /* Decode buffer Index. 0 to ( dwBufferNumber -1) */
    unsigned long dwAlternateScan;              /* Zig-Zag Scan or Alternative Scan */
    unsigned int ForwardRefPictIsInterlace;     /* Is forward reference picture interlace? 1: interlace, 0: non-interlace */
    unsigned int BackwardRefPictIsInterlace;    /* Is backward reference picture interlace? 1: interlace, 0: non-interlace */

    /* Display picture */
    unsigned long dwDisplayPictStruct;          /* Top Field, Bottom Field or Frame */
    unsigned long dwDisplayBuffIndex;           /* Display buffer Index. 0 to ( dwBufferNumber -1) */
    unsigned long dwFrameRate;                  /* Frame rate  multiplied by 1000 ( 29.97=29700 ) */
    unsigned long dwDeinterlaceMode;            /* BOB / Weave */
    unsigned long dwQMatrix[2][16];             /* Quantiser Matrix, 0:intra, 1:non_intra */
    unsigned long dwQMatrixChanged;             /* If The Quantiser Matrix is changed? */
    unsigned long dwMBwidth;                    /* Macroblock Width */
    unsigned long dwTopFirst;
    unsigned long dwFramePredDct;
    unsigned long dwMBAmax;                     /* It is equal to mb_width * mb_height; */
    unsigned long dwIntravlc;
    unsigned long dwDcPrec;
    unsigned long dwQscaleType;
    unsigned long dwConcealMV;
    unsigned long dwSecondField;
    unsigned long dwOldRefFrame;                /* Fordward reference */
    unsigned long dwRefFrame;                   /* Backward reference */
    unsigned long BVMVRange;                    /* Backward Vertical Motion Vector Range   */
    unsigned long BHMVRange;                    /* Backward Horizontal Motion Vector Range */
    unsigned long FVMVRange;                    /* Fordward Vertical Motion Vector Range   */
    unsigned long FHMVRange;                    /* Fordward Horizontal Motion Vector Range */
    unsigned long dwMPEGProgressiveMode;        /* default value : VIA_PROGRESSIVE */
    int   framenum;
    unsigned long dwDecoderMode;                /* indicate H/W mpeg decoder mode: USE_MC, USE_IDCT, USE_SLICE */

    unsigned long dwLumaFilterMode;	            /* 0:bi-linear, 1:bi-cubic */
    unsigned long dwOverlapTSF;                 /* Indicate to enable overlay transform filter */	
    unsigned long dwIntensityComp;              /* Indicate to enable intensity compensation */	
    unsigned long dwLumScale;                   /* for intensity compensation */
    unsigned long dwLumShift;	                /* for intensity compensation */
    
    /* Slice Information */
    unsigned int wRefIdxNum[2];                 /* [0]:Fordward, [1]:Backward */
    unsigned long dwCurFieldOrderCnt[2];
    unsigned long dwRefFieldOrderCnt[16][2];
    unsigned char bSliceType;                   /* 0:P, 1:B, 2:I */
    unsigned char bFieldPicFlag;
    unsigned char bBottomFieldFlag;
    unsigned char bDisableDeblock;
    unsigned char bRefList[2][32];
    unsigned char bRefFieldTerm[16][2];    
    unsigned long dwRreserved[6];               /* for future expansion */
} VIAMPGSURFACE, * LPVIAMPGSURFACE;

/* definition for dwDecoderMode */
#define USE_SLICE                 0
#define USE_MC                    1
#define USE_MC_IDCT               2

/* dwTaskType of VIAMPGSURFACE */
#define VIA_TASK_NULL              0x00000000    /* Used only to check state and version */
#define VIA_TASK_DECODE            0x00000001    /* Decode new Picture also Display decoded picture */
#define VIA_TASK_DISPLAY           0x00000002    /* Display only */

/* dwPictureType of VIAMPGSURFACE */
#define VIA_PIC_TYPE_I              0x00000001
#define VIA_PIC_TYPE_P              0x00000002
#define VIA_PIC_TYPE_B              0x00000003

/* dwDecodePictStruct and dwDisplayPictStruct of VIAMPGSURFACE */
#define VIA_PICT_STRUCT_TOP                 0x00000001
#define VIA_PICT_STRUCT_BOTTOM              0x00000002
#define VIA_PICT_STRUCT_FRAME               0x00000003
#define VIA_PICT_STRUCT_MBAFF               0x00000004

/* dwAltScan */
#define VIA_ALTERNATE_SCAN                  0x1
#define VIA_ZIG_ZAG_SCAN                    0x0

/* dwDeinterlaceMode of VIAMPGSURFACE */
#define VIA_DEINTERLACE_WEAVE               0x00000000
#define VIA_DEINTERLACE_BOB                 0x00000001
#define VIA_DEINTERLACE_MEDIAN              0x00000002


/* dwMPEGProgressiveMode of VIAMPGSURFACE */
#define VIA_NON_PROGRESSIVE                 0x00000000
#define VIA_PROGRESSIVE                     0x00000010

typedef enum _MB_TYPE{
    MB_TYPE_INTER_16X16 = 0,
    MB_TYPE_INTER_16X8,
    MB_TYPE_INTER_8X16,
    MB_TYPE_INTER_8X8,
    MB_TYPE_MB_SUB,
    MB_TYPE_INTRA_16X16,
    MB_TYPE_INTRA_8X8,
    MB_TYPE_INTRA_4X4,
    MB_TYPE_MB_IPCM,
} MB_TYPE, *LPMB_TYPE;


    
/**************************************************************************
   VIA Macro Block Structure
***************************************************************************/

/* store the IDCT coefficient values. */
typedef struct _IDCTCoef4Group {
    unsigned char  CoefIndex[4];
    short CoefValue[4];
} IDCTCoef4Group, *LPIDCTCoef4Group;



#endif

