/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 

#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <linux/version.h>
#include "via_driver.h"
#ifdef RANDRV12_TEST_ENABLE
#include "globals.h"
#include "debug.h"      /* for DBG_DD */
#include "xf86RAC.h"
#include "xf86Priv.h"
#include "xf86Crtc.h"
#include "xf86RandR12.h"
#include "via_bios.h"
#include "via_rotate.h"


extern void VIAGetHWCursorRealPosition(ScrnInfoPtr pScrn, int iga_path, int x, int y, int frameX0, int frameY0,
        int *xpos, int *ypos, unsigned char *xoff, unsigned char *yoff);

extern void VIAARGBCursorInit(ScrnInfoPtr pScrn);

static int
get_crtc_num(xf86CrtcPtr crtc)
{
    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcConfigPtr   via_crtc_config = XF86_CRTC_CONFIG_PTR (pScrn);
    int i = 0;
    
    for(i = 0; i < via_crtc_config->num_output; i++)
    {
        if (via_crtc_config->crtc[i] == crtc)
            return i;
    }
}

static int
get_crtc_device_type(xf86CrtcPtr crtc)
{
    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcConfigPtr   via_crtc_config = XF86_CRTC_CONFIG_PTR (pScrn);
    int i = 0;
    
    for(i = 0; i < via_crtc_config->num_output; i++)
    {
        if (via_crtc_config->output[i]->crtc == crtc)
        {
            if (!strcmp(via_crtc_config->output[i]->name, "CRT"))
                return VIA_DEVICE_CRT1;
            
            if (!strcmp(via_crtc_config->output[i]->name, "DVI"))
                return VIA_DEVICE_DFP;
            
            if (!strcmp(via_crtc_config->output[i]->name, "LCD"))
                return VIA_DEVICE_LCD;
            
        }
    }

}


static int get_crtc_active_dev(xf86CrtcPtr crtc)
{
    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    xf86CrtcConfigPtr   via_crtc_config = XF86_CRTC_CONFIG_PTR (pScrn);
    int i;
    int desiredActive = 0;
        
    for(i = 0; i < via_crtc_config->num_output; i++)
    {
        if (via_crtc_config->output[i]->crtc)
        {
            if (!strcmp(via_crtc_config->output[i]->name, "CRT"))
            {
                desiredActive |= VIA_DEVICE_CRT1;
            }

            if (!strcmp(via_crtc_config->output[i]->name, "DVI"))
            {
                desiredActive |= VIA_DEVICE_DFP;
            }   

            if (!strcmp(via_crtc_config->output[i]->name, "LCD"))
            {
                desiredActive |= VIA_DEVICE_LCD;
            }   
        }
    }

    pBIOSInfo->ActiveDevice = desiredActive;

    return desiredActive;
}



Bool via_init(VIABIOSInfoPtr pBIOSInfo)
{
    VIAModeTablePtr pViaModeTable;    
    int             i = 0;
    int             modeNum;
    int             ModeIndex;
    
    DEBUG(xf86DrvMsg(pBIOSInfo->scrnIndex, X_INFO, "via_init\n"));
    pViaModeTable = pBIOSInfo->pModeTable;

    /* Initial down scaling variable. */
    pBIOSInfo->IsDownScaleEnable = FALSE;

    /* detemine support highest resolution by Memory clk */
    pBIOSInfo->MemClk = (read_reg(VIACR, CR3D) & 0xF0) >> 4;
    
    if ((pBIOSInfo->bitsPerPixel > 16) && (pBIOSInfo->HDisplay == 1600) &&
        (pBIOSInfo->VDisplay == 1200) && (pBIOSInfo->MemClk < VIA_MEM_DDR266))
    {
        ErrorF("\n1600x1200 True Color only support in MEMCLK higher than DDR266 platform!!\n");
        ErrorF("Please use lower bpp or resolution.\n");
        return FALSE;
    }       

    if (pBIOSInfo->VirtualX > 0)
    {
        /* If set Virtual Desktop Size in xorg.conf: */
        pBIOSInfo->ActualDesktopSizeX = pBIOSInfo->VirtualX;
        pBIOSInfo->ActualDesktopSizeY = pBIOSInfo->VirtualY;
    }
    else
    {
        pBIOSInfo->ActualDesktopSizeX = pBIOSInfo->CrtcHDisplay;
        pBIOSInfo->ActualDesktopSizeY = pBIOSInfo->CrtcVDisplay;
    }


    for (i = 0; i < pViaModeTable->NumModes; i++)
    {
        if ((pViaModeTable->Modes[i].Bpp == pBIOSInfo->bitsPerPixel) &&
            (pViaModeTable->Modes[i].Width == pBIOSInfo->HDisplay) &&
            (pViaModeTable->Modes[i].Height == pBIOSInfo->VDisplay))
            break;
    }
    
    if (i == pViaModeTable->NumModes)
    {
        ErrorF("\nVIASetModeUseBIOSTable: Cannot find suitable mode!!\n");
        ErrorF("Mode setting in XF86Config(-4) is not supported!!\n");
        return FALSE;
    }
    
    pBIOSInfo->modeNum = modeNum = (int)pViaModeTable->Modes[i].Mode;
    pBIOSInfo->mode = i;


    ModeIndex = VIAGetModeIndex(pBIOSInfo->CrtcHDisplay, pBIOSInfo->CrtcVDisplay);
    pBIOSInfo->resMode = ModeIndex;
    pBIOSInfo->resMode2 = ModeIndex;

    pBIOSInfo->offsetWidthByQWord = (pBIOSInfo->displayWidth * (pBIOSInfo->bitsPerPixel >> 3)) >> 3;
    pBIOSInfo->countWidthByQWord = (pBIOSInfo->CrtcHDisplay * (pBIOSInfo->bitsPerPixel >> 3)) >> 3;

    return TRUE;
    
}


/**
 * Sets the power management mode of the pipe and plane.
 *
 * This code should probably grow support for turning the cursor off and back
 * on appropriately at the same time as we're turning the pipe off/on.
 */
static void
via_crtc_dpms(xf86CrtcPtr crtc, int mode)
{
    DEBUG(ErrorF("via_crtc_dpms, mode = %d\n", mode));
    
    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn); 
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;       

    switch (mode)
    {
        case DPMSModeOn:        /* 0 */
            break;
            
        case DPMSModeStandby:   /* 1 */
        case DPMSModeSuspend:   /* 2 */
        case DPMSModeOff:       /* 3 */
            break;
    }
}


static Bool
via_crtc_lock (xf86CrtcPtr crtc)
{   
    DEBUG(ErrorF("via_crtc_lock\n"));

    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr      pVia = VIAPTR(pScrn); 
    /* Wait Hardware Engine idle to switch graphicd mode */
    WaitIdle();
    
    return FALSE;
}

static void
via_crtc_unlock (xf86CrtcPtr crtc)
{
    DEBUG(ErrorF("via_crtc_unlock\n"));
}


static void
via_crtc_prepare (xf86CrtcPtr crtc)
{
    DEBUG(ErrorF("via_crtc_prepare\n"));
    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;
    VIACrtcPrivatePtr   via_private_crtc = VIACrtcPrivate(crtc);     
    
    if (pVia->VQEnable) 
    { 
        /* if we use VQ, disable it before we exit */
        switch (pVia->Chipset)
        {
            case VIA_P4M890:             
            case VIA_K8M890:
                VIASETREG(0x41c, 0x00100000);
                VIASETREG(0x420, 0x74301000);
                break;
                
            default:  
                VIASETREG(0x43c, 0x00fe0000);
                VIASETREG(0x440, 0x00000004);
                VIASETREG(0x440, 0x40008c0f);
                VIASETREG(0x440, 0x44000000);
                VIASETREG(0x440, 0x45080c04);
                VIASETREG(0x440, 0x46800408);
                break;
        }
    }    
       
    /* Get the desired active device */
    get_crtc_active_dev(crtc);
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pBIOSInfo->ActiveDevice = 0x%x\n", pBIOSInfo->ActiveDevice));        
    
    VIACheckIfUseDuoView(pScrn);

    via_init(pBIOSInfo);
              
    /* Turn off Screen */
    write_reg_mask(CR17, VIACR, 0x00, BIT7);

    if ((pBIOSInfo->Chipset == VIA_VX800) &&
        (pBIOSInfo->ActiveDevice & VIA_DEVICE_LCD) &&
        (pBIOSInfo->DisplayHardwareLayout == HW_LAYOUT_TTL_LCD))
    {    
        write_reg_mask(CR91, VIACR, 0x00, 0x02);
    }
    
    DEBUG(xf86DrvMsg(pScrn->scrnIndex, X_INFO, "crtc->rotation = %x\n", crtc->rotation));        
    switch(crtc->rotation & 0x0F)
    {
        case RR_Rotate_0:
            via_private_crtc->RotateDegree = VIA_ROTATE_DEGREE_0;
            break;

        case RR_Rotate_90:
            via_private_crtc->RotateDegree = VIA_ROTATE_DEGREE_90;   
            break;

        case RR_Rotate_180:
            via_private_crtc->RotateDegree = VIA_ROTATE_DEGREE_180; 
            break;

        case RR_Rotate_270:
            via_private_crtc->RotateDegree = VIA_ROTATE_DEGREE_270;
            break;
        
        default:            
            via_private_crtc->RotateDegree = VIA_ROTATE_DEGREE_0;
            break;
    } 

    switch(crtc->rotation & 0xF0)
    {    
        case RR_Reflect_X:
            via_private_crtc->RotateDegree |= VIA_ROTATE_REFLECT_X;
            break;

        case RR_Reflect_Y:
            via_private_crtc->RotateDegree |= VIA_ROTATE_REFLECT_Y;
            break;

        case RR_Reflect_X | RR_Reflect_Y:
            via_private_crtc->RotateDegree |= (VIA_ROTATE_REFLECT_X | VIA_ROTATE_REFLECT_Y);
            break;
    }
}



/**
 * Sets up registers for the given mode/adjusted_mode pair.
 *
 * The clocks, CRTCs and outputs attached to this CRTC must be off.
 *
 * This shouldn't enable any clocks, CRTCs, or outputs, but they should
 * be easily turned on/off after this.
 */
static void
via_crtc_mode_set(xf86CrtcPtr crtc, DisplayModePtr mode,
                                        DisplayModePtr adjusted_mode,
                                        int x, int y)
{    
    DEBUG(ErrorF("via_crtc_mode_set\n"));    
    
    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;

    xf86CrtcConfigPtr   xf86_config = XF86_CRTC_CONFIG_PTR(pScrn);
    
}



static void
via_crtc_commit (xf86CrtcPtr crtc)
{
    DEBUG(ErrorF("via_crtc_commit\n"));

    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;    

    /* Enable CRT Controller (3D5.17 Hardware Reset) */
    write_reg_mask(CR17, VIACR, BIT7, BIT7);

    /* Open Screen */
    (void) VGAIN8(0x3da);
    VGAOUT8(0x3c0, 0x20);
    
#ifdef USEHwIconSimulateHwCursor
    VIAHideCursor(pScrn);
    VIALoadIconImage(pScrn, pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart);
    VIAShowCursor(pScrn);
#endif

}



static Bool
via_crtc_mode_fixup(xf86CrtcPtr crtc, DisplayModePtr mode,
                        DisplayModePtr adjusted_mode)
{
    DEBUG(ErrorF("via_crtc_mode_fixup\n"));
    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;    
    
    return TRUE;
}


/** Sets the color ramps on behalf of RandR */
static void
via_crtc_gamma_set(xf86CrtcPtr crtc, CARD16 *red, CARD16 *green, CARD16 *blue,
                        int size)
{
    DEBUG(ErrorF("via_crtc_gamma_set\n"));
}


/**
 * Allocates memory for a locked-in-framebuffer shadow of the given
 * width and height for this CRTC's rotated shadow framebuffer.
 */
 
static void *
via_crtc_shadow_allocate (xf86CrtcPtr crtc, int width, int height)
{
    DEBUG(ErrorF("via_crtc_shadow_allocate, width = %d, height = %d\n", width, height));
    
    ScrnInfoPtr         pScrn = crtc->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr      pBIOSInfo = pVia->pBIOSInfo;    
    VIACrtcPrivatePtr   via_private_crtc = VIACrtcPrivate(crtc);                         

    if (pVia->NumOfShadowInUse < 0x2)
    {
        via_private_crtc->RotateShadowOffset = pVia->RotateShadowOffset[pVia->NumOfShadowInUse];
        pVia->NumOfShadowInUse += 1;
    }        
    
    return (pVia->FBBase + via_private_crtc->RotateShadowOffset);
}
    
/**
 * Creates a pixmap for this CRTC's rotated shadow framebuffer.
 */
static PixmapPtr
via_crtc_shadow_create(xf86CrtcPtr crtc, void *data, int width, int height)
{
    DEBUG(ErrorF("via_crtc_shadow_create, width = %d, height = %d\n", width, height));

    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr      pVia = VIAPTR(pScrn);
    PixmapPtr   rotate_pixmap;
    CARD32      rotate_pitch;

    if (!data) 
    {
        data = via_crtc_shadow_allocate (crtc, width, height);
    }
    
    rotate_pitch = pScrn->displayWidth * pVia->Bpp;

    rotate_pixmap = GetScratchPixmapHeader(pScrn->pScreen,
                       width, height,
                       pScrn->depth,
                       pScrn->bitsPerPixel,
                       rotate_pitch,
                       data);

    if (rotate_pixmap == NULL) 
    {
        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Couldn't allocate shadow pixmap for rotated CRTC\n");
    }       
    
    return rotate_pixmap;
}

static void
via_crtc_shadow_destroy(xf86CrtcPtr crtc, PixmapPtr rotate_pixmap, void *data)
{        
    DEBUG(ErrorF("via_crtc_shadow_destroy\n"));      

    ScrnInfoPtr         pScrn = crtc->scrn;
    VIAPtr              pVia = VIAPTR(pScrn);
    VIACrtcPrivatePtr   via_private_crtc = VIACrtcPrivate(crtc);   
    
    if (rotate_pixmap)
        FreeScratchPixmapHeader(rotate_pixmap);                
}


void
via_crtc_set_cursor_colors (xf86CrtcPtr crtc, int bg, int fg)
{
    DEBUG(ErrorF("Call via_crtc_set_cursor_colors\n"));
    
    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);    
    
    VIASETREG(VIA_REG_CURSOR_FG, fg);
    VIASETREG(VIA_REG_CURSOR_BG, bg); 
}


void
via_crtc_show_cursor (xf86CrtcPtr crtc)
{
    DEBUG(ErrorF("via_crtc_show_cursor\n"));
    
    ScrnInfoPtr pScrn = crtc->scrn;        
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 dwCursorMode;
    CARD32 dwHiControl;
    unsigned long* Screen_IGA_Map = pVia->pVidData->VIAGraphicInfo.Screen_IGA_Map;


    if ((pBIOSInfo->Chipset == VIA_CX700) ||
        (pBIOSInfo->Chipset == VIA_CN750) ||
        (pBIOSInfo->Chipset == VIA_VX800))
    {
        /* Set color to white and black. */
        /* Use 30 bits [29:0] to specify HW cursor background/foreground color. */
        via_crtc_set_cursor_colors (crtc, 0x3fffffff, 0);        
    }

    dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);
    dwCursorMode &=~0x80000003;

    /* Turn on Hardware Cursor */
    if(Screen_IGA_Map[pScrn->scrnIndex] == IGA1)
    {        
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode | 0x1);
    }
    else
    {        
        VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode | 0x80000001);
    }    
}


void
via_crtc_hide_cursor (xf86CrtcPtr crtc)
{
    DEBUG(ErrorF("via_crtc_hide_cursor\n"));
    
    ScrnInfoPtr pScrn = crtc->scrn;    
    VIAPtr pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    CARD32 dwCursorMode;
    CARD32 dwHiControl;
    unsigned long* Screen_IGA_Map = pVia->pVidData->VIAGraphicInfo.Screen_IGA_Map;

    dwCursorMode = VIAGETREG(VIA_REG_CURSOR_MODE);

    /* Turn cursor off. */
    VIASETREG(VIA_REG_CURSOR_MODE, dwCursorMode & 0xFFFFFFFE);
            
}



void
via_crtc_set_cursor_position (xf86CrtcPtr crtc, int x, int y)
{
    ScrnInfoPtr pScrn = crtc->scrn;    
    VIAPtr          pVia = VIAPTR(pScrn);
    VIABIOSInfoPtr  pBIOSInfo = pVia->pBIOSInfo;
    int             xpos, ypos;    
    unsigned char   xoff, yoff;
    unsigned long* Screen_IGA_Map = pVia->pVidData->VIAGraphicInfo.Screen_IGA_Map;
    
    VIAGetHWCursorRealPosition(pScrn, (int)Screen_IGA_Map[pScrn->scrnIndex], x, y, pScrn->frameX0, pScrn->frameY0, &xpos, &ypos, &xoff, &yoff);
    VIASETREG(VIA_REG_CURSOR_ORG, ((xoff << 16) | (yoff & 0x003f)));
    VIASETREG(VIA_REG_CURSOR_POS, ((xpos << 16) | (ypos & 0x07ff)));       
}


void
via_crtc_load_cursor_image (xf86CrtcPtr crtc, CARD8 *src)
{
    DEBUG(ErrorF("via_crtc_load_cursor_image\n"));
    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);

    memcpy(pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart, src, MAX_CURS * MAX_CURS / 8 * 2);
}


void
via_crtc_load_cursor_argb (xf86CrtcPtr crtc, CARD32 *image)
{
    DEBUG(ErrorF("via_crtc_load_cursor_argb\n"));

    ScrnInfoPtr pScrn = crtc->scrn;
    VIAPtr pVia = VIAPTR(pScrn);    
    
    CARD8* hw_cursor_addr = pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HwCursorBufStart;
    CARD32 hw_cursor_size = MAX_CURS * MAX_CURS/8*2;
    
    CARD8* hw_icon_addr = pVia->FBBase + pVia->pScreensPublicInfo->CursorInfo.HIBufStart;
    CARD32 hw_icon_size = MAX_CURS * MAX_CURS * 4;
        
    memcpy(hw_icon_addr, image, 64 * 64 * 4);    
}


static const xf86CrtcFuncsRec via_crtc_funcs = {
    .dpms = via_crtc_dpms,
    .save = NULL, 
    .restore = NULL, 
    .lock = via_crtc_lock,
    .unlock = NULL,
    .mode_fixup = via_crtc_mode_fixup,
    .prepare = via_crtc_prepare,
    .mode_set = via_crtc_mode_set,
    .commit = via_crtc_commit,
    .gamma_set = via_crtc_gamma_set,
    .shadow_create = via_crtc_shadow_create,
    .shadow_allocate = via_crtc_shadow_allocate,  
    .shadow_destroy = via_crtc_shadow_destroy,
    .destroy = NULL,
};


void via_crtc_init(ScrnInfoPtr pScrn, int iga_num)
{
    DEBUG(ErrorF("Call via_crtc_init.\n"));

    xf86CrtcPtr crtc;
    VIACrtcPrivatePtr via_private_crtc;
    
    int i;
    
    for(i = 0; i < iga_num; i++)
    {        
        crtc = xf86CrtcCreate (pScrn, &via_crtc_funcs);
                
        if (crtc == NULL)
        {
            ErrorF("xf86CrtcCreate fail.\n");
            return;
        }
        via_private_crtc = xnfcalloc (sizeof (VIACrtcPrivateRec), 1);
        via_private_crtc->RotateShadowOffset = 0;        
        crtc->driver_private = via_private_crtc;
    }            
}

#endif

