/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#ifdef HAVE_CONFIG_H            /*add for driver compiling on X11R7(FC5)*/
#include "config_via.h"
#endif

#ifndef _VIA_DRIVER_H_
#define _VIA_DRIVER_H_ 1

/* This Macro is used to switch using hw icon simulate hw cursor */
#define USEHwIconSimulateHwCursor

/*#define DEBUG_PRINT*/
#ifdef DEBUG_PRINT
#define DEBUG(x) x
#else
#define DEBUG(x)
#endif

/*#define RANDRV12_TEST_ENABLE    1*/

#include "vgaHW.h"
#include "xf86.h"
#include "xf86Resources.h"
#include "xf86_ansic.h"
#include "xf86Pci.h"
#include "xf86PciInfo.h"
#include "xf86_OSproc.h"
#include "compiler.h"
#include "xf86Cursor.h"
#include "mipointer.h"
#include "micmap.h"

#define USE_FB
#ifdef USE_FB
#include "fb.h"
#else
#include "cfb.h"
#include "cfb16.h"
#include "cfb32.h"
#endif

#include "xf86cmap.h"
#include "vbe.h"
#include "xaa.h"

#include "via_regs.h"
#include "via_bios.h"
#include "via_gpioi2c.h"

#include "drm.h"
#include "mm.h"
#ifdef XF86DRI
#define _XF86DRI_SERVER_
#include "sarea.h"
#include "dri.h"
#include "GL/glxint.h"
#include "via_dri.h"
#endif

#include <X11/extensions/randr.h>

#include "capture.h"
#include "VidStruc.h"

#define DRIVER_NAME     "via"
#define DRIVER_VERSION  "5.73.23"
#define VERSION_MAJOR   5       /* Linux Driver number */
#define VERSION_MINOR   72      /* Xorg Version supported max version number*/
#define VERSION_OS      23      /* 3D Family, H2: 21 / H5: 22, Trunk both H2/H5: 23 */
#define PATCHLEVEL      0xff    /* Driver sub version number, Trunk: 0xff*/
#define VIA_VERSION     ((VERSION_MAJOR<<24) | (VERSION_MINOR<<16) | PATCHLEVEL)    

#ifdef XORG_VERSION_CURRENT 
    #if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,9,0,0,0)
        #define _XORG_VER_6_9_LATER_
    #endif

    #if XORG_VERSION_CURRENT >= XF86_VERSION_NUMERIC(6,8,0,0,0)
        #define _XORG_VER_6_8_LATER_
    #endif    
#endif

#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,14,0)
#undef XFREE86_44
#else
#define XFREE86_44 1
#endif

#define VGAIN8(addr)            MMIO_IN8(pVia->MapBase+0x8000, addr)
#define VGAIN16(addr)           MMIO_IN16(pVia->MapBase+0x8000, addr)
#define VGAIN(addr)             MMIO_IN32(pVia->MapBase+0x8000, addr)

#define VGAOUT8(addr, val)      MMIO_OUT8(pVia->MapBase+0x8000, addr, val)
#define VGAOUT16(addr, val)     MMIO_OUT16(pVia->MapBase+0x8000, addr, val)
#define VGAOUT(addr, val)       MMIO_OUT32(pVia->MapBase+0x8000, addr, val)

#define INREG(addr)             MMIO_IN32(pVia->MapBase, addr)
#define OUTREG(addr, val)       MMIO_OUT32(pVia->MapBase, addr, val)
#define INREG16(addr)           MMIO_IN16(pVia->MapBase, addr)
#define OUTREG16(addr, val)     MMIO_OUT16(pVia->MapBase, addr, val)

#define VIA_PIXMAP_CACHE_SIZE       (2048 * 1024)
#define VIA_CURSOR_SIZE             (16 * 1024)
#define VIA_FB_CURSOR_SIZE          (8 * 1024)      /* reserve for frame buffer device driver */
#define VIA_VQ_SIZE                 (256 * 1024)
#define VIA_DOWN_SCALE_BUFFER_SIZE  (4*1024*1024)   /* 4MB, we need 3 buffer size */
#define	VIA_AGP_CMD_BUFFER_SIEZ     (128 * 1024)

/* used by get video memory size */
/* device ID */
#define CLE266                  0x3123
#define KM400                   0x3205
#define CN400_FUNCTION2         0x2259
#define CN400_FUNCTION3         0x3259
/* support VT3314 chipset */
#define CN700_FUNCTION2         0x2314
#define CN700_FUNCTION3         0x3208
/* VT3324 chipset */
#define CX700_FUNCTION2         0x2324
#define CX700_FUNCTION3         0x3324
/* VT3204 chipset*/
#define KM800_FUNCTION3         0x3204
/* VT3336 chipset*/
#define KM890_FUNCTION3         0x3336
/* VT3327 chipset*/
#define P4M890_FUNCTION3        0x3327
/* VT3293 chipset*/
#define CN750_FUNCTION3         0x3208
/* VT3364 chipset*/
#define P4M900_FUNCTION3        0x3364
/* VT3353 chipset*/
#define VX800_FUNCTION3         0x3353

/*for vt3336/vt3364/vt3293/vt3353(h5s1) 3D support*/
#define H5_UMA_CHIPID (pVia->ChipId==PCI_CHIP_VT3336 || pVia->ChipId == PCI_CHIP_VT3364 || pVia->ChipId == PCI_CHIP_VT3293 )
#define H6_UMA_CHIPID (pVia->ChipId == PCI_CHIP_VT3353)

/* vertical timings */
#define DISPLAY_END         0x04
#define BLANK_START         DISPLAY_END
#define SYNC_START          BLANK_START
#define SYNC_END            0x09
#define BLANK_END           SYNC_END
#define V_TOTAL             BLANK_END

#define MAX_CURS    64

struct ViaModePriv {
    char   id[12]; /* "via" */
};

static struct ViaModePriv ViaModePrivate = {
    { 'v', 'i', 'a', 0, 0, 0, 0, 0, 0, 0, 0, 0 },
};
#define MODEPREFIX(name) NULL, NULL, name, 0,M_T_DEFAULT
#define MODESUFFIX       0,0, 0,0,0,0,0,0,0, 0,0,0,0,0,0,FALSE,FALSE,\
    sizeof(struct ViaModePriv),(void *)&ViaModePrivate,0,0.0,0.0

/* Information of modeline. */
static DisplayModeRec ViaSupportModes[] = {
    /*  60 Hz */
    { MODEPREFIX("480x640"),     24820,   480,  504,  552,  624, 0,   640,  641,  644,  663, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("720x480"),     26700,   720,  736,  808,  896, 0,   480,  481,  484,  497, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },    
    { MODEPREFIX("720x576"),     32700,   720,  744,  816,  912, 0,   576,  577,  580,  597, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("800x480"),     40000,   800,  832,  960, 1056, 0,   480,  541,  545,  628, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("848x480"),     31500,   848,  864,  952, 1056, 0,   480,  481,  484,  497, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("856x480"),     31700,   856,  872,  960, 1064, 0,   480,  481,  484,  497, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("960x600"),     45980,   960, 1000, 1096, 1232, 0,   600,  601,  604,  622, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1000x600"),    48070,  1000, 1040, 1144, 1288, 0,   600,  601,  604,  622, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1024x512"),    41300,  1024, 1056, 1160, 1296, 0,   512,  513,  516,  531, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1024x600"),    48960,  1024, 1064, 1168, 1312, 0,   600,  601,  604,  622, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1088x612"),    52950,  1088, 1128, 1240, 1392, 0,   612,  613,  616,  634, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1152x720"),    67320,  1152, 1208, 1328, 1504, 0,   720,  721,  724,  746, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1200x720"),    70180,  1200, 1256, 1384, 1568, 0,   720,  721,  724,  746, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1280x600"),    61500,  1280, 1336, 1464, 1648, 0,   600,  601,  604,  622, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1280x720"),    74600,  1280, 1341, 1474, 1688, 0,   720,  721,  724,  746, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1280x768"),    80100,  1280, 1344, 1480, 1680, 0,   768,  769,  772,  795, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1360x768"),    85500,  1360, 1392, 1712, 1744, 0,   768,  783,  791,  807, 0, V_PHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1366x768"),    85860,  1366, 1440, 1584, 1800, 0,   768,  769,  772,  795, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1440x900"),   106470,  1440, 1520, 1672, 1904, 0,   900,  901,  904,  932, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1440x1050"),  126200,  1440, 1536, 1688, 1936, 0,  1050, 1051, 1054, 1087, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1600x900"),   119000,  1600, 1696, 1864, 2128, 0,   900,  901,  904,  932, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1600x1024"),  136360,  1600, 1704, 1872, 2144, 0,  1024, 1025, 1028, 1060, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1792x1344"),  202970,  1792, 1920, 2112, 2432, 0,  1344, 1345, 1348, 1391, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1856x1392"),  218570,  1856, 1992, 2192, 2528, 0,  1392, 1393, 1396, 1441, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("1920x1080"),  172900,  1920, 2043, 2249, 2578, 0,  1080, 1081, 1084, 1118, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },
    { MODEPREFIX("2048x1536"),  266950,  2048, 2200, 2424, 2800, 0,  1536, 1537, 1540, 1589, 0, V_NHSYNC | V_PVSYNC, MODESUFFIX },                      
    { MODEPREFIX(NULL), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, MODESUFFIX },
};


struct _vgaDdcSave {
    unsigned char cr03;
    unsigned char cr06;
    unsigned char cr07;
    unsigned char cr09;
    unsigned char cr10;
    unsigned char cr11;
    unsigned char cr12;
    unsigned char cr15;
    unsigned char cr16;
    unsigned char msr;
};


typedef struct {
    unsigned int    mode, refresh, resMode;
    int             countWidthByQWord;
    int             offsetWidthByQWord;

    unsigned char   Clock;

    /* Sequencer registers */
    unsigned char   SRRegs[256];
    
    /* CRTC registers */
    unsigned char   CRTCRegs[256];
    unsigned char   VT1636Regs[9];      /* We will save from Tx08 to Tx10. */

    /* HW Cursor registers */
    CARD32          dwCursorMode, dwCursorPOS, dwCursorORG, dwCursorBG, dwCursorFG;

} VIARegRec, *VIARegPtr;

typedef struct{
    int width;
    int height;
    int hotx;
    int hoty;
    CARD32 HwCursorBufStart;
    CARD32 HIBufStart;
    CARD32 CursorBG;
    CARD32 CursorFG;
    CARD32 CursorMC;
}VIACursorInfo, *VIACursorInfoPtr;

typedef struct{
    Bool Agp_Need_Fire;
    Bool Agp_IsH2;
    unsigned int* Agp_Cmd_Buf;
    unsigned int Agp_Cmd_Size;
}VIAAgpCmd, *VIAAgpCmdPtr;


typedef struct {
    Bool UseHwCursor;
    VIACursorInfo CursorInfo;
    VIAAgpCmd AgpCmd;
} VIAScreensPublicInfo, *VIAScreensPublicInfoPtr;

#define     EXPAND_BUF_CNT     2


typedef struct _VIA {
    VIARegRec           SavedReg;
    VIARegRec           ModeReg;
    VIAScreensPublicInfoPtr      pScreensPublicInfo;
    xf86CursorInfoPtr   CursorInfoRec;
    int                 stateMode;
    VIAModeInfoPtr      VIAModeList;
    Bool                ModeStructInit;
    int                 Bpp, Bpl, ScissB;
    unsigned            PlaneMask;

    unsigned long       videoRambytes;
    int                 videoRamKbytes;

    int                 FBFreeStart;
    int                 FBFreeEnd;
    int                 VQStart;
    int                 VQEnd;

    /* These are physical addresses. */
    unsigned long       FrameBufferBase;
    unsigned long       MmioBase;

    /* These are linear addresses. */
    unsigned char*      MapBase;
    unsigned char*      VidMapBase;
    unsigned char*      BltBase;
    unsigned char*      MapBaseDense;
    unsigned char*      FBBase;
    unsigned char*      FBStart;

    Bool                PrimaryVidMapped;
    int                 dacSpeedBpp;
    int                 minClock, maxClock;
    int                 MCLK, REFCLK, LCDclk;
    double              refclk_fact;

    /* Here are all the Options */
    Bool                VQEnable;
    Bool                pci_burst;
    Bool                NoPCIRetry;
    Bool                ForceSWCursor;
    Bool                NoAccel;
    Bool                NoXVideo;
    Bool                shadowFB;       

    CloseScreenProcPtr  CloseScreen;
    pciVideoPtr         PciInfo;
    PCITAG              PciTag;
    int                 Chipset;
    int                 ChipId;
    int                 ChipRev;
    vbeInfoPtr          pVbe;
    int                 EntityIndex;

    /* Support for shadowFB and rotation */
    unsigned char*      ShadowPtr;
    int                 ShadowPitch;

    /* For RandR v1.2 Rotation */
    int                 RotateShadowOffset[2];
    int                 NumOfShadowInUse;
    
    /* Support for XAA acceleration */
    XAAInfoRecPtr       AccelInfoRec;
    xRectangle          Rect;
    CARD32              SavedCmd;
	CARD32              SavedBpp;
    CARD32              SavedFgColor;
    CARD32              SavedBgColor;
    CARD32              SavedPattern0;
    CARD32              SavedPattern1;
    CARD32              SavedPatternAddr;

    /* Support for Int10 processing */
    xf86Int10InfoPtr    pInt10;

    /* BIOS Info Ptr */
    VIABIOSInfoPtr      pBIOSInfo;
    VGABIOSVERPtr       pBIOSVer;

    Bool                IsSuspending;
    Bool                IsVIAModeInitRunOnceDone;
    
    /* Support for DGA */
    int                 numDGAModes;
    DGAModePtr          DGAModes;
    Bool                DGAactive;
    int                 DGAViewportStatus;

    /* The various wait handlers. */
    int                 (*myWaitIdle)(struct _VIA*);

    /* I2C & DDC */
    I2CBusPtr           I2C_Port1;
    I2CBusPtr           I2C_Port2;
    xf86MonPtr          DDC1;
    xf86MonPtr          DDC2;

    /* MHS */
    Bool                IsSecondary;
    Bool                HasSecondary;

#ifdef XF86DRI
    Bool directRenderingEnabled;
    Bool useAgp;
    DRIInfoPtr pDRIInfo;
    int drmFD;
    int numVisualConfigs;
    __GLXvisualConfig* pVisualConfigs;
    VIAConfigPrivPtr pVisualConfigsPriv;
    unsigned long agpHandle;
    unsigned long registerHandle;
/* For AMD64 */
#ifndef __x86_64__
    CARD32 agpAddr;
#else
    CARD64 agpAddr;
#endif
    unsigned char *agpBase;
    unsigned int agpSize;
    Bool IsPCI;
    Bool drixinerama;
#endif

    CARD16  ActiveDevice;   /* if SAMM, non-equal pBIOSInfo->ActiveDevice */
    unsigned char       *HwCursorImage;
    unsigned char       *HwIconImage;
    unsigned char       *HIScalingPrimImage;
    unsigned char       *HIScalingSecImage;
    PVIDDATA pVidData;
    int     shmid;      /* shared memory ID */

    /*agp flush drm*/
    unsigned long agp_handle;/*map public agp  area*/

    /* Rotation */
    Bool    RandRRotation;
    int     SWRotate;           /* For Software rotation */
    Bool    IsHWRotateEnabled;  /* For Hardware rotation */
    Bool    DynamicRotation;
    int     RotateFBStart;
    int     RotateFBEnd;
    
    /* For MAMM case, VIA as secondary display device. */
    Bool    IsMAMMEnable; 
    
    /*for vt3336(h5s1) 3d support*/
    AddrConfPtr pAddrConf;
    ChipConfPtr pChipConf;
    DriConfPtr pDriConf;
    ScrnConfPtr pScrnConf;
    ServerInfoPtr pServerInfo;
    PciConfPtr pPciConf;
    unsigned long        hostBltHandle;    /*2d host bitblt map handle*/

    OsTimerPtr devicesTimer;
    OsTimerPtr eventTimer;      /* Add for checking pmevent */

    /* For hotkey timer */
    Bool       IsHotkeyTimerEnabled;
    OsTimerPtr hotkeyTimer;    
    CARD32     deviceSequence[4];

    /* For Down Scale Used */
    int     DownScaleBufferStartAddr;
    
} VIARec, *VIAPtr;

#ifdef RANDRV12_TEST_ENABLE
typedef struct _VIACrtcPrivateRec {
    int     RotateShadowOffset;    
    int     RotateDegree;
} VIACrtcPrivateRec, *VIACrtcPrivatePtr;

#define VIACrtcPrivate(c) ((VIACrtcPrivatePtr) (c)->driver_private)
#endif



 
typedef struct
{
    Bool IsDRIEnabled;

    Bool HasSecondary;
    Bool BypassSecondary;
    Bool IsSecondaryRestored;
    Bool RestorePrimary;

    ScrnInfoPtr pSecondaryScrn;
    ScrnInfoPtr pPrimaryScrn;
}VIAEntRec, *VIAEntPtr;

/* structure for passing information to UpdateOverlay fn */
typedef struct _RECTL
{
    long     left;
    long     top;
    long     right;
    long     bottom;
} RECTL, *RECTLPtr;

/* Shortcuts.  These depend on a local symbol "pVia". */
#define WaitIdle()      pVia->myWaitIdle(pVia)
#define VIAPTR(p)       ((VIAPtr)((p)->driverPrivate))


/* Prototypes. */
void VIAAdjustFrame(int scrnIndex, int y, int x, int flags);
Bool VIASwitchMode(int scrnIndex, DisplayModePtr mode, int flags);
Bool VIACheckLCDExpand(ScrnInfoPtr pScrn);
void VIADebugBreak(void);
#ifdef XFree86LOADER
Bool VIALoadVideoOption(VIAPtr pVia,LPVIAGRAPHICINFO lpVIAGraphicInfo);
#endif

/* In via_cursor.c. */
Bool VIAHWCursorInit(ScreenPtr pScreen);
void VIAShowCursor(ScrnInfoPtr);
void VIAHideCursor(ScrnInfoPtr);


/* In via_accel.c. */
Bool VIAInitAccel(ScreenPtr);
void VIAInitialize2DEngine(ScrnInfoPtr);
void VIAAccelSync(ScrnInfoPtr);


/* In via_shadow.c */
void VIAPointerMoved(int index, int x, int y);
void VIARefreshArea(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
void VIARefreshArea8(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
void VIARefreshArea16(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
void VIARefreshArea24(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
void VIARefreshArea32(ScrnInfoPtr pScrn, int num, BoxPtr pbox);


/* In via_bios.c */
Bool VIAPrimarySetMode(ScrnInfoPtr pScrn);
Bool VIAGetBIOSInfoFromROM(VIAPtr pVia, unsigned char flags);


/* In via_dga.c */
Bool VIADGAInit(ScreenPtr);

/* In via_i2c.c */
Bool VIAI2CInit(ScrnInfoPtr pScrn);

/* In via_gpioi2c.c */
Bool GPIOI2C_Write(VIABIOSInfoPtr pBIOSInfo, int SubAddress, CARD8 Data);
Bool GPIOI2C_Read(VIABIOSInfoPtr pBIOSInfo, int SubAddress, CARD8 *Buffer, int BufferLen);
Bool GPIOI2C_ReadByte(VIABIOSInfoPtr pBIOSInfo, int SubAddress, CARD8 *Buffer);
Bool GPIOI2C_Initial(VIABIOSInfoPtr pBIOSInfo, CARD8 SlaveDevice);

/*In via_video.c*/
void viaInitVideo(ScreenPtr pScreen);
void viaExitVideo(ScrnInfoPtr pScrn);
void viaResetVideo(ScrnInfoPtr pScrn);
void viaSaveVideo(ScrnInfoPtr pScrn);
void viaRestoreVideo(ScrnInfoPtr pScrn);

/* In Hw.c */
void VIADIPortPadOn(int DIPort);
void VIADIPortPadOff(int DIPort);


/* Global Variables */
CARD8   *MMIOMapBase;

#endif  /* _VIA_DRIVER_H_ */
