 /*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

 
/*
 * I N C L U D E S
 */
#include "xf86.h"
#include "xf86xv.h"
#include "Xv.h"
#include "xaa.h"
#include "xaalocal.h"
#include "dixstruct.h"
#include "xf86_OSproc.h"
#include "xf86Resources.h"
#include "xf86_ansic.h"
#include "compiler.h"
#include "xf86PciInfo.h"
#include "xf86Pci.h"
#include "xf86fbman.h"
#include "regionstr.h"
#include "via_driver.h"
#include "via_video.h"

#include "debug.h"      /* for DBG_DD */
#include "ddmpeg.h"
#include "capture.h"
#include "via.h"
#include "overlay.h"
#include "VidInit.h"
#include "VidStruc.h"
#include "fourcc.h"

/*
 * D E F I N E
 */
#define OFF_DELAY       200  /* milliseconds */
#define FREE_DELAY      60000
#define PARAMSIZE       1024
#define SLICESIZE       65536
#define OFF_TIMER       0x01
#define FREE_TIMER      0x02
#define TIMER_MASK      (OFF_TIMER | FREE_TIMER)

#define LOW_BAND 0x0CB0
#define MID_BAND 0x1f10

#define  XV_IMAGE          0
#define  NTSC_COMPOSITE    1
#define  NTSC_TUNER        2
#define  NTSC_SVIDEO       3
#define  PAL_SVIDEO        4
#define  PAL_60_COMPOSITE  5
#define  PAL_60_TUNER      6
#define  PAL_60_SVIDEO     7
#define MAKE_ATOM(a) MakeAtom(a, sizeof(a) - 1, TRUE)

#define  IN_FLIP     (viaVidEng->ramtab & 0x00000003)
#define  IN_DISPLAY  (viaVidEng->interruptflag & 0x00000200)
#define  IN_VBLANK   (!IN_DISPLAY)

#define MAXWIDTH 2048
#define MAXHEIGHT 2048
#ifndef REGION_NULL
#define REGION_NULL(_pScreen,_pReg)\
    REGION_INIT(_pScreen,_pReg,NullBox,1)
#endif

#ifndef REGION_EQUAL
#define  REGION_EQUAL(_pScreen, _pReg1, _pReg2)\
    miRegionEqual(_pReg1,_pReg2)	
#endif

#ifndef XvExtension
void viaInitVideo(ScreenPtr pScreen) {}
#else

/*
 *  F U N C T I O N   D E C L A R A T I O N
 */
XF86VideoAdaptorPtr viaSetupImageVideoG(ScreenPtr);
static void viaStopVideoG(ScrnInfoPtr, pointer, Bool);
static int viaSetPortAttributeG(ScrnInfoPtr, Atom, INT32, pointer);
static int viaGetPortAttributeG(ScrnInfoPtr, Atom ,INT32 *, pointer);
static void viaQueryBestSizeG(ScrnInfoPtr, Bool,
        short, short, short, short, unsigned int *, unsigned int *, pointer);

#ifdef NEW_XV_INTERFACE
static int viaPutImageG( ScrnInfoPtr,
        short, short, short, short, short, short, short, short,
        int, unsigned char*, short, short, Bool, RegionPtr, pointer, DrawablePtr );
static int viaReputImageG( ScrnInfoPtr,
        short, short, RegionPtr, pointer, DrawablePtr );
static int viaPutVideo(ScrnInfoPtr ,
    short , short , short , short ,short , short , short , short ,
    RegionPtr , pointer, DrawablePtr );
static int viaPutStill(ScrnInfoPtr ,
    short , short , short , short ,short , short , short , short ,
    RegionPtr , pointer, DrawablePtr );
static int viaGetVideo(ScrnInfoPtr ,
    short , short , short , short ,short , short , short , short ,
    RegionPtr , pointer, DrawablePtr );
static int viaGetStill(ScrnInfoPtr ,
    short , short , short , short ,short , short , short , short ,
    RegionPtr , pointer, DrawablePtr );
#else
static int viaPutImageG( ScrnInfoPtr,
        short, short, short, short, short, short, short, short,
        int, unsigned char*, short, short, Bool, RegionPtr, pointer);
static int viaReputImageG( ScrnInfoPtr,
        short, short, RegionPtr, pointer );
static int viaPutVideo(ScrnInfoPtr ,
    short , short , short , short ,short , short , short , short ,
    RegionPtr , pointer );
static int viaPutStill(ScrnInfoPtr ,
    short , short , short , short ,short , short , short , short ,
    RegionPtr , pointer );
static int viaGetVideo(ScrnInfoPtr ,
    short , short , short , short ,short , short , short , short ,
    RegionPtr , pointer );
static int viaGetStill(ScrnInfoPtr ,
    short , short , short , short ,short , short , short , short ,
    RegionPtr , pointer );
#endif

static int viaQueryImageAttributesG(ScrnInfoPtr,
        int, unsigned short *, unsigned short *,  int *, int *);

unsigned long CreateOVSurface(ScrnInfoPtr pScrn,
        viaPortPrivPtr pPriv, int fd, int fourcc, short width, short height);

void DestroyOVSurface(ScrnInfoPtr pScrn, viaPortPrivPtr pPriv, int fd);

void StopOVerlay(ScrnInfoPtr pScrn, int fd);

void SetTunerChannel (viaPortPrivPtr pChanPriv, INT32 frequency);

__inline unsigned long VIACreatOVSurface(ScrnInfoPtr pScrn,  viaPortPrivPtr pPriv,
        int id, short src_w, short src_h, short width, short height);
__inline int VIAOVFlip(ScrnInfoPtr pScrn, viaPortPrivPtr pPriv);
__inline int VIAOVUpdateOverlay(ScrnInfoPtr pScrn,  viaPortPrivPtr pPriv, 
        LPDDUPDATEOVERLAY lpUpdateOverlay, short drw_x, short drw_y, short src_w,
        short src_h, short drw_w, short drw_h);
#ifdef NEW_XV_INTERFACE
__inline void VIADrawColorKeyRegion(ScrnInfoPtr pScrn, viaPortPrivPtr pPriv, RegionPtr clipBoxes,DrawablePtr pDraw);
#else
__inline void VIADrawColorKeyRegion(ScrnInfoPtr pScrn, viaPortPrivPtr pPriv, RegionPtr clipBoxes);
#endif

/*
 *  E X T E R N   F U N C T I O N S
 */

extern unsigned int via_xf86_version_current;

/*
 *  G L O B A L S
 */
static Atom xvBrightness, xvContrast, xvColorKey,xvHue,xvSaturation
            ,xvEncoding ,xvMute, xvVolume, xvFreq, xvAudioCtrl
            , xvOverlayStatus, xv_autopaint_colorkey, xvHotkeyTimer
            , xvDeviceControl, xvTVStandard, xvDeviceDriverSync;


static unsigned long dwFrameNum[2] = {0, 0};    /* for startaddr select */
viaPortPrivRec *gviaPortPriv[XV_PORT_NUM];

/*  E X T E R N   G L O B A L S -----------------------------------------*/

/*
 *  S T R U C T S
 */
/* client libraries expect an encoding */
static XF86VideoEncodingRec DummyEncoding[8] =
{
    { XV_IMAGE        , "XV_IMAGE",MAXWIDTH, MAXHEIGHT,{1, 1}},
    { NTSC_COMPOSITE  , "ntsc-composite",720, 480, { 1001, 60000 }},
    { NTSC_TUNER      , "ntsc-tuner",720, 480, { 1001, 60000 }},
    { NTSC_SVIDEO     , "ntsc-svideo",720, 480, { 1001, 60000 }},
    { PAL_SVIDEO      , "pal-svideo",720, 576, { 1, 50 }},
    { PAL_60_COMPOSITE, "pal_60-composite", 704, 576, { 1, 50 }},
    { PAL_60_TUNER    , "pal_60-tuner", 720, 576, { 1, 50 }},
    { PAL_60_SVIDEO   , "pal_60-svideo",720, 576, { 1, 50 }}
};

#define NUM_FORMATS_G 9

static XF86VideoFormatRec FormatsG[NUM_FORMATS_G] =
{
    { 8, TrueColor }, /* Dithered */
    { 8, PseudoColor }, /* Using .. */
    { 8, StaticColor },
    { 8, GrayScale },
    { 8, StaticGray }, /* .. TexelLUT */
    {16, TrueColor},
    {24, TrueColor},
    {16, DirectColor},
    {24, DirectColor}
};

#define NUM_ATTRIBUTES_G 16

static XF86AttributeRec AttributesG[NUM_ATTRIBUTES_G] =
{
    {XvSettable | XvGettable, 0, 10000, "XV_BRIGHTNESS"},
    {XvSettable | XvGettable, 0, 20000, "XV_CONTRAST"},
    {XvSettable | XvGettable, 0, 20000, "XV_SATURATION"},
    {XvSettable | XvGettable, -180, 180,"XV_HUE"},
    {XvGettable,0,1,"XV_OVERLAYSTATUS"},
    {XvSettable | XvGettable, 0, (1 << 24) - 1, "XV_COLORKEY"},
    {XvSettable | XvGettable,0,255,"XV_MUTE"},
    {XvSettable | XvGettable,0,255,"XV_VOLUME"},
    {XvSettable | XvGettable,0, 255,"XV_ENCODING"},
    {XvSettable,0,-1,"XV_FREQ"},
    {XvSettable,0,2,"XV_AUDIOCTRL"},
    {XvSettable | XvGettable,0,1,"XV_AUTOPAINT_COLORKEY"},
    {XvSettable | XvGettable, 0, 1, "XV_HOTKEYTIMER"},
    {XvSettable | XvGettable, 0, 0x2F, "XV_DEVICECONTROL"},
    {XvSettable | XvGettable, 0, 0x4FFFF, "XV_TVSTANDARD"},
    {XvSettable | XvGettable, 0, 1, "XV_DEVICEDRVSYNC"}   
};

#define NUM_ATTRIBUTES_G_CLEAX 11

static XF86AttributeRec AttributesG_CLEAX[NUM_ATTRIBUTES_G_CLEAX] =
{
    {XvSettable | XvGettable, 0, (1 << 24) - 1, "XV_COLORKEY"},
    {XvSettable | XvGettable,0,255,"XV_MUTE"},
    {XvSettable | XvGettable,0,255,"XV_VOLUME"},
    {XvSettable | XvGettable,0, 255,"XV_ENCODING"},
    {XvSettable,0,-1,"XV_FREQ"},
    {XvSettable,0,2,"XV_AUDIOCTRL"},
    {XvSettable | XvGettable,0,1,"XV_AUTOPAINT_COLORKEY"},
    {XvSettable | XvGettable, 0, 1, "XV_HOTKEYTIMER"},
    {XvSettable | XvGettable, 0, 0x2F, "XV_DEVICECONTROL"},
    {XvSettable | XvGettable, 0, 0x4FFFF, "XV_TVSTANDARD"},
    {XvSettable | XvGettable, 0, 1, "XV_DEVICEDRVSYNC"}
};

#define NUM_IMAGES_G 3

static XF86ImageRec ImagesG[NUM_IMAGES_G] =
{
    XVIMAGE_YUY2,
    XVIMAGE_YV12,
    XVIMAGE_I420
};

static char * XVPORTNAME[1] =
{
    "XV_SWOV"
};


/*
 *  F U N C T I O N
 */

void viaResetVideo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    volatile video_via_regs *viaVidEng = NULL;
    CARD32  dwTemp;

    DBG_DD((" via_video.c : viaResetVideo: \n"));
    viaVidEng = (volatile video_via_regs *)pVia->VidMapBase;

    /* in CLE266Ax, 0x230[21] is FIFO length extended control. (others is reserved)
     * so, we do not touch it in this */
    dwTemp = viaVidEng->video1_ctl;
    viaVidEng->video1_ctl = dwTemp & 0x00200000;

    viaVidEng->video3_ctl = 0;
    viaVidEng->compose    = 0x80000000;
    viaVidEng->compose    = 0x40000000;

}

static CARD32 dwV1, dwV3;
void viaSaveVideo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    volatile video_via_regs  *viaVidEng = NULL;

    DBG_DD((" via_video.c : viaSaveVideo : \n"));

    viaVidEng = (volatile video_via_regs *)pVia->VidMapBase;
    dwV1 = viaVidEng->video1_ctl;
    dwV3 = viaVidEng->video3_ctl;
    viaVidEng->video1_ctl = dwV1 & 0x00200000;;
    viaVidEng->video3_ctl = 0;
    viaVidEng->compose    = 0x80000000;
    viaVidEng->compose    = 0x40000000;
}

void viaRestoreVideo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    volatile video_via_regs  *viaVidEng = NULL;

    DBG_DD((" via_video.c : viaRestoreVideo : \n"));

    viaVidEng = (volatile video_via_regs *)pVia->VidMapBase;
    viaVidEng->video1_ctl = dwV1 ;
    viaVidEng->video3_ctl = dwV3 ;
    viaVidEng->compose    = 0x80000000;
    viaVidEng->compose    = 0x40000000;
}

void viaExitVideo(ScrnInfoPtr pScrn)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    volatile video_via_regs  *viaVidEng = NULL;
    CARD32  dwTemp;
    DBG_DD((" via_video.c : viaExitVideo : \n"));
    /*destroy XV reserved memory structure here.*/
    viaVidEng = (volatile video_via_regs *)pVia->VidMapBase;
    /* in CLE266Ax, 0x230[21] is FIFO length extended control. (others is reserved)
     * so, we do not touch it in this */
    dwTemp = viaVidEng->video1_ctl;
    viaVidEng->video1_ctl = dwTemp & 0x00200000;
    viaVidEng->video3_ctl = 0;
    viaVidEng->compose    = 0x80000000;
    viaVidEng->compose    = 0x40000000;
}


XF86VideoAdaptorPtr adaptPtr[XV_PORT_NUM];
void viaInitVideo(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr  pVia = VIAPTR(pScrn);
    XF86VideoAdaptorPtr *adaptors, *newAdaptors = NULL;
    XF86VideoAdaptorPtr newAdaptor = NULL;
    int num_adaptors;

    DBG_DD((" via_video.c : viaInitVideo : \n"));
    DBG_DD((" via_video.c : Screen[%d]\n", pScrn->scrnIndex));

    num_adaptors = xf86XVListGenericAdaptors(pScrn, &adaptors);
    DBG_DD((" via_video.c : num_adaptors : %d\n",num_adaptors));

    if (!pVia->NoXVideo)
    {
        newAdaptor = viaSetupImageVideoG(pScreen);
    }
    else
    {
        if(num_adaptors)
        {
            xf86XVScreenInit(pScreen, adaptPtr, num_adaptors);
            xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Using generic Xv\n" );
        }
        return;
    }

    if(newAdaptor)
    {
        if(!num_adaptors)
        {
            num_adaptors = 1;
            adaptors = &newAdaptor; 
        }
        else
        {
            DBG_DD((" via_video.c : viaInitVideo : Warning !!! MDS not supported yet !\n"));
            newAdaptors = xalloc((num_adaptors + 1) * sizeof(XF86VideoAdaptorPtr*));
            if(newAdaptors)
            {
                memcpy(newAdaptors, adaptors, num_adaptors *
                                        sizeof(XF86VideoAdaptorPtr));
                newAdaptors[num_adaptors] = newAdaptor;
                adaptors = newAdaptors;
                num_adaptors++;
            }
        }
    }

    if(num_adaptors)
        xf86XVScreenInit(pScreen, adaptPtr, XV_PORT_NUM);

    if(newAdaptors)
        xfree(newAdaptors);
}

XF86VideoAdaptorPtr
viaSetupImageVideoG(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    VIAPtr  pVia = VIAPTR(pScrn);
    PVIDDATA pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    DevUnion  *         pdevUnion[XV_PORT_NUM];
    int  i;

    DBG_DD((" via_video.c : viaSetupImageVideoG: \n"));
    DBG_DD((" via_video.c : Screen[%d]\n", pScrn->scrnIndex));

    /* CLE_AX platform doesn't support Overlay color setting */
    if (lpVIAGraphicInfo->dwProjectIDRevision != UNICHROME_VT3123AX)
    {
        xvBrightness      = MAKE_ATOM("XV_BRIGHTNESS");
        xvContrast        = MAKE_ATOM("XV_CONTRAST");
        xvSaturation      = MAKE_ATOM("XV_SATURATION");
        xvHue             = MAKE_ATOM("XV_HUE");
        xvOverlayStatus   = MAKE_ATOM("XV_OVERLAYSTATUS");
    }
    xvColorKey        = MAKE_ATOM("XV_COLORKEY");
    xvMute            = MAKE_ATOM("XV_MUTE");
    xvVolume          = MAKE_ATOM("XV_VOLUME");
    xvEncoding        = MAKE_ATOM("XV_ENCODING");
    xvFreq            = MAKE_ATOM("XV_FREQ");
    xvAudioCtrl       = MAKE_ATOM("XV_AUDIOCTRL");
    xv_autopaint_colorkey = MAKE_ATOM("XV_AUTOPAINT_COLORKEY");
    xvHotkeyTimer     = MAKE_ATOM("XV_HOTKEYTIMER");
    xvDeviceControl   = MAKE_ATOM("XV_DEVICECONTROL");
    xvTVStandard      = MAKE_ATOM("XV_TVSTANDARD");
    xvDeviceDriverSync   = MAKE_ATOM("XV_DEVICEDRVSYNC");

    /* AllocatePortPriv();*/
    for ( i = 0; i< XV_PORT_NUM; i ++ )
    {
        if(!(adaptPtr[i] = xf86XVAllocateVideoAdaptorRec(pScrn)))
            return NULL;
        if(XV_SWOV_PORTID == i){
            gviaPortPriv[i] =  (viaPortPrivPtr)xcalloc(2, sizeof(viaPortPrivRec));
        }else{
            gviaPortPriv[i] = (viaPortPrivPtr)xcalloc(1, sizeof(viaPortPrivRec));
        }
        if (!gviaPortPriv[i])
        {
            DBG_DD((" via_video.c : Fail to allocate gviaPortPriv: \n"));
        }
        else
        {
            DBG_DD((" via_video.c : gviaPortPriv[%d] = 0x%08x \n"
                    , i,gviaPortPriv[i]));
        }

        adaptPtr[i]->type = XvInputMask | XvWindowMask | XvImageMask | XvVideoMask | XvStillMask;
        adaptPtr[i]->flags = VIDEO_OVERLAID_IMAGES | VIDEO_CLIP_TO_VIEWPORT;
        adaptPtr[i]->name = XVPORTNAME[i];
        adaptPtr[i]->nEncodings = 8;
        adaptPtr[i]->pEncodings = DummyEncoding;
        adaptPtr[i]->nFormats = sizeof(FormatsG) / sizeof(FormatsG[0]);
        adaptPtr[i]->pFormats = FormatsG;

        adaptPtr[i]->nPorts = 2;
        if(XV_SWOV_PORTID == i){
            pdevUnion[i] = (DevUnion  *)xcalloc(2, sizeof(DevUnion) );
            adaptPtr[i]->pPortPrivates = pdevUnion[i];
            pdevUnion[i]->ptr = (pointer) gviaPortPriv[i];
            (pdevUnion[i] + 1)->ptr = (pointer) (gviaPortPriv[i] + 1);
        }else {
            pdevUnion[i] = (DevUnion  *)xcalloc(1, sizeof(DevUnion) );
            adaptPtr[i]->pPortPrivates = pdevUnion[i];
            adaptPtr[i]->pPortPrivates->ptr = (pointer) gviaPortPriv[i];
        }

        /* CLE_AX platform doesn't support Overlay color setting */
        if (lpVIAGraphicInfo->dwProjectIDRevision == UNICHROME_VT3123AX)
        {
            adaptPtr[i]->nAttributes = NUM_ATTRIBUTES_G_CLEAX;
            adaptPtr[i]->pAttributes = AttributesG_CLEAX;
        }
        else
        {
            adaptPtr[i]->nAttributes = NUM_ATTRIBUTES_G;
            adaptPtr[i]->pAttributes = AttributesG;
        }

        adaptPtr[i]->nImages = NUM_IMAGES_G;
        adaptPtr[i]->pImages = ImagesG;
        adaptPtr[i]->PutVideo = viaPutVideo;
        adaptPtr[i]->PutStill = viaPutStill;
        adaptPtr[i]->GetVideo = viaGetVideo;
        adaptPtr[i]->GetStill = viaGetStill;
        adaptPtr[i]->StopVideo = viaStopVideoG;
        adaptPtr[i]->SetPortAttribute = viaSetPortAttributeG;
        adaptPtr[i]->GetPortAttribute = viaGetPortAttributeG;
        adaptPtr[i]->QueryBestSize = viaQueryBestSizeG;
        adaptPtr[i]->PutImage = viaPutImageG;

        /* On FC7, gnome, fvwm, blackbox, SAMM, xv path, move video window X crash, KDE OK. */
        adaptPtr[i]->ReputImage= 0;        
        
        adaptPtr[i]->QueryImageAttributes = viaQueryImageAttributesG;
        if(XV_SWOV_PORTID == i){
            gviaPortPriv[i]->colorKey= lpVIAGraphicInfo->VideoColorKey;
            gviaPortPriv[i]->brightness = 5000;
            gviaPortPriv[i]->saturation = 10000;
            gviaPortPriv[i]->contrast = 10000;
            gviaPortPriv[i]->hue = 0;
            gviaPortPriv[i]->xv_portnum = XV_SWOV_PORTID;
            gviaPortPriv[i]->autopaint_colorkey = TRUE;
            gviaPortPriv[i]->fd = -1;
            (gviaPortPriv[i] + 1)->colorKey= lpVIAGraphicInfo->VideoColorKey -1;
            (gviaPortPriv[i] + 1)->brightness = 5000;
            (gviaPortPriv[i] + 1)->saturation = 10000;
            (gviaPortPriv[i] + 1)->contrast = 10000;
            (gviaPortPriv[i] + 1)->hue = 0;
            (gviaPortPriv[i] + 1)->xv_portnum = XV_SWOV_PORTID2;
            (gviaPortPriv[i] + 1)->autopaint_colorkey = TRUE;
            (gviaPortPriv[i] + 1)->fd = -1;
        }else{
            gviaPortPriv[i]->colorKey = lpVIAGraphicInfo->VideoColorKey;
            gviaPortPriv[i]->brightness = 5000;
            gviaPortPriv[i]->saturation = 10000;
            gviaPortPriv[i]->contrast = 10000;
            gviaPortPriv[i]->hue = 0;
            gviaPortPriv[i]->xv_portnum = i;
            gviaPortPriv[i]->autopaint_colorkey = TRUE;
            gviaPortPriv[i]->fd = -1;
        }

        if (via_xf86_version_current < XF86_VERSION_NUMERIC(4,3,99,14,0))
        {
            REGION_INIT(pScreen, &gviaPortPriv[i]->clip, NullBox, 0);        
        }    
        else
        {
            REGION_NULL(pScreen, &gviaPortPriv[i]->clip);
        }    
        
    } /* End of for */


    viaResetVideo(pScrn);

    return adaptPtr[0];

}


static Bool
RegionsEqual(RegionPtr A, RegionPtr B)
{
    int *dataA, *dataB;
    int num;

    num = REGION_NUM_RECTS(A);
    if(num != REGION_NUM_RECTS(B))
        return FALSE;

    if((A->extents.x1 != B->extents.x1) ||
       (A->extents.x2 != B->extents.x2) ||
       (A->extents.y1 != B->extents.y1) ||
       (A->extents.y2 != B->extents.y2))
        return FALSE;

    dataA = (int*)REGION_RECTS(A);
    dataB = (int*)REGION_RECTS(B);

    while(num--)
    {
        if((dataA[0] != dataB[0]) || (dataA[1] != dataB[1]))
            return FALSE;

        dataA += 2;
        dataB += 2;
    }

    return TRUE;
}


unsigned long CreateOVSurface(ScrnInfoPtr pScrn, viaPortPrivPtr pPriv, int fd, int fourcc, short width, short height)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    LPDDSURFACEDESC lpSurfaceDesc = &pPriv->SurfaceDesc;
    PVIDDATA pVidData = pVia->pVidData;
    LPDDLOCK lpddLock = &pVidData->ddLock;
    unsigned long handle = pVidData->VMI_Info.handle;
    LPSWDEVICE  lpSWDevice  = &(pVidData->SWDevice[handle]);
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;

    /*Added for SAMM Mode*/
    unsigned char* FBBase;
    unsigned char* FBStart;
    ScrnInfoPtr pScrn0 = xf86Screens[0];
    VIAPtr pVia0 = VIAPTR(pScrn0);

    DBG_DD((" via_video.c : CreateOVSurface fourcc=%p\n",fourcc));

    if (pVia->IsSecondary)
    {
        /* For original SAMM case, screen 1 just use the same frame buffer base as screen 0. */
        FBBase = pVia0->FBBase;
        FBStart = pVia0->FBStart;
    }
    else
    {
        FBBase = pVia->FBBase;
        FBStart = pVia->FBStart;
    }
    
    lpSurfaceDesc->dwWidth  = (unsigned long)width;
    lpSurfaceDesc->dwHeight = (unsigned long)height;
    lpSurfaceDesc->dwBackBufferCount =1;

    switch( lpVIAGraphicInfo->dwDeviceID )
    {        
        case VIA_DEVICE_CLE1:
        case VIA_DEVICE_CLE2:
        case VIA_DEVICE_VT3204:
        case VIA_DEVICE_VT3314:      
            lpSurfaceDesc->dwFourCC = ((FOURCC_YV12 == fourcc) ? fourcc : FOURCC_YUY2);
            break;                     
        default:
            lpSurfaceDesc->dwFourCC = ((FOURCC_NV12 == fourcc) ? fourcc : FOURCC_YUY2);
            break;
    }
    
    vfLoadColorSetting(pVidData,pPriv);
    if(pVia->pBIOSInfo->SAMM)
    {
         gviaPortPriv[0]->SurfaceDesc.dwFourCC = lpSurfaceDesc->dwFourCC;
    }

     /*when create surface fails,fpCreateSurface return none zero*/
    if (fpCreateSurface(pVidData, lpSurfaceDesc)!=0)
        return FALSE;

    lpSWDevice->lpSWOverlaySurface[0] = FBBase + lpSWDevice->dwSWPhysicalAddr[0];
    lpSWDevice->lpSWOverlaySurface[1] = FBBase + lpSWDevice->dwSWPhysicalAddr[1];
    DBG_DD(("// lpSWOverlaySurface[0]: %p\n", lpSWDevice->lpSWOverlaySurface[0]));
    DBG_DD(("// lpSWOverlaySurface[1]: %p\n", lpSWDevice->lpSWOverlaySurface[1]));

    return TRUE;
}


void DestroyOVSurface(ScrnInfoPtr pScrn,  viaPortPrivPtr pPriv, int fd)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    LPDDSURFACEDESC lpSurfaceDesc = &pPriv->SurfaceDesc;
    PVIDDATA pVidData = pVia->pVidData;
    unsigned long handle = pVidData->VMI_Info.handle;
    LPVIDEOSTATUS lpVidStatus = &(pVidData->VidStatus);
    
     if(pVia->pBIOSInfo->SAMM)
    {
        lpSurfaceDesc->dwFourCC =   gviaPortPriv[0]->SurfaceDesc.dwFourCC;
    }
    
    DBG_DD((" via_video.c : Destroy SW Overlay Surface, fourcc =0x%08x : \n", lpSurfaceDesc->dwFourCC));
    DBG_DD((" via_video.c : Screen[%d]\n", pScrn->scrnIndex));

    if (lpVidStatus->dwVideoFlagSW[handle])
    {
        DBG_DD((" via_video.c : Destroy SW Overlay Surface, dwVideoFlagSW =0x%08x : \n",
            lpVidStatus->dwVideoFlagSW[handle]));
    }
    else
    {
        DBG_DD((" via_video.c : No SW Overlay Surface Destroyed, dwVideoFlagSW =0x%08x : \n",
            lpVidStatus->dwVideoFlagSW[handle]));
        return;
    }

    fpDestroySurface(pVidData, lpSurfaceDesc);

    return;
}


void  StopOVerlay(ScrnInfoPtr pScrn, int fd)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    PVIDDATA pVidData = pVia->pVidData;
    unsigned long handle = pVidData->VMI_Info.handle;
    LPDDUPDATEOVERLAY    lpUpdateOverlay = &(pVidData->UpdateOverlayRec);
    LPVIDEOSTATUS lpVidStatus = &(pVidData->VidStatus);
    lpUpdateOverlay->dwFourcc = FOURCC_YUY2;

    if (lpVidStatus->dwVideoFlagSW[handle])
    {
        lpUpdateOverlay->dwFlags  = DDOVER_HIDE;
        UpdateOverlay(pVidData, lpUpdateOverlay);
    }
}


static void
viaStopVideoG(ScrnInfoPtr pScrn, pointer data, Bool exit)
{
    VIAPtr  pVia = VIAPTR(pScrn);
    viaPortPrivPtr pPriv = (viaPortPrivPtr)data;
    PVIDDATA pVidData = pVia->pVidData;
    if(XV_SWOV_PORTID == pPriv->xv_portnum) pVidData->VMI_Info.handle=0;
    else if(XV_SWOV_PORTID2 == pPriv->xv_portnum) pVidData->VMI_Info.handle=1;
    int fd = -1;
    int zero=0;

    DBG_DD((" via_video.c : viaStopVideoG: exit=%d\n", exit));

    switch ( pPriv->xv_portnum )
    {       
        case COMMAND_FOR_SWOV:
        case XV_SWOV_PORTID2:
            REGION_EMPTY(pScrn->pScreen, &pPriv->clip);
            if(exit)
            {
                StopOVerlay(pScrn, fd);
                DestroyOVSurface(pScrn, pPriv, fd);
    
                dwFrameNum[pVidData->VMI_Info.handle] = 0; 
                pPriv->old_drw_x= 0;
                pPriv->old_drw_y= 0;
                pPriv->old_drw_w= 0;
                pPriv->old_drw_h= 0;
            }
            else
            {
                /*StopOVerlay(pScrn, fd);*/
            }
            break;
        default:
            DBG_DD((" via_video.c : viaStopVideoG: Shall not happen! \n"));
            break;
    }             
}


/****************************************************************************
 *  SetTunerChannel                                                       *
 *  Function: Sets the tuner to a requested channel                         *
 *  Inputs: CARD16 channel - the tuner channel to be set.                   *
 *  Outputs: NONE                                                          *
 ****************************************************************************/
void SetTunerChannel (viaPortPrivPtr pChanPriv, INT32 frequency)
{

    LPVIASETTUNERDATA lpTunerParam =  (LPVIASETTUNERDATA)xalloc(sizeof(VIASETTUNERDATA));
    short    divider = 0;

    switch(pChanPriv->dwEncoding)
    {
        case PAL_60_COMPOSITE :
        case PAL_60_TUNER     :
        case PAL_60_SVIDEO    :
            divider=633+(short)frequency;
            break;
        case NTSC_COMPOSITE   :
        case NTSC_TUNER       :
        case NTSC_SVIDEO      :
            divider=733+(short)frequency;
            break;
        default:
            divider=frequency;
    }

    lpTunerParam->divider = divider;
    lpTunerParam->control = 0x8E00;

    if ( divider <= LOW_BAND )
    {
        lpTunerParam->control = lpTunerParam->control | 0xA0;
    }
    else
    {
        if ( divider <= MID_BAND )
        {
            lpTunerParam->control = lpTunerParam->control | 0x90;
        }
        else
        {
            lpTunerParam->control = lpTunerParam->control | 0x30;
        }
    }

    DBG_DD((" via_video.c : SetTunerChannel : Divider = 0x%08x, Control= 0x%08x, \n",
        lpTunerParam->divider,lpTunerParam->control));

    if ( lpTunerParam )
    {
        xfree(lpTunerParam);
    }
} /* SetTunerChannel ()... */

static int
viaSetPortAttributeG(
    ScrnInfoPtr pScrn,
    Atom attribute,
    INT32 value,
    pointer data
){
    VIAPtr  pVia = VIAPTR(pScrn);
    volatile video_via_regs  *viaVidEng = NULL;
    viaPortPrivPtr pPriv = (viaPortPrivPtr)data;
    LPVIAAUDCTRL lpAudCtrl = &(pPriv->AudCtrl);
    LPVIASETPORTATTR lpParam =  (LPVIASETPORTATTR)xalloc(sizeof(VIASETPORTATTR));
    PVIDDATA pVidData = pVia->pVidData;
    if(XV_SWOV_PORTID == pPriv->xv_portnum) pVidData->VMI_Info.handle=0;
    else if(XV_SWOV_PORTID2 == pPriv->xv_portnum) pVidData->VMI_Info.handle=1;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    struct video_channel chan;
    int fd = -1;
    viaVidEng = (volatile video_via_regs *)pVia->VidMapBase;

    DBG_DD((" via_video.c : viaSetPortAttributeG : \n"));
    DBG_DD((" via_video.c : Screen[%d]\n", pScrn->scrnIndex));

    switch (lpVIAGraphicInfo->dwProjectIDRevision)
    {
        case UNICHROME_VT3123AX:
            pVidData->dwRejectOverlay = RejectOverlay(pVidData);
            break;
        case UNICHROME_VT3123CX:
            pVidData->dwRejectOverlay = RejectOverlay_3123C0(pVidData);
            break;
        case UNICHROME_VT3204:
            pVidData->dwRejectOverlay = RejectOverlay_3204(pVidData);
            DBG_DD((" via_video.c : dwRejectOverlay returned value=%d : \n",pVidData->dwRejectOverlay));
            break;
              
        case UNICHROME_VT3205:
            pVidData->dwRejectOverlay = RejectOverlay_3205(pVidData);
            break;
        default:
            pVidData->dwRejectOverlay = FALSE;
            DBG_DD((" via_video.c : Don't need dwRejectOverlay()\n")); 
            break;           
    }

    /* Color Key */
    if(attribute == xvColorKey)
    {
        DBG_DD(("  V4L Disable  xvColorKey = %08x\n",value));

        if (pScrn->bitsPerPixel == 16)
        {
            value &= 0xFFFF;
        }
        else if (pScrn->bitsPerPixel == 8)
        {
            value &= 0xFF;
        }

        pPriv->colorKey = value;

        viaVidEng->color_key = value;
        viaVidEng->snd_color_key = value;
        REGION_EMPTY(pScrn->pScreen, &pPriv->clip);
    }
    /* Overlay Color Control */
    else if (attribute == xvContrast)
    {
        DBG_DD(("     xvContrast = %08d\n",value));
        pPriv->contrast = value;
        SetVideoColor(pVidData, pPriv->hue, pPriv->saturation, pPriv->brightness, pPriv->contrast);
    }
    else if (attribute == xvSaturation)
    {
        DBG_DD(("     xvSaturation = %08d\n",value));
        pPriv->saturation = value;
        SetVideoColor(pVidData, pPriv->hue, pPriv->saturation, pPriv->brightness, pPriv->contrast);
    }
    else if (attribute == xvHue)
    {
        DBG_DD(("     xvHue = %08d\n",value));
        pPriv->hue = value;
        SetVideoColor(pVidData, pPriv->hue, pPriv->saturation, pPriv->brightness, pPriv->contrast);
    }
    else if (attribute == xvBrightness)
    {
        DBG_DD(("     xvBrightness = %08d\n",value));
        pPriv->brightness = value;
        SetVideoColor(pVidData, pPriv->hue, pPriv->saturation, pPriv->brightness, pPriv->contrast);
    }
    /* Audio control */
    else if (attribute == xvMute)
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : xvMute = %08d\n",value));

        if ( value )
        {
            lpAudCtrl->dwAudioMode = ATTR_MUTE_ON;
            lpParam->attribute = ATTR_MUTE_ON;
        }
        else
        {
            lpAudCtrl->dwAudioMode = ATTR_MUTE_OFF;
            lpParam->attribute = ATTR_STEREO;
            lpParam->attribute = ATTR_MUTE_OFF;
        }
    }
    else if (attribute == xvVolume)
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : xvVolume = %08d\n",value));
        lpAudCtrl->nVolume = value;
        lpParam->attribute = ATTR_VOLUME;
        lpParam->value = value;
    }
    /* Tuner control. Channel switch */
    else if (attribute == xvFreq)
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : xvFreq = %08x\n",value));
        SetTunerChannel(pPriv, value );
    }
    /* Video decoder control. NTSC/PAL, SVIDEO/COMPOSITIVE/TV */
    else if (attribute == xvEncoding)
    {
        DBG_DD(("     xvEncoding = %d. \n",value));

        pPriv->dwEncoding = value;
        
        switch ( value )
        {
            case NTSC_COMPOSITE:
                chan.channel = 0;       /* Composite input */
                chan.norm    = VIDEO_MODE_NTSC;
                break;

            case NTSC_TUNER:
                chan.channel = 1;       /* Tuner input */
                chan.norm    = VIDEO_MODE_NTSC;
                break;

            case NTSC_SVIDEO:
                chan.channel = 2;       /* S-Video input */
                chan.norm    = VIDEO_MODE_NTSC;
                break;

            case PAL_SVIDEO:
                chan.channel = 2;       /* S-Video input */
                chan.norm    = VIDEO_MODE_PAL;
                break;

            case PAL_60_COMPOSITE:
                chan.channel = 0;       /* Composite input */
                chan.norm    = VIDEO_MODE_PAL;
                break;

            case PAL_60_TUNER:
                chan.channel = 1;       /* Tuner input */
                chan.norm    = VIDEO_MODE_PAL;
                break;

            case PAL_60_SVIDEO:
                chan.channel = 2;       /* S-Video input */
                chan.norm    = VIDEO_MODE_PAL;
                break;

            default:
                break;
        }
    }

    /* VIA Proprietary Attribute for AUDIO control */
    else if (attribute == xvAudioCtrl )
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : xvAudioSwitch=%d\n", value));

        lpParam->attribute = ATTR_AUDIO_CONTROLByAP;
        lpParam->value = value;
    }
    else if (attribute == xv_autopaint_colorkey )
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : xv_autopaint_colorkey=%d\n", value));
        pPriv->autopaint_colorkey = value;
    }
    /*=* for device switch hotkey utility *=*/
    else if (attribute == xvHotkeyTimer)
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : xvHotkeyTimer=%d\n", value));
        VIACheckLCDExpand(pScrn);
    }
    else if (attribute == xvDeviceControl)
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : xvDeviceControl=%d\n", value));
    }
    else if (attribute == xvTVStandard)
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : xvTVStandard=%d\n", value));
    }
    else if (attribute == xvDeviceDriverSync)
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : xvDeviceDriverSync=%d\n", value));
    }        
    else
    {
        DBG_DD((" via_video.c : viaSetPortAttributeG : is not supported the attribute\n"));
        return BadMatch;
    }

    if ( lpParam )
    {
        xfree(lpParam);
    }

    return Success;
}

static int
viaGetPortAttributeG(
    ScrnInfoPtr pScrn,
    Atom attribute,
    INT32 *value,
    pointer data
){
    VIAPtr  pVia = VIAPTR(pScrn);
    viaPortPrivPtr pPriv = (viaPortPrivPtr)data;
    PVIDDATA pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    unsigned char* pjVideo = pVidData->Vhwinfo.pjVideo;
    int fd = -1;

    DBG_DD((" via_video.c : viaGetPortAttributeG : port %d\n",pPriv->xv_portnum));
    DBG_DD((" via_video.c : Screen[%d]\n", pScrn->scrnIndex));

    lpVIAGraphicInfo->IsSupportedColorSpaceChange = TRUE;

    if (attribute == xvColorKey )
    {
        *value =(INT32) pPriv->colorKey;
        DBG_DD((" via_video.c :    ColorKey 0x%x\n",pPriv->colorKey));
    }
    /* Color Control */
    else if (attribute == xvContrast)
    {
        *value = pPriv->contrast;
        DBG_DD(("    xvContrast = %08d\n", *value));
    }
    else if (attribute == xvSaturation)
    {
        *value = pPriv->saturation;
        DBG_DD(("    xvSaturation = %08d\n", *value));
    }
    else if (attribute == xvHue)
    {
        *value = pPriv->hue;
        DBG_DD(("    xvHue = %08d\n", *value));
    }
    else if (attribute == xvBrightness)
    {
        *value = pPriv->brightness;
        DBG_DD(("    xvBrightness = %08d\n", *value));
    }
    else if(attribute == xvOverlayStatus)
    {
        if ( (VIDInD(V1_CONTROL) & V1_ENABLE) ||
             (VIDInD(V3_CONTROL) & V3_ENABLE) )

        {
            *value = 1;
            DBG_DD((" via_video.c : XV Overlay Enable.\n"));
        }
        else
        {
            *value = 0;
            DBG_DD((" via_video.c : XV Overlay Disable.\n"));
        }
    }
    /* autopaint colorkey */
    else if (attribute == xv_autopaint_colorkey)
    {
        *value = pPriv->autopaint_colorkey;
        DBG_DD((" via_video.c :   xv_autopaint_colorkey= %d\n",pPriv->autopaint_colorkey));
    }
    else if (attribute == xvMute)
    {
        DBG_DD((" via_video.c : viaGetPortAttributeG : xvMute = %08d\n",value));
    }
    else if (attribute == xvVolume)
    {
        DBG_DD((" via_video.c : viaGetPortAttributeG : xvVolume = %08d\n",value));
    }
    else if (attribute == xvFreq)
    {
        DBG_DD((" via_video.c : viaGetPortAttributeG : xvFreq = %08x\n",value));
    }
    else if (attribute == xvEncoding)
    {
        DBG_DD((" via_video.c : viaGetPortAttributeG : xvEncoding = %08x\n",value));
    }
    else if (attribute == xvHotkeyTimer)
    {
        *value = 0;
        DBG_DD((" via_video.c : viaGetPortAttributeG : xvHotkeyTimer = %08d\n", *value));
    }
    else if (attribute == xvDeviceControl)
    {
        *value = 0;
        DBG_DD((" via_video.c : viaGetPortAttributeG : xvDeviceControl = %08d\n", *value));
    }
    else if (attribute == xvTVStandard)
    {
        *value = 0;
        DBG_DD((" via_video.c : viaGetPortAttributeG : xvTVStandard = %08d\n", *value));
    }
    else if (attribute == xvDeviceDriverSync)
    {
        *value = 0;
        DBG_DD((" via_video.c : viaGetPortAttributeG : xvDeviceDriverSync = %08d\n", *value));
    }        
    else
    {
        DBG_DD((" via_video.c : viaGetPortAttributeG : is not supported the attribute\n"));
        return BadMatch;
    }    

    return Success;
}


static void
viaQueryBestSizeG(
    ScrnInfoPtr pScrn,
    Bool motion,
    short vid_w, short vid_h,
    short drw_w, short drw_h,
    unsigned int *p_w, unsigned int *p_h,
    pointer data
){
    DBG_DD((" via_video.c : viaQueryBestSizeG :\n"));
    *p_w = drw_w;
    *p_h = drw_h;

    if(*p_w > 2048 )
    {
        *p_w = 2048;
    }
}



/*
 *  For FourCC : YV12 case
 */
static void CopyDataYUV420(unsigned char *src1, unsigned char *src2, unsigned char *src3,
    unsigned char *dst1, unsigned char *dst2,  unsigned char *dst3,
    int srcPitch,  int dstPitch, int h, int w)
{
    int count;

    /* copy Y component to video memory */
    count = h;

    while(count--)
    {
        memcpy(dst1, src1, w);
        src1 += srcPitch;
        dst1 += dstPitch;
    }

    /* UV component is 1/4 of Y */
    w >>= 1;
    h >>= 1;
    srcPitch >>= 1;
    dstPitch >>= 1;

    /* copy V(Cr) component to video memory */
    count = h;

    while(count--)
    {
        memcpy(dst2, src2, w);
        src2 += srcPitch;
        dst2 += dstPitch;
    }

    /* copy U(Cb) component to video memory */
    count = h;

    while(count--)
    {
        memcpy(dst3, src3, w);
        src3 += srcPitch;
        dst3 += dstPitch;
    }
}

/*
 *  For FourCC : YUY2 case
 */
static void CopyDataYUV422(
    unsigned char * src,
    unsigned char * dst,
    int srcPitch,
    int dstPitch,
    int h,
    int w )
{
    int count;

    DBG_DD((" via_video.c : CopyDataYUV422() \n"));

    /*  copy YUY2 data to video memory,
     *  do 32 bits alignment.
     */
    count = h;

    while(count--)
    {
        memcpy(dst, src, w);
        src += srcPitch;
        dst += dstPitch;
    }
}


/*  CopyYUV420To422 :
 *  for VT3259 use,
 *  copy image data to frame buffer & transfer
 *  format from YUV420 to YUV422, coz VT3259
 *  HQV input source only support FourCC NV12, YUY2
 *  YUV420 format in memory is Y, V, U 3 planar mode
 *  YUV422 is Y0 U0 Y1 V0
 *            low ....High byte
 */
static void CopyYUV420To422(
    unsigned char* src1,    /* Y */
    unsigned char* src2,    /* V */
    unsigned char* src3,    /* U */
    unsigned char* dst1,
    int srcPitch,
    int srcPitch2,
    int dstPitch,
    int h,
    int w )
{
    CARD32 *dst = (CARD32*)dst1;
    int i, j;

    DBG_DD((" via_video.c : CopyYUV420To422() \n"));

    dstPitch >>= 2;
    w >>= 1;

    for(j = 0; j < h; j++)
    {
        for(i = 0; i < w; i++) 
        {
            dst[i] = src1[i << 1] | (src1[(i << 1) + 1] << 16) |
                      (src3[i] << 8) | (src2[i] << 24);
        }
        
        dst += dstPitch;
        src1 += srcPitch;

        if(j & 1)
        {
            src2 += srcPitch2;
            src3 += srcPitch2;
        }
    }
}

/*  CopyI420To422 :
 *  for VT3259 use,
 *  copy image data to frame buffer & transfer
 *  format from I420 to YUY2
 *  I420 format in memory is Y, U, V 3 planar mode
 *  YUV422 is Y0 U0 Y1 V0
 *            low ....High byte
 */
static void CopyI420To422(
    unsigned char* src1,    /* Y */
    unsigned char* src2,    /* U */
    unsigned char* src3,    /* V */
    unsigned char* dst1,
    int srcPitch,
    int srcPitch2,
    int dstPitch,
    int h,
    int w )
{
    CARD32 *dst = (CARD32*)dst1;
    int i, j;

    DBG_DD((" via_video.c : CopyI420To422() \n"));

    dstPitch >>= 2;
    w >>= 1;

    for(j = 0; j < h; j++)
    {
        for(i = 0; i < w; i++)
        {
            dst[i] = src1[i << 1] | (src1[(i << 1) + 1] << 16) |
                     (src2[i] << 8) | (src3[i] << 24);
        }
        
        dst += dstPitch;
        src1 += srcPitch;

        if(j & 1) 
        {
            src2 += srcPitch2;
            src3 += srcPitch2;
        }
    }
}

/*
 *  For FourCC : NV12 case  
 */
static void CopyYUV420ToNV12(unsigned char *src1, unsigned char *src2, unsigned char *src3,
                       unsigned char *desty,  unsigned char *destuv,
                       int srcPitch,  int dstPitch, int h, int w)
{
    int i,j,k;

    /* copy Y component to video memory */
    if (srcPitch==dstPitch)
    {
        memcpy(desty, src1, dstPitch*h);
    }
    else
    {
        for (i=0;i<h;i++)
        {
            memcpy(desty, src1, w);
            src1 += srcPitch;
            desty += dstPitch;
        }
    }

    h>>=1;   

    for(j = 0; j < h; j++)
    {
        for(i = 0,k=0; i <w; i+=4,k+=2)
        {
            *((unsigned long *)(destuv+i)) = src3[k] |(src2[k] <<8) | (src3[k+1] <<16)|(src2[k+1]<<24);
        }

        destuv += dstPitch;
        src2 += srcPitch>>1;
        src3 += srcPitch>>1;        
    }
}

/*===========================================================
  *     Ceat OV surface 
  *     author : VIA UniChrome
  *     data : 2004/09/07
  *===========================================================*/
__inline unsigned long  
VIACreatOVSurface (
    ScrnInfoPtr pScrn,
    viaPortPrivPtr pPriv,
    int id,
    short src_w, short src_h, 
    short width, short height
){
    VIAPtr  pVia = VIAPTR(pScrn);
    unsigned long handle = pVia->pVidData->VMI_Info.handle;
    LPVIDEOSTATUS lpVidStatus = &(pVia->pVidData->VidStatus);

    DBG_DD((" via_video.c : VIACreatOVSurface() \n"));

    /*  Allocate video memory(CreateSurface),
     *  add codes to judge if need to re-create surface
     */
    if ((pPriv->old_src_w != src_w) || (pPriv->old_src_h != src_h)) 
    {
        DestroyOVSurface(pScrn, pPriv, pPriv->fd);
    }
    else
    {
        DBG_DD(("// No need to destroy surface \n"));
    }

    if (!(lpVidStatus->dwVideoFlagSW[handle]))
    {
        if (CreateOVSurface(pScrn, pPriv, pPriv->fd, id, width, height)!=TRUE)
            return FALSE;
    }
    else
    {
        DBG_DD(("// No need to create surface \n"));
    }
    return TRUE;
}   

/*===========================================================
  *     OV Flip via V4L or not
  *===========================================================*/
__inline int 
VIAOVFlip (
    ScrnInfoPtr pScrn,
    viaPortPrivPtr pPriv
){
    VIAPtr  pVia = VIAPTR(pScrn);
    PVIDDATA pVidData = pVia->pVidData;

    unsigned long dwStartAddr = 0;   /* for startaddr select */
    unsigned long dwDisplayIndex;
    FLIPHANDLE fliphandle;
    dwStartAddr = pVidData->SWDevice[pVidData->VMI_Info.handle].dwSWPhysicalAddr[dwFrameNum[pVidData->VMI_Info.handle]&1];
    DBG_DD(("          : dwStartAddr: %x\n", dwStartAddr));

    fpFlip(pVidData, dwFrameNum[pVidData->VMI_Info.handle]&1);

    dwFrameNum[pVidData->VMI_Info.handle] ++;

    return Success;
}

/*===========================================================
  *     OV update overlay via V4L or not
  *===========================================================*/
 __inline int 
 VIAOVUpdateOverlay (
    ScrnInfoPtr pScrn,
    viaPortPrivPtr pPriv,
    LPDDUPDATEOVERLAY    lpUpdateOverlay,
    short drw_x, short drw_y,
    short src_w, short src_h,
    short drw_w, short drw_h
){
    VIAPtr  pVia = VIAPTR(pScrn);
    PVIDDATA pVidData = pVia->pVidData;


    pPriv->old_drw_x = drw_x;
    pPriv->old_drw_y = drw_y;
    pPriv->old_drw_w = drw_w;
    pPriv->old_drw_h = drw_h;
    
    /* add to judge if need to re-create surface */
    pPriv->old_src_w = src_w;
    pPriv->old_src_h = src_h;

    if(XV_SWOV_PORTID == pPriv->xv_portnum) pVidData->VMI_Info.handle=0;
    else if(XV_SWOV_PORTID2 == pPriv->xv_portnum) pVidData->VMI_Info.handle=1;

    /* update overlay */
    lpUpdateOverlay->dwFourcc = pPriv->SurfaceDesc.dwFourCC;

    if (UpdateOverlay(pVidData, lpUpdateOverlay) == -1)
    {
        DBG_DD((" via_video.c : : call updateoverlay fail. \n"));
        return XvBadAlloc;
    }    

    return Success;
}

/*===========================================================
  *     draw color key to on screen region
  *===========================================================*/
#ifdef NEW_XV_INTERFACE
 __inline void
VIADrawColorKeyRegion(
    ScrnInfoPtr pScrn,
    viaPortPrivPtr pPriv,
    RegionPtr clipBoxes,DrawablePtr pDraw
){
#else
 __inline void
VIADrawColorKeyRegion(
    ScrnInfoPtr pScrn,
    viaPortPrivPtr pPriv,
    RegionPtr clipBoxes
){
#endif
    VIAPtr  pVia = VIAPTR(pScrn);
    PVIDDATA pVidData = pVia->pVidData;
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    int i;
    BoxPtr pbox;

    DBG_DD((" via_video.c : VIADrawColorKeyRegion() \n"));


    if (via_xf86_version_current < XF86_VERSION_NUMERIC(4,3,99,3,0))
    {
        if ((!RegionsEqual(&pPriv->clip, clipBoxes) && pPriv->autopaint_colorkey) ||
            (lpVIAGraphicInfo->ReDrawColorkey))
        {
            DBG_DD((" // Draw Colorkey!! \n"));
            REGION_COPY(pScrn->pScreen, &pPriv->clip, clipBoxes);
            
            if (via_xf86_version_current < XF86_VERSION_NUMERIC(4,1,99,1,0))
            {
                pVia->AccelInfoRec->SetupForSolidFill(pScrn,pPriv->colorKey,GXcopy,~0);
                pbox=REGION_RECTS(clipBoxes);
                
                for(i=REGION_NUM_RECTS(clipBoxes);i;i--,pbox++) 
                {
                    pVia->AccelInfoRec->SubsequentSolidFillRect(pScrn,pbox->x1,pbox->y1,
                        pbox->x2-pbox->x1,pbox->y2-pbox->y1);
                }
            }
            else
            {
            
                xf86XVFillKeyHelper(pScrn->pScreen, pPriv->colorKey, clipBoxes);
            }
        }
        else 
        {
            DBG_DD((" // No need to draw Colorkey!! \n"));
        }
    }        
    else
    {
        if ((!REGION_EQUAL(pScrn->pScreen, &pPriv->clip, clipBoxes) && pPriv->autopaint_colorkey) ||
            (lpVIAGraphicInfo->ReDrawColorkey))
        {
            DBG_DD((" // Draw Colorkey!! \n"));
            REGION_COPY(pScrn->pScreen, &pPriv->clip, clipBoxes);    
            if (via_xf86_version_current < XF86_VERSION_NUMERIC(7,3,0,0,0))			
                xf86XVFillKeyHelper(pScrn->pScreen, pPriv->colorKey, clipBoxes);
            else
#ifdef NEW_XV_INTERFACE						
                xf86XVFillKeyHelperDrawable(pDraw, pPriv->colorKey, clipBoxes);				
#else
                xf86XVFillKeyHelper(pScrn->pScreen, pPriv->colorKey, clipBoxes);
#endif
        }
        else
        {
            DBG_DD((" // No need to draw Colorkey!! \n"));
        }
    }
}

/*===========================================================*/
#ifdef NEW_XV_INTERFACE
static int
viaPutImageG(
    ScrnInfoPtr pScrn,
    short src_x, short src_y,
    short drw_x, short drw_y,
    short src_w, short src_h,
    short drw_w, short drw_h,
    int id, unsigned char* buf,
    short width, short height,
    Bool sync,
    RegionPtr clipBoxes, pointer data,DrawablePtr pDraw
){
#else
static int
viaPutImageG(
    ScrnInfoPtr pScrn,
    short src_x, short src_y,
    short drw_x, short drw_y,
    short src_w, short src_h,
    short drw_w, short drw_h,
    int id, unsigned char* buf,
    short width, short height,
    Bool sync,
    RegionPtr clipBoxes, pointer data
){
#endif
    VIAPtr  pVia = VIAPTR(pScrn);
    viaPortPrivPtr pPriv = (viaPortPrivPtr)data;
    PVIDDATA pVidData = pVia->pVidData;
    if(XV_SWOV_PORTID == pPriv->xv_portnum) pVidData->VMI_Info.handle=0;
    else if(XV_SWOV_PORTID2 == pPriv->xv_portnum) pVidData->VMI_Info.handle=1;
    unsigned long handle = pVidData->VMI_Info.handle;
    LPVIDEOSTATUS lpVidStatus = &(pVidData->VidStatus);
    LPVIAGRAPHICINFO lpVIAGraphicInfo = &pVidData->VIAGraphicInfo;
    LPSWDEVICE  lpSWDevice  = &(pVidData->SWDevice[handle]);
    LPDDSURFACEDESC lpSurfaceDesc = &pPriv->SurfaceDesc;
    BoxPtr pbox = REGION_RECTS(clipBoxes);
    int i, nbox = REGION_NUM_RECTS(clipBoxes);

    if ((lpVIAGraphicInfo->rotate == 1) || (lpVIAGraphicInfo->rotate == -1))
    {
        if ((drw_w == lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwWidth) ||
            (drw_h == lpVIAGraphicInfo->Screen[pScrn->scrnIndex].dwHeight))
        {            
            for(i = 0; i < nbox; i++, pbox++)
            {
                pbox->x1 = drw_x;
                pbox->y1 = drw_y;
                pbox->x2 = drw_h + 1;
                pbox->y2 = drw_w;
            }
        }
    }    
    
    if((lpVIAGraphicInfo->Screen1IsLeft)&&(pScrn->scrnIndex==0))
    {
        drw_x=lpVIAGraphicInfo->Screen[1-pScrn->scrnIndex].dwWidth+drw_x;
    }

    if((lpVIAGraphicInfo->Screen1IsAbove)&&(pScrn->scrnIndex==0))
    {
        drw_y=lpVIAGraphicInfo->Screen[1-pScrn->scrnIndex].dwHeight+drw_y;
    }       

    DBG_DD((" via_video.c : viaPutImageG : called\n"));
    DBG_DD((" via_video.c : Screen[%d]\n", pScrn->scrnIndex));
    DBG_DD((" via_video.c : FourCC=0x%x width=%d height=%d sync=%d\n",id,width,height,sync));
    DBG_DD((" via_video.c : src_x=%d src_y=%d src_w=%d src_h=%d colorkey=0x%x\n",src_x, src_y, src_w, src_h, pPriv->colorKey));
    DBG_DD((" via_video.c : drw_x=%d drw_y=%d drw_w=%d drw_h=%d\n",drw_x,drw_y,drw_w,drw_h));

    /*Added for SAMM Mode*/
    if (pScrn->scrnIndex == 0)
    {
        pVidData->bVideoOnSND = FALSE;DBG_DD(("pVidData->bVideoOnSND=false\n"));
    }
    else
    {
        pVidData->bVideoOnSND = TRUE;DBG_DD(("pVidData->bVideoOnSND=true\n"));
    }

    switch ( pPriv->xv_portnum )
    {
        case COMMAND_FOR_SWOV:
        case XV_SWOV_PORTID2: 
        {

            int srcPitch, dstPitch;
            int srcYSize, srcUVSize;
            int dstYSize, dstUVSize;
            unsigned long dwUseExtendedFIFO=0;

            /* If there is bandwidth issue, block the H/W overlay */
            if (pVidData->dwRejectOverlay)
            {
                DBG_DD((" via_video.c : BadAlloc!\n"));
                return BadAlloc;
            }


            /* creat surface : */
            if (VIACreatOVSurface(pScrn, pPriv, id, src_w, src_h, width, height)!=TRUE)
            {
                DBG_DD(("VIACreateOVSurface memory allocation failure!\n"));
                StopOVerlay(pScrn, pPriv->fd);
                DestroyOVSurface(pScrn, pPriv, pPriv->fd);
                return BadAlloc;
             } 
            /*  Copy image data from system memory to video memory
             *  TODO: use DRM's DMA feature to accelerate data copy
             */
            srcPitch = width;
            srcYSize  = width * height;
            srcUVSize = srcYSize >>2;
            dstPitch = lpSWDevice->dwPitch;
            dstYSize  = dstPitch * height;
            dstUVSize = dstYSize >>2;

            switch(id)
            {
                case FOURCC_YV12:
                    /*  Coz in CreateSurface, we unify fourcc to YUY2,
                     *  so we use CopyYUV420To422 instead of CopyDataYUV420.
                     */
                    if (lpSurfaceDesc->dwFourCC == FOURCC_YUY2)
                    {
                        CopyYUV420To422( buf, buf + srcYSize, buf + srcYSize + srcUVSize,
                            lpSWDevice->lpSWOverlaySurface[dwFrameNum[handle]&1],
                            srcPitch, srcPitch>>1, dstPitch, height, width);
                    }
                    else if (lpSurfaceDesc->dwFourCC == FOURCC_YV12)
                    {
                        CopyDataYUV420(buf, buf + srcYSize, buf + srcYSize + srcUVSize,
                            lpSWDevice->lpSWOverlaySurface[dwFrameNum[handle]&1], 
                            lpSWDevice->lpSWOverlaySurface[dwFrameNum[handle]&1]+ dstYSize,  
                            lpSWDevice->lpSWOverlaySurface[dwFrameNum[handle]&1]+ dstYSize + dstUVSize,
                            srcPitch,  dstPitch, height, width);
                    }
                    else if (lpSurfaceDesc->dwFourCC == FOURCC_NV12)
                    {
                        CopyYUV420ToNV12(buf, buf + srcYSize, buf + srcYSize + srcUVSize,
                            lpSWDevice->lpSWOverlaySurface[dwFrameNum[handle]&1], 
                            lpSWDevice->lpSWOverlaySurface[dwFrameNum[handle]&1]+ dstYSize,  
                            srcPitch,  dstPitch, height, width);
                    }
                    
                    break;
                case FOURCC_I420:
                    CopyI420To422( buf, buf + srcYSize, buf + srcYSize + srcUVSize,
                        lpSWDevice->lpSWOverlaySurface[dwFrameNum[handle]&1],
                        srcPitch, srcPitch>>1, dstPitch, height, width);
                    break;
                    
                case FOURCC_UYVY:
                case FOURCC_YUY2:
                default:
                    CopyDataYUV422(buf, lpSWDevice->lpSWOverlaySurface[dwFrameNum[handle]&1],
                        srcPitch<<1, dstPitch, height, width<<1);
                    break;
            }

           

            /* If use extend FIFO mode */
            if ( (lpVIAGraphicInfo->Screen[0].dwWidth > 1024) || (lpVIAGraphicInfo->SAMM) )
            {
                dwUseExtendedFIFO = 1;
            }

            /*  BitBlt: Draw the colorkey rectangle */
#ifdef NEW_XV_INTERFACE				
            VIADrawColorKeyRegion(pScrn, pPriv, clipBoxes,pDraw);
#else			
            VIADrawColorKeyRegion(pScrn, pPriv, clipBoxes);
#endif
            /* UpdateOverlay : If the dest rec. & extendFIFO doesn't change, don't do UpdateOverlay */
            if ((pPriv->old_drw_x == drw_x) && 
                (pPriv->old_drw_y == drw_y) && 
                (pPriv->old_drw_w == drw_w) && 
                (pPriv->old_drw_h == drw_h) && 
                (pPriv->old_src_w == src_w) && 
                (pPriv->old_src_h == src_h) && 
                (pPriv->old_dwUseExtendedFIFO == dwUseExtendedFIFO) && 
                (lpVidStatus->dwVideoFlagSW[handle] & VIDEO_ON))
            {
                DBG_DD(("If the dest rec. & extendFIFO doesn't change, don't do UpdateOverlay\n"));
            }
            else 
            {
                LPDDUPDATEOVERLAY    lpUpdateOverlay = &pVidData->UpdateOverlayRec;
                /*
                *  fill video overlay parameter
                */
                lpUpdateOverlay->rSrc.left = src_x;
                lpUpdateOverlay->rSrc.top = src_y;
                lpUpdateOverlay->rSrc.right = src_x + width;
                lpUpdateOverlay->rSrc.bottom = src_y + height;

                lpUpdateOverlay->rDest.left = drw_x;
                lpUpdateOverlay->rDest.top = drw_y;
                lpUpdateOverlay->rDest.right = lpUpdateOverlay->rDest.left + drw_w;
                lpUpdateOverlay->rDest.bottom = drw_y + drw_h;

                lpUpdateOverlay->dwFlags = DDOVER_SHOW | DDOVER_KEYDEST;

                /*Added for SAMM Mode*/
                if ((pScrn->scrnIndex==1) || 
                    ((lpVIAGraphicInfo->duoview&0x01) && (lpVIAGraphicInfo->duoview&0x02))) 
                {
                    lpUpdateOverlay->dwFlags |= DDOVER_ON_SND;
                }

                if (pScrn->bitsPerPixel == 8)
                {
                    lpUpdateOverlay->dwColorSpaceLowValue = pPriv->colorKey & 0xff;
                }
                else
                {
                    lpUpdateOverlay->dwColorSpaceLowValue = pPriv->colorKey;
                }
                pPriv->old_dwUseExtendedFIFO = dwUseExtendedFIFO;
                
                if (VIAOVUpdateOverlay( pScrn, pPriv, lpUpdateOverlay,
                                        drw_x, drw_y,
                                        src_w, src_h, 
                                        drw_w, drw_h ) != Success)
                {
                    return XvBadAlloc;
                }
            }


            /* Flip */
            if (VIAOVFlip(pScrn, pPriv) != Success) 
            {
                return XvBadAlloc;
            }          
        }

        break;
        
        default:
            DBG_DD((" via_video.c : XVPort not supported\n"));
            break;
    }

    return Success;
}

#ifdef NEW_XV_INTERFACE
static int 
viaReputImageG( ScrnInfoPtr pScrn, short drw_x, short drw_y,
        RegionPtr clipBoxes, pointer data, DrawablePtr pDraw )
#else
static int 
viaReputImageG( ScrnInfoPtr pScrn, short drw_x, short drw_y,
        RegionPtr clipBoxes, pointer data )
#endif
{

    viaPortPrivPtr pPriv = (viaPortPrivPtr)data;

    DBG_DD((" via_video.c : viaReputImageG\n"));

    /* 1, it need to re-draw colorkey region when clip */   
    /* 2, clean the data staruct clip record */     
    REGION_EMPTY(pScrn->pScreen, &pPriv->clip);

    return Success;
}

static int
viaQueryImageAttributesG(
    ScrnInfoPtr pScrn,
    int id,
    unsigned short *w, unsigned short *h,
    int *pitches, int *offsets
){
    int size, tmp;

    DBG_DD((" via_video.c : viaQueryImageAttributesG : FourCC=0x%x\n", id));
    DBG_DD((" via_video.c : Screen[%d]\n", pScrn->scrnIndex));

    if ( (!w) || (!h) )
        return 0;

    if(*w > MAXWIDTH)
    {
        *w = MAXWIDTH;
    }
        
    if(*h > MAXHEIGHT)
    {
        *h = MAXHEIGHT;
    }

    *w = (*w + 1) & ~1;
    if(offsets)
    {
        offsets[0] = 0;
    }

    switch(id)
    {
        case FOURCC_YV12:  /*Planar format : YV12 -4:2:0*/
        case FOURCC_I420:
            *h = (*h + 1) & ~1;
            size = *w;

            if(pitches)
            {
                pitches[0] = size;
            }
            
            size *= *h;
            
            if(offsets)
            {
                offsets[1] = size;
            }
            
            tmp = (*w >> 1);
            
            if(pitches)
            {
                pitches[1] = pitches[2] = tmp;
            }
            
            tmp *= (*h >> 1);
            size += tmp;
            
            if(offsets)
            {
                offsets[2] = size;
            }
            
            size += tmp;
            break;
            
        case FOURCC_AI44:
        case FOURCC_IA44:
            size = *w * *h;
            if (pitches)
            {
                pitches[0] = *w;
            }
            
            if (offsets)
            {
                offsets[0] = 0;
            }
            
            break;
            
        case FOURCC_UYVY:  /*Packed format : UYVY -4:2:2*/
        case FOURCC_YUY2:  /*Packed format : YUY2 -4:2:2*/
        default:
            size = (*w << 1);
            
            if(pitches)
            {
                pitches[0] = size;
            }
            size *= *h;
            break;
    }

    if ( pitches )
    {
        DBG_DD((" pitches[0]=%d, pitches[1]=%d, pitches[2]=%d, ", pitches[0], pitches[1], pitches[2]));
    }

    if ( offsets )
    {
        DBG_DD((" offsets[0]=%d, offsets[1]=%d, offsets[2]=%d, ", offsets[0], offsets[1], offsets[2]));
    }

    DBG_DD((" width=%d, height=%d \n", *w, *h));

    return size;
}

#ifdef NEW_XV_INTERFACE
static int
viaPutVideo(ScrnInfoPtr pScrn,
    short src_x, short src_y, short drw_x, short drw_y,
    short src_w, short src_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data, DrawablePtr pDraw)
#else
static int
viaPutVideo(ScrnInfoPtr pScrn,
    short src_x, short src_y, short drw_x, short drw_y,
    short src_w, short src_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data)
#endif    
{

    VIAPtr  pVia = VIAPTR(pScrn);
    viaPortPrivPtr pPriv=(viaPortPrivPtr)data;
    int fd = -1;

    DBG_DD((" via_video.c : viaPutVideo : Src %dx%d, %d, %d, %p\n",src_w,src_h,src_x,src_y,clipBoxes));
    DBG_DD((" via_video.c : Dst %dx%d, %d, %d \n",drw_w,drw_h,drw_x,drw_y));
    DBG_DD((" via_video.c : colorkey : 0x%x \n",pPriv->colorKey));
    DBG_DD((" via_video.c : clipBoxes : x1 %d, x2 %d, y1 %d, y2 %d\n",clipBoxes->extents.x1,clipBoxes->extents.x2,clipBoxes->extents.y1,clipBoxes->extents.y2));
    DBG_DD((" via_video.c : old_clip : x1 %d, x2 %d, y1 %d, y2 %d\n",pPriv->clip.extents.x1,pPriv->clip.extents.x2,pPriv->clip.extents.y1,pPriv->clip.extents.y2));

    switch ( pPriv->xv_portnum )
    {
        case COMMAND_FOR_SWOV:
            /* If the dest rec. & extendFIFO doesn't change, don't do UpdateOverlay 
               unless the surface clipping has changed */
            if ((pPriv->old_drw_x == drw_x) && 
                (pPriv->old_drw_y == drw_y) && 
                (pPriv->old_drw_w == drw_w) && 
                (pPriv->old_drw_h == drw_h) &&
                (pPriv->old_src_w == src_w) && 
                (pPriv->old_src_h == src_h) && 
                RegionsEqual(&pPriv->clip, clipBoxes))
            {
                return Success;
            }

            pPriv->old_src_w = src_w;
            pPriv->old_src_h = src_h;
    
            pPriv->old_drw_x = drw_x;
            pPriv->old_drw_y = drw_y;
            pPriv->old_drw_w = drw_w;
            pPriv->old_drw_h = drw_h;
    
            /*  BitBlt: Draw the colorkey rectangle */
            if(!RegionsEqual(&pPriv->clip, clipBoxes)) 
            {
                REGION_COPY(pScrn->pScreen, &pPriv->clip, clipBoxes);    
                
                if (pPriv->autopaint_colorkey)
                {
                    xf86XVFillKeyHelper(pScrn->pScreen, pPriv->colorKey, clipBoxes); 
                }
            } 
        default:
            DBG_DD((" via_video.c : Error port access.\n"));
            return XvBadAlloc;
    }

    return Success;
}

#ifdef NEW_XV_INTERFACE
static int
viaPutStill(ScrnInfoPtr pScrn,
    short vid_x, short vid_y, short drw_x, short drw_y,
    short vid_w, short vid_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data, DrawablePtr pDraw )
#else
static int
viaPutStill(ScrnInfoPtr pScrn,
    short vid_x, short vid_y, short drw_x, short drw_y,
    short vid_w, short vid_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data)
#endif
{
    DBG_DD((" via_video.c : viaPutStill :\n"));

    return Success;
}

#ifdef NEW_XV_INTERFACE
static int
viaGetVideo(ScrnInfoPtr pScrn,
    short vid_x, short vid_y, short drw_x, short drw_y,
    short vid_w, short vid_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data, DrawablePtr pDraw)
#else
static int
viaGetVideo(ScrnInfoPtr pScrn,
    short vid_x, short vid_y, short drw_x, short drw_y,
    short vid_w, short vid_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data)
#endif    
{
    DBG_DD((" via_video.c : viaGetVideo :\n"));

    return Success;
}

#ifdef NEW_XV_INTERFACE
static int
viaGetStill(ScrnInfoPtr pScrn,
    short vid_x, short vid_y, short drw_x, short drw_y,
    short vid_w, short vid_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data, DrawablePtr pDraw)
#else
static int
viaGetStill(ScrnInfoPtr pScrn,
    short vid_x, short vid_y, short drw_x, short drw_y,
    short vid_w, short vid_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data)
#endif    
{

    DBG_DD((" via_video.c : viaGetStill :\n"));

    return Success;
}

#endif  /* !XvExtension */
